<?php
require_once(dirname(__FILE__).'/library/sofortLib.php');
/**
 * Bootstrap for sofort
 * 
 * $Date: 2012-11-30 14:18:05 +0100 (Fri, 30 Nov 2012) $
 * @version sofort 1.0  $Id: Bootstrap.php 5812 2012-11-30 13:18:05Z dehn $
 * @author SOFORT AG http://www.sofort.com (f.dehn@sofort.com)
 * @package Shopware 3.5.x, sofort.com
 *
 */
class Shopware_Plugins_Frontend_Sofort_Bootstrap extends Shopware_Components_Plugin_Bootstrap {
		
	private $snippets = null;
	
	private $configGroup = 'SOFORT Gateway';
	
	private $sofortConfigId = 0;
	
	private $statuses = array();
	
	private $statusSettings = array();
	
	private $statusDescriptions = array();
	
	/**
	 * Contains generated ID
	 * @var array
	 */
	private $initialStates = array();
	
	/**
	 * Products of Payment Network
	 * consisting of name, description, action, template and additionaldescription
	 * @var Array
	 */
	private $products = array();
	
	
	/**
	 * 
	 * Constructor for bootstrap of module
	 * @param Enlight_Plugin_Namespace $namespace
	 * @param string $name
	 */
	public function __construct(Enlight_Plugin_Namespace $namespace, $name) {
		$sql = 'SELECT count(*) FROM `s_core_snippets` WHERE namespace LIKE "sofort_multipay%"';
		$snippetCount = Shopware()->Db()->FetchOne($sql);	
		$lang = '';
		
		if($snippetCount == 0) {
			$lang = $this->insertGermanLanguage();
		}
		
		$this->snippets = Shopware()->Snippets();
		$this->initProducts();
		parent::__construct($namespace, $name);
	}
	
	
	/**
	 * Common install routine
	 * uninstall is called first to make sure everything's fine before installing
	 */
	public function install() {
		$this->uninstall();
		$this->statuses = $this->fetchStatuses();
		$this->statusSettings = $this->fetchStatusSettings();
		$this->statusDescriptions = $this->fetchStatusDescriptions();
		$this->setUeberweisungBack();
		$this->createPayments();
		$this->createLanguages();
		$this->createEvents();
		$this->createHooks();
		$this->createOrdersTable();
		$this->createProductTable();
		$this->createStatusTable();
		$this->createTemporaryOrdersTable();
		$this->createSettingsTable();
		$this->createCoreConfigTable();
		$this->createCoreStates();
		$this->createStatuses();
		$this->createBackendMenu();
		// $this->activateSofortPlugin();
		return true;
	}
	
	
	/**
	 * Common uninstall routine
	 */
	public function uninstall() {
		$this->cleanUp();
		$this->statusDescriptions = $this->fetchStatusDescriptions();
		$this->removePaymentMeans();
		$this->unsubscribeEvents();
		$this->unsubscribeHooks();
		$this->deleteMenuItems();
		//$this->removeOrdersTable(); //must not be removed in case of an update!
		//$this->removeStatusTable();
		//$this->removeProductTable();
		//$this->removeSettingsTable();
		$this->removeCoreConfigTable();
		$this->removeCoreStates();
		$this->removeConfigOptions();
		$this->removeLanguageSnippets();
		return true;
	}
	
	
	/**
	* Activate Plugin after installation
	*/
	protected function activateSofortPlugin() {
		try {
			$sofortPluginId = $this->getSofortPluginId();
			$sql = "UPDATE s_core_plugins SET active= 1 WHERE id = ?";
			Shopware()->Db()->query($sql, array($sofortPluginId));
		}
		catch (Exception $e) {
			throw new Exception('<b>Error during activation (activatePlugin)</b><br />' . $e);
		}
	}
	
	
	/**
	 * 
	 * Getter for sofort plugin id
	 */
	private function getSofortPluginId() {
		$sql = "SELECT id FROM s_core_plugins WHERE name LIKE 'Sofort'";
		$id = Shopware()->Db()->fetchOne($sql);
		return $id;
	}
	
	
	/**
	 * 
	 * Cleanup database
	 */
	private function cleanUp() {
		//clean up deprecated tables
		$sql = 'SELECT id FROM s_core_config_groups WHERE name = "sofort multipay"';
		$configGroup = Shopware()->Db()->fetchOne($sql);
		if($configGroup > 0) {
			$sql = 'DELETE FROM s_core_config WHERE `group` = "'.$configGroup.'" ';
			Shopware()->Db()->query($sql);
			$sql = 'DELETE FROM s_core_config_groups WHERE name = "sofort multipay"';
			Shopware()->Db()->query($sql);
		}
		
	}
	
	
	/**
	 * initiate product array
	 */
	private function initProducts() {
		
		$sueLandingUrl = $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_su_landing_url');
		$sueBanner = '';
		$sueLogo = $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_su_title_img');
		
		$svLandingUrl = $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sv_landing_url');
		$svBanner = $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sv_banner_img');
		$svLogo = $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sv_title_img');

		$srLandingUrl = $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sr_landing_url');
		$srBanner = $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sr_banner_img');
		$srLogo = $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sr_title_img');

		$slLandingUrl = $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sl_landing_url');
		$slBanner = $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sl_banner_img');
		$slLogo = $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sl_title_img');

		$lsLandingUrl = $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_ls_landing_url');
		$lsBanner = $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_ls_banner_img');
		$lsLogo = $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_ls_title_img');
		
		$idealLandingUrl = $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_ideal_landing_url');
		$idealBanner = $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_ideal_banner_image');
		
		$this->products = array(
			array(
				'name' => 'sofortueberweisung_multipay',
				'description' => $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_su_public_title'),
				'action' => 'sofort',
				'esdactive' => 1,
				'template' => '/Templates/Frontend/payment_methods/sofortueberberweisung.tpl',
				'additionaldescription' => '
					<a href="'.$sueLandingUrl.'" target="_blank">
						'.$sueBanner.'
					</a>
				',
				'logo' => '
					<a style="padding-right: 2px" href="'.$sueLandingUrl.'" target="_blank">
						'.$sueLogo.'
					</a>
				',
			),
			array(
				'name' => 'vorkassebysofort_multipay',
				'description' => $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sv_public_title'),
				'action' => 'sofort',
				'esdactive' => 0,
				'template' => 'Templates/Frontend/payment_methods/vorkassebysofort.tpl',
				'additionaldescription' => $this->snippets->getSnippet("sofort_multipay_bootstrap")->get("sofort_multipay_sv_public_title"),
				'logo' => '
					'.$svLogo.'
				',
			),
			array(
				'name' => 'sofortrechnung_multipay',
				'description' => $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sr_public_title'),
				'action' => 'sofort',
				'esdactive' => 0,
				'template' => 'Templates/Frontend/payment_methods/sofortrechnung.tpl',
				'additionaldescription' => '
					<a href="'.$srLandingUrl.'" target="_blank">
						'.$srBanner.'
					</a>
				',
				'logo' => '
					<a style="padding-right: 2px" href="'.$srLandingUrl.'" target="_blank">
						'.$srLogo.'
					</a>
				',
			),
			array(
				'name' => 'sofortlastschrift_multipay',
				'description' => $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sl_public_title'),
				'action' => 'sofort',
				'esdactive' => 0,
				'template' => 'Templates/Frontend/payment_methods/sofortlastschrift.tpl',
				'additionaldescription' => '
					<a href="'.$slLandingUrl.'" target="_blank">
						'.$slBanner.'
					</a>
				',
				'logo' => '
					<a style="padding-right: 2px" href="'.$slLandingUrl.'" target="_blank">
						'.$slLogo.'
					</a>
				',
			),
			array(
				'name' => 'lastschriftbysofort_multipay',
				'description' => $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_ls_public_title'),
				'action' => 'sofort',
				'esdactive' => 0,
				'template' => 'Templates/Frontend/lastschriftbysofort.tpl',
				'additionaldescription' => '
					<a href="'.$lsLandingUrl.'" target="_blank">
						'.$lsBanner.'
					</a>
				',
				'logo' => '
					<a style="padding-right: 2px" href="'.$lsLandingUrl.'" target="_blank">
						'.$lsLogo.'
					</a>
				',
			),
			array(
				'name' => 'iDeal',
				'description' => 'iDEAL',
				'action' => 'sofort',
				'esdactive' => 1,
				'template' => '/Templates/Frontend/payment_methods/ideal.tpl',
				'additionaldescription' => '
					<a href="'.$idealLandingUrl.'" target="_blank">
						'.$idealBanner.'
					</a>
					',
					'logo' => '
					<a style="padding-right: 2px" href="'.$this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_ideal_landing_url').'" target="_blank">
						'.$this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_ideal_banner_image').'
					</a>
				',
			),
		);
	}
	
	
	/**
	 * 
	 * Fetch statuses from file
	 */
	private function fetchStatuses() {
		$statuses = parse_ini_file(dirname(__FILE__).'/config/config.ini', true);
		return $statuses;
	}
	
	
	/**
	 * 
	 * Fetch status settings from file
	 */
	private function fetchStatusSettings() {
		$status = parse_ini_file(dirname(__FILE__).'/config/config.ini', true);
		return $status['statusSettings'];
	}
	
	
	/**
	 * 
	 * Fetch status descriptions from file
	 */
	private function fetchStatusDescriptions() {
		$status = parse_ini_file(dirname(__FILE__).'/config/config.ini', true);
		return $status['statusDescription'];
	}
	
	
	/**
	 * In future versions: include all supported languages
	 */
	private function createLanguages() {
		$this->insertGermanLanguage();
		$this->insertEnglishLanguage();
		// $this->insertItalianLanguage();
		$this->insertDutchLanguage();
		$this->insertPolishLanguage();
		$this->insertItalianLanguage();
		$this->insertFrenchLanguage();
	}
	
	
	/**
	 * 
	 * Insert English language snippets
	 * @throws Exception
	 */
	private function insertEnglishLanguage() {
		$langFile = dirname(__FILE__).'/build/shopware_en.sql';
		if(file_exists($langFile)) {	
			$sql = file_get_contents($langFile);
			Shopware()->Db()->exec($sql);
		} else {
			throw new Exception('English language data not found');
		}
	}
	
	
	/**
	 * 
	 * Insert German language snippets
	 * @throws Exception
	 */
	private function insertGermanLanguage() {
		$langFile = dirname(__FILE__).'/build/shopware_de.sql';
		if(file_exists($langFile)) {	
			$sql = file_get_contents($langFile);
			Shopware()->Db()->exec($sql);
			return true;
		} else {
			throw new Exception('German language data not found');
			return false;
		}
	}
	
	
	/**
	 * 
	 * Insert Dutch language snippets
	 * @throws Exception
	 */
	private function insertDutchLanguage() {
		$langFile = dirname(__FILE__).'/build/shopware_nl.sql';
		if(file_exists($langFile)) {	
			$sql = file_get_contents($langFile);
			Shopware()->Db()->exec($sql);
			return true;
		} else {
			throw new Exception('Dutch language data not found');
			return false;
		}
	}
	
	
	/**
	 * 
	 * Insert Polish language snippets
	 * @throws Exception
	 */
	private function insertPolishLanguage() {
		$langFile = dirname(__FILE__).'/build/shopware_pl.sql';
		if(file_exists($langFile)) {	
			$sql = file_get_contents($langFile);
			Shopware()->Db()->exec($sql);
			return true;
		} else {
			throw new Exception('Polish language data not found');
			return false;
		}
	}
	
	
	/**
	 * 
	 * Insert Italian language snippets
	 * @throws Exception
	 */
	private function insertItalianLanguage() {
		$langFile = dirname(__FILE__).'/build/shopware_it.sql';
		if(file_exists($langFile)) {	
			$sql = file_get_contents($langFile);
			Shopware()->Db()->exec($sql);
			return true;
		} else {
			throw new Exception('Italian language data not found');
			return false;
		}
	}
	
	
	/**
	 * 
	 * Insert French language snippets
	 * @throws Exception
	 */
	private function insertFrenchLanguage() {
		$langFile = dirname(__FILE__).'/build/shopware_fr.sql';
		if(file_exists($langFile)) {	
			$sql = file_get_contents($langFile);
			Shopware()->Db()->exec($sql);
			return true;
		} else {
			throw new Exception('French language data not found');
			return false;
		}
	}
	
	
	/**
	 * 
	 * Insert Turkish language snippets
	 * @throws Exception
	 */
	private function insertTurkishLanguage() {
		$langFile = dirname(__FILE__).'/build/shopware_tr.sql';
		if(file_exists($langFile)) {	
			$sql = file_get_contents($langFile);
			Shopware()->Db()->exec($sql);
			return true;
		} else {
			throw new Exception('Turkish language data not found');
			return false;
		}
	}
	
	
	/**
	 * Remove any language snippets
	 */
	private function removeLanguageSnippets() {
		$sql = 'DELETE FROM `s_core_snippets` WHERE `namespace` LIKE "sofort_multipay%";';
		Shopware()->Db()->exec($sql);
		return true;
	}
	
	
	/**
	 * 
	 * Used a hook for Shopware_Controllers_Frontend_Account
	 * (or let's say: created a proxy object for the above mentioned controller, implemented an own paymentAction
	 * to validate some custom fields when choosing payment)
	 * http://wiki.shopware.de/Einfuehrung-Hook-System_detail_615.html
	 */
	private function createHooks() {
		$hook = $this->createHook(
			'Shopware_Controllers_Frontend_Account',
			'savePaymentAction',
			'sofortPaymentAction',
			Enlight_Hook_HookHandler::TypeBefore,
			0
		);
		$this->subscribeHook($hook);
		return true;
	}
	
	
	/**
	 * The above mentioned custom payment action to validate user's credentials for SOFORT AG
	 * @param Enlight_Hook_HookArgs $args
	 */
	static function sofortPaymentAction(Enlight_Hook_HookArgs $args) {
		$doc = $args->getSubject();
		$params = $args->getArgs();
		// the chosen payment method
		$chosenPayment = $doc->Request()->register[payment];
		// fetch the paymentMeans from s_core_paymentmeans
		$p = self::getActiveSofortPaymentMeans();
		
		// take the chosen payment method into account
		switch($p[$chosenPayment]['name']) {
			case('vorkassebysofort_multipay'):
				if($doc->Request()->vorkassebysofort_dhw != 'on') {
					$url = $doc->Front()->Router()->assemble(array(
						'controller' => 'account',
						'action' => 'payment',
						'sofort_error' => 'vorkassebysofort_dhw_not_accepted'
					));
					header('location: '.$url);
					die;
				}
				break;
			case('sofortrechnung_multipay'):
				if($doc->Request()->sofortrechnung_dhw != 'on') {
					$url = $doc->Front()->Router()->assemble(array(
						'controller' => 'account',
						'action' => 'payment',
						'sofort_error' => 'sofortrechnung_dhw_not_accepted'
					));
					header('location: '.$url);
					die;
				}
				
				break;
			case('sofortueberweisung_multipay'):
				// nothing to be done here
				break;
			case('vorkassebysofort_multipay'):
				// nothing to be done here
				break;
			case('sofortlastschrift_multipay'):
				// nothing to be done here
				break;
			case('lastschriftbysofort_multipay'):
				
				$validationErrors = array();
				
				if($doc->Request()->lastschriftbysofort_account_number == '') {
					$validationErrors[] = 'lastschriftbysofort_account_number';
				} 
				if($doc->Request()->lastschriftbysofort_bank_code == '') {
					$validationErrors[] = 'lastschriftbysofort_bank_code';
				}
				if($doc->Request()->lastschriftbysofort_holder == '') {
					$validationErrors[] = 'lastschriftbysofort_holder';
				}
				if($doc->Request()->lastschriftbysofort_dhw != 'on') {
					$validationErrors[] = 'lastschriftbysofort_dhw_not_accepted';
				}
				
				$i = 0;
				$errorString = '';
				foreach($validationErrors as $error) {
					($i == 0) ? $errorString = $error : $errorString .= '|'.$error;
					$i++;
				}
				
				if(!empty($validationErrors)) {
					
					$url = $doc->Front()->Router()->assemble(array(
						'controller' => 'account',
						'action' => 'payment',
						'sofort_error' => $errorString,
					));
					header('location: '.$url);
					die;
				}
				break;
			case('iDeal'):
				// nothing to be done here
				break;	
			default: return;	// allow other payment methods to be chosen. if no return here, there wouldn't be any chance to select any other
		}
		
		return true;
	}
	
	
	/**
	 * 
	 * Create all the events necessary for handling both the frontend and the backend ...
	 */
	private function createEvents() {
		/**
		 * 
		 * Event is triggered when page is loaded
		 * Used to show Payment Network Logo on specific parts of the template
		 */
		$event = $this->createEvent(
			'Enlight_Controller_Action_PostDispatch_Frontend_Index',
			'onLoadFrontpage'
		);
		$this->subscribeEvent($event);
		
		/**
		 * 
		 * Frontend Controller
		 */
		$event = $this->createEvent(
			'Enlight_Controller_Dispatcher_ControllerPath_Frontend_Sofort',
			'onSofortPaymentCall'
		);
		$this->subscribeEvent($event);
		
		/**
		 * 
		 * Frontend Controller used for notifications
		 */
		$event = $this->createEvent(
			'Enlight_Controller_Dispatcher_ControllerPath_Frontend_SofortNotification',
			'onNotificationCall'
		);
		$this->subscribeEvent($event);
		
		/**
		 * 
		 * Backend Controller
		 */
		$event = $this->createEvent(
			'Enlight_Controller_Dispatcher_ControllerPath_Backend_SofortSettings',
			'onBackendSofortSettings'
		);
		$this->subscribeEvent($event);
		
		/**
		 * 
		 * Backend Controller
		 */
		$event = $this->createEvent(
			'Enlight_Controller_Dispatcher_ControllerPath_Backend_SofortOrders',
			'onBackendSofortOrders'
		);
		$this->subscribeEvent($event);
		
		/**
		 * 
		 * Frontend Account Controller
		 */
		$event = $this->createEvent(
			'Enlight_Controller_Action_PostDispatch_Frontend_Account',
			'onPostDispatchAccount'
		);
		$this->subscribeEvent($event);
		
		/**
		 * 
		 * Frontend Controller
		 */
		$event = $this->createEvent(
			'Enlight_Controller_Action_PostDispatch_Frontend_Checkout',
			'onPostDispatchCheckout'
		);
		$this->subscribeEvent($event);
		
		$event = $this->createEvent(
			'Enlight_Controller_Action_PostDispatch_Frontend_Register',
			'onPostDispatchRegister'
		);
		$this->subscribeEvent($event);
		
		/*
		$event = $this->createEvent(
			'Shopware_Modules_Order_SendMail_BeforeSend',
			'onSendMailForVorkasse'
		);
		$this->subscribeEvent($event);
		*/
		
		return true;
	}
	
	
	/**
	 * Creates the backend menu
	 */
	private function createBackendMenu() {
		$parent = $this->Menu()->findOneBy('label', 'Zahlungen');
		$item = $this->createMenuItem(array(
			'label' => 'sofort.com',
			'onclick' => 'openAction(\'SofortOrders\');',
			'class' => 'ico2 date2',
			'active' => 1,
			'parent' => $parent,
			'style' => 'background-position: 5px 5px;'
		));
		$this->Menu()->addItem($item);
		$this->Menu()->save();
	}
	
	
	/**
	 * 
	 * create core config table
	 * @return true
	 */
	private function createCoreConfigTable() {
		
		if(!$this->isExistingConfigGroup()) {
			$sql = 'INSERT INTO `s_core_config_groups` (`name`, `position`, `parent`, `file`, `description`, `action`) 
					VALUES (?, 1, 17, "", "SOFORT AG", "backend/SofortSettings")';
			$fields = array(
				$this->configGroup,
			);
			Shopware()->Db()->query($sql, $fields);
			$this->sofortConfigId = Shopware()->Db()->lastInsertId();
		} else {
			$this->sofortConfigId = $this->isExistingConfigGroup();
			return;
		}
		
		$sql = 'INSERT INTO `s_core_config` (`group`, `name`, `value`, `description`, `required`, `detailtext`, `fieldtype`)
				VALUES (?, "sofort_api_key", "", ?, 0, "", "text");
		';
		$fields = array(
			$this->sofortConfigId,
			$this->snippets->getSnippet('sofort_multipay')->get('sofort_multipay_api_key'),
		);
		Shopware()->Db()->query($sql, $fields);
		$sql = 'INSERT INTO `s_core_config` (`group`, `name`, `value`, `description`, `required`, `detailtext`, `fieldtype`)
				VALUES (?, "ideal_api_key", "", ?, 0, "", "text");
		';
		$fields = array(
			$this->sofortConfigId,
			$this->snippets->getSnippet('sofort_multipay')->get('sofort_multipay_api_key'),
		);
		Shopware()->Db()->query($sql, $fields);
		$sql = 'INSERT INTO `s_core_config` (`group`, `name`, `value`, `description`, `required`, `detailtext`, `fieldtype`)
				VALUES (?, "sofort_api_key_valid", "", ?, 0, "", "text");
		';
		$fields = array(
			$this->sofortConfigId,
			'Valid API Key (SOFORT gateway)',
		);
		Shopware()->Db()->query($sql, $fields);
		$sql = 'INSERT INTO `s_core_config` (`group`, `name`, `value`, `description`, `required`, `detailtext`, `fieldtype`)
				VALUES (?, "ideal_api_key_valid", "", ?, 0, "", "text");
		';
		$fields = array(
			$this->sofortConfigId,
			'Valid API Key (SOFORT iDeal)',
		);
		Shopware()->Db()->query($sql, $fields);
		$sql = 'INSERT INTO `s_core_config` (`group`, `name`, `value`, `description`, `required`, `detailtext`, `fieldtype`)
				VALUES (?, "ideal_project_password", "", ?, 0, "", "text");
		';
		$fields = array(
			$this->sofortConfigId,
			'iDeal Project Password',
		);
		Shopware()->Db()->query($sql, $fields);
		$sql = 'INSERT INTO `s_core_config` (`group`, `name`, `value`, `description`, `required`, `detailtext`, `fieldtype`)
				VALUES (?, "ideal_notification_password", "", ?, 0, "", "text");
		';
		$fields = array(
			$this->sofortConfigId,
			'iDeal Notification Password',
		);
		Shopware()->Db()->query($sql, $fields);
		$sql = 'INSERT INTO `s_core_config` (`group`, `name`, `value`, `description`, `required`, `detailtext`, `fieldtype`)
				VALUES (?, "ideal_encryption_algorithm", "", ?, 0, "", "text");
		';
		$fields = array(
			$this->sofortConfigId,
			'iDeal Encryption Algorithm',
		);
		Shopware()->Db()->query($sql, $fields);
		$sql = 'INSERT INTO `s_core_config` (`group`, `name`, `value`, `description`, `required`, `detailtext`, `fieldtype`)
				VALUES (?, "sofort_payment_secret", "", ?, 0, "", "text");
		';
		$fields = array(
			$this->sofortConfigId,
			$this->snippets->getSnippet('sofort_multipay_bootstrap')->get('payment_secret'),
		);
		Shopware()->Db()->query($sql, $fields);
		
		$paymentReason = '-TRANSACTION-';
		$sql = 'INSERT INTO `s_core_config` (`group`, `name`, `value`, `description`, `required`, `detailtext`, `fieldtype`)
				VALUES (?, "sofort_payment_reason", ?, "Payment Reason", 0, "", "text");
		';
		$fields = array(
			$this->sofortConfigId,
			$paymentReason,
		);
		Shopware()->Db()->query($sql, $fields);
		
		$paymentReason = Shopware()->Config()->shopname;
		$sql = 'INSERT INTO `s_core_config` (`group`, `name`, `value`, `description`, `required`, `detailtext`, `fieldtype`)
				VALUES (?, "sofort_payment_reason2", ?, "Payment Reason", 0, "", "text");
		';
		$fields = array(
			$this->sofortConfigId,
			$paymentReason,
		);
		Shopware()->Db()->query($sql, $fields);
		
		$sql = 'INSERT INTO `s_core_config` (`group`, `name`, `value`, `description`, `required`, `detailtext`, `fieldtype`)
				VALUES (?, "show_sofort_banner", "", ?, 0, "", "text");
		';
		$fields = array(
			$this->sofortConfigId,
			'Banner auf der Startseite anzeigen ja/nein',
		);
		Shopware()->Db()->query($sql, $fields);
		$sql = 'INSERT INTO `s_core_config` (`group`, `name`, `value`, `description`, `required`, `detailtext`, `fieldtype`)
				VALUES (?, "sofort_ueberweisung_customer_protection", "", ?, 0, "", "text");
		';
		$fields = array(
			$this->sofortConfigId,
			$this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_su_public_title_ks'),
		);
		Shopware()->Db()->query($sql, $fields);
		$sql = 'INSERT INTO `s_core_config` (`group`, `name`, `value`, `description`, `required`, `detailtext`, `fieldtype`)
				VALUES (?, "vorkassebysofort_customer_protection", "", ?, 0, "", "text");
		';
		$fields = array(
			$this->sofortConfigId,
			$this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sv_public_title_ks'),
		);
		Shopware()->Db()->query($sql, $fields);
		$sql = 'INSERT INTO `s_core_config` (`group`, `name`, `value`, `description`, `required`, `detailtext`, `fieldtype`)
				VALUES (?, "sofort_su_text_or_banner", "1", ?, 0, "", "text");
		';
		$fields = array(
			$this->sofortConfigId,
			$this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_banner_or_text'),
		);
		Shopware()->Db()->query($sql, $fields);
		$sql = 'INSERT INTO `s_core_config` (`group`, `name`, `value`, `description`, `required`, `detailtext`, `fieldtype`)
				VALUES (?, "sofort_sl_text_or_banner", "1", ?, 0, "", "text");
		';
		$fields = array(
			$this->sofortConfigId,
			$this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_banner_or_text'),
		);
		Shopware()->Db()->query($sql, $fields);
		return true;
	}
	
	
	/**
	 * 
	 * Is existing config group
	 */
	private function isExistingConfigGroup() {
		$sql = 'SELECT id FROM `s_core_config_groups` WHERE name = ?';
		$fields = array(
			$this->configGroup,
		);
		return Shopware()->Db()->FetchOne($sql, $fields);
	}
	
	
	/**
	 * 
	 * Create statuses, insert into database
	 */
	private function createStatuses() {
		foreach($this->statuses as $paymentMethod => $status) {
			switch($paymentMethod) {
				case('sr'): 
					foreach($this->statuses['sr'] as $paymentState => $paymentStateDescription) {
						$this->setWordingForPaymentState($paymentState, $paymentStateDescription, $this->sofortConfigId);
						$this->initStatusWithDefaultValue('sr', $paymentState);
					}
				break;
				case('su'): 
					foreach($this->statuses['su'] as $paymentState => $paymentStateDescription) {
						$this->setWordingForPaymentState($paymentState, $paymentStateDescription, $this->sofortConfigId);
						$this->initStatusWithDefaultValue('su', $paymentState);
					}
				break;
				case('sl'): 
					foreach($this->statuses['sl'] as $paymentState => $paymentStateDescription) {
						$this->setWordingForPaymentState($paymentState, $paymentStateDescription, $this->sofortConfigId);
						$this->initStatusWithDefaultValue('sl', $paymentState);
					}
				break;
				case('ls'): 
					foreach($this->statuses['ls'] as $paymentState => $paymentStateDescription) {
						$this->setWordingForPaymentState($paymentState, $paymentStateDescription, $this->sofortConfigId);
						$this->initStatusWithDefaultValue('ls', $paymentState);
					}
				break;
				case('sv'): 
					foreach($this->statuses['sv'] as $paymentState => $paymentStateDescription) {
						$this->setWordingForPaymentState($paymentState, $paymentStateDescription, $this->sofortConfigId);
						$this->initStatusWithDefaultValue('sv', $paymentState);
					}
				break;
				case('iDeal'): 
					foreach($this->statuses['iDeal'] as $paymentState => $paymentStateDescription) {
						$this->setWordingForPaymentState($paymentState, $paymentStateDescription, $this->sofortConfigId);
						$this->initStatusWithDefaultValue('iDeal', $paymentState);
					}
				break;
			}
		}
		return true;
	}
	
	
	/**
	 * 
	 * Set the wording for payment state
	 */
	private function setWordingForPaymentState($paymentState, $paymentStateDescription, $id) {
		$fields = array();
		switch($paymentState) {
			case('temporary'): 
				$fields = array(
					$id,
					$paymentStateDescription,
					$paymentState,
				);
			break;
			case('pending'): 
				$fields = array(
					$id,
					$paymentStateDescription,
					$paymentState,
				);
			break;
			case('canceled'):
				$fields = array(
					$id,
					$paymentStateDescription,
					$paymentState,
				);
			break;
			case('confirmed'):
				$fields = array(
					$id,
					$paymentStateDescription,
					$paymentState,
				);
			break;
			case('partially'):
				$fields = array(
					$id,
					$paymentStateDescription,
					$paymentState,
				);
			break;
			case('overpayment'):
				$fields = array(
					$id,
					$paymentStateDescription,
					$paymentState,
				);
			break;
			case('refunded'): 
				$fields = array(
					$id,
					$paymentStateDescription,
					$paymentState,
				);
			break;
			case('compensation'): 
				$fields = array(
					$id,
					$paymentStateDescription,
					$paymentState,
				);
			break;
			case('loss'): 
				$fields = array(
					$id,
					$paymentStateDescription,
					$paymentState,
				);
			break;
			case('received'): 
				$fields = array(
					$id,
					$paymentStateDescription,
					$paymentState,
				);
			break;
			case('overpayment'): 
				$fields = array(
					$id,
					$paymentStateDescription,
					$paymentState,
				);
			break;
		}
		$coreConfig = $this->getCoreConfig();
		$coreConfig = array_flip($coreConfig);
		if (!empty($fields) && !key_exists($fields[1], $coreConfig)) {
			$sql = 'INSERT INTO `s_core_config` (`group`, `name`, `value`, `description`, `required`, `detailtext`, `fieldtype`)
				VALUES (?, ?, "", ?, 0, "", "text");
			';
			Shopware()->Db()->query($sql, $fields);
		}
	} 
	
	
	/**
	 * 
	 * Initiate status with it's default value
	 * @param string $paymentMethod
	 * @param string $status
	 */
	private function initStatusWithDefaultValue($paymentMethod, $status) {
		if(!empty($this->statusSettings[$paymentMethod.'_'.$status])) {
			$sql = 'UPDATE `s_core_config` SET value = ? WHERE name = ?';
			$fields = array(
				$this->initialStates[$status],
				$this->statusSettings[$paymentMethod.'_'.$status],
			);
			return Shopware()->Db()->query($sql, $fields);
		}
	}
	
	
	/**
	 * 
	 * Getter for core states
	 */
	private function getCoreStates() {
		$states = array();
		$sql = 'SELECT description FROM `s_core_states` WHERE description LIKE ?';
		$fields = array(
			'<span class="sofort">%',
		);
		$states = Shopware()->Db()->fetchAll($sql, $fields);
		
		$newStates = array();
		
		// make unique numeric indexes
		foreach($states as $state) {
			$newStates[] = $state['description'];
		}
		
		return $newStates;
	}
	
	
	/**
	 * 
	 * Getter for core state ids
	 */
	private function getCoreStateIds() {
		$states = array();
		$sql = 'SELECT id FROM `s_core_states` WHERE description LIKE ?';
		$fields = array(
			'<span class="sofort">%',
		);
		$states = Shopware()->Db()->fetchAll($sql, $fields);
		return $states;
	}
	
	
	/**
	 * 
	 * Getter for core config
	 */
	private function getCoreConfig() {
		$config = array();
		$sql = 'SELECT name FROM `s_core_config` WHERE name LIKE ?';
		$fields = array(
			'Sofort_%',
		);
		$configs = Shopware()->Db()->fetchAll($sql, $fields);
		$config = array();
		
		foreach($configs as $item) {
			$config[] = $item['name'];
		}
		
		return $config;
	}
	
	
	/**
	 * 
	 * create all core states
	 */
	private function createCoreStates() {
		$sql = 'select max(id) FROM `s_core_states`';
		$nextCoreStateId = Shopware()->Db()->fetchOne($sql);
		$nextCoreStateId++;
		$coreStates = $this->getCoreStates();
		//$coreStates = array_flip($coreStates);
		
		foreach ($this->statusDescriptions as $state => $description) {
			$stateDescription = '<span class="sofort">'.$this->snippets->getSnippet('sofort_multipay_bootstrap')->get($description).'</span>';
			
			if(!in_array($stateDescription, $coreStates)) {
				$sql = 'INSERT INTO `s_core_states` (`id`, `description`, `position`, `group`, `mail`)
					VALUES (?, ?, 2, "payment", 0);
				';
				$fields = array(
					$nextCoreStateId,
					$stateDescription,
				);
				Shopware()->Db()->query($sql, $fields);
				$this->initialStates[$state] = $nextCoreStateId;
				$nextCoreStateId++;
			} else {
				$coreStates = $this->getCoreStateIds();
				foreach($coreStates as $state) {
					$this->initialStates[$state] = $state;
				}
			}
		}
		
		return true;
	}
	
	
	/**
	 * 
	 * Remove all core states for this module
	 */
	private function removeCoreStates() {
		foreach ($this->statusDescriptions as $state => $description) {
			$sql = 'DELETE FROM `s_core_states` WHERE description = ? AND `group` = "payment"';
			$fields = array(
				$description,
			);
			
			Shopware()->Db()->query($sql, $fields);
		}
	}
	
	
	/**
	 * remove core config table
	 * @return true
	 */
	private function removeCoreConfigTable() {
		$sql = 'SELECT id FROM `s_core_config_groups` WHERE name = ?';
		$fields = array(
			$this->configGroup,
		);
		$sofortConfigId = Shopware()->Db()->FetchOne($sql, $fields);
		
		if(!empty($sofortConfigId)) {
			$sql = 'DELETE FROM `s_core_config_groups` WHERE id = ?';
			$fields = array(
				$sofortConfigId,
			);
			Shopware()->Db()->query($sql, $fields);
		
			$sql = 'DELETE FROM `s_core_config` WHERE s_core_config.group = '.$sofortConfigId.'';
			$fields = array(
				$sofortConfigId,
			);
			Shopware()->Db()->query($sql, $fields);
		}
		return true;
	}
	
	
	/**
	 * Remove all module's config options
	 * @deprecated
	 */
	private function removeConfigOptions() {
		$sql = 'DELETE FROM `s_core_plugin_configs` WHERE name LIKE "%sofort%"';
		Shopware()->Db()->exec($sql);
	}
	
	
	/**
	 * Remove payment means
	 * @return true
	 */
	private function removePaymentMeans() {
		foreach($this->products as $product) {
			$sql = 'DELETE FROM s_core_paymentmeans WHERE `name` = ?';
			$fields = array(
				$product['name'],
			);
			Shopware()->Db()->query($sql, $fields);
		}	
		
		return true;
	}
	
	
	/**
	 * Create and save payments
	 * @return true
	 */
	protected function createPayments() {
		
		$i = 1;
		foreach($this->products as $product) {
			$paymentRow = Shopware()->Payments()->createRow(array(
				'name' => $product['name'],
				'description' => $product['description'],
				'position' => $i,
				'action' => $product['action'],
				'active' => 0,
				'esdactive' => $product['esdactive'],
				'pluginID' => $this->getId(),
				'template' => $product['template'], 
				'additionaldescription' => $product['additionaldescription'],
			))->save();
			$i++;
			if($product['name'] === 'iDeal') {
				$this->setIdealOnlyForNl($paymentRow);
			}
		}
		
		
		return true;
	}
	
	
	/**
	 * 
	 * Set iDeal active only for the Netherlands
	 * @param int $paymentRow
	 */
	private function setIdealOnlyForNl($paymentRow) {
		// select the netherlands
		$sql = 'SELECT id FROM `s_core_countries` WHERE `countryiso` = "nl"';
		$countryId = Shopware()->Db()->fetchOne($sql);
		$sql = 'INSERT INTO `s_core_paymentmeans_countries` (`paymentID`, `countryID`) VALUES (?, ?)';
		$fields = array(
			$paymentRow,
			$countryId,
		);
		Shopware()->Db()->query($sql, $fields);
		return true;
	}
	
	
	/**
	 * Create the orders table (if none exist)
	 * @return true
	 */
	private function createOrdersTable() {
		Shopware()->Db()->exec(
			'CREATE TABLE IF NOT EXISTS `sofort_orders` (
			`id` INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
			`paymentMethod` VARCHAR(32),
			`paymentDescription` VARCHAR(64),
			`transactionId` VARCHAR(32) NOT NULL,
			`secret` VARCHAR(32) NOT NULL,
			`paymentStatus` VARCHAR(32),
			`dateTime` TIMESTAMP NOT NULL default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
			KEY `paymentMethod` (`paymentMethod`),
			KEY `paymentDescription` (`paymentDescription`),
			KEY `transactionId` (`transactionId`),
			KEY `secret` (`secret`),
			KEY `paymentStatus` (`paymentStatus`)
			)'
		);
		
		$databaseConfig = Shopware()->Db()->getConfig();
		$costumerProtectionColumnExists = Shopware()->Db()->fetchOne('SELECT count(COLUMN_NAME) as count FROM information_schema.COLUMNS 
			WHERE TABLE_SCHEMA = "'.$databaseConfig['dbname'].'" AND TABLE_NAME = "sofort_orders" AND COLUMN_NAME = "customerProtection"'
		);
		
		if($costumerProtectionColumnExists == 0) {
			Shopware()->Db()->exec('ALTER TABLE `sofort_orders` ADD `customerProtection` TINYINT NOT NULL AFTER `paymentDescription`');
		}
		
		return true;
	}
	
	
	/**
	 * Create the invoice table (if none exist)
	 */
	private function createProductTable() {
		Shopware()->Db()->exec(
			'CREATE TABLE IF NOT EXISTS `sofort_products` (
			`id` int(10) unsigned NOT NULL AUTO_INCREMENT,
			`order_id` int(11) unsigned NOT NULL,
			`transactionId` varchar(32) NOT NULL,
			`amount` float DEFAULT NULL,
			`comment` text,
			`date_modified` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
			PRIMARY KEY (`id`)
			)'
		);
		return true;
	}
	
	
	/**
	 * Create the inovice status table (if none exist)
	 */
	private function createStatusTable() {
		Shopware()->Db()->exec(
			'CREATE TABLE IF NOT EXISTS `sofort_status` (
			`id` int(10) unsigned NOT NULL AUTO_INCREMENT,
			`sofort_product_id` int(11) unsigned NOT NULL,
			`status_id` int(11) unsigned NOT NULL,
			`status` varchar(32) NOT NULL,
			`status_reason` varchar(32) NOT NULL,
			`invoice_status` varchar(32) DEFAULT NULL,
			`invoice_objection` varchar(45) DEFAULT NULL,
			`items` text,
			`date_modified` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
			PRIMARY KEY (`id`)
			)'
		);
		$databaseConfig = Shopware()->Db()->getConfig();
		$itemColumnExists = Shopware()->Db()->fetchOne('SELECT count(COLUMN_NAME) as count FROM information_schema.COLUMNS 
			WHERE TABLE_SCHEMA = "'.$databaseConfig['dbname'].'" AND TABLE_NAME = "sofort_status" AND COLUMN_NAME = "items"'
		);
		$commentColumnExists = Shopware()->Db()->fetchOne('SELECT count(COLUMN_NAME) as count FROM information_schema.COLUMNS 
			WHERE TABLE_SCHEMA = "'.$databaseConfig['dbname'].'" AND TABLE_NAME = "sofort_status" AND COLUMN_NAME = "comment"'
		);
		
		if($commentColumnExists == 0) {
			Shopware()->Db()->exec('ALTER TABLE `sofort_status` ADD `comment` TEXT NOT NULL AFTER `invoice_objection`');
		}
		if($itemColumnExists == 0) {
			Shopware()->Db()->exec('ALTER TABLE `sofort_status` ADD `items` TEXT NOT NULL AFTER `invoice_objection`');
		}
		
		return true;
	}
	
	
	/**
	 * 
	 * Create sofort_temp_orders table
	 */
	private function createTemporaryOrdersTable() {
		Shopware()->Db()->exec(
			'CREATE TABLE IF NOT EXISTS `sofort_temp_orders` (
			`id` int(11) unsigned NOT NULL AUTO_INCREMENT,
			`ordernumber` int(11) unsigned NOT NULL,
			`secret` varchar(32) NOT NULL,
			`cart_content` text NOT NULL,
			`user_data` text NOT NULL,
			`customer_comment` text NOT NULL,
			`comment` text NOT NULL,
			`order_time` datetime NOT NULL DEFAULT "0000-00-00 00:00:00",
			PRIMARY KEY (`id`)
			)'
		);
		return true;
	}
	
	
	/**
	 * 
	 * Remove the orders table when plugin is being uninstalled ...
	 */
	private function removeOrdersTable() {
		Shopware()->Db()->exec(
			'DROP TABLE IF EXISTS `sofort_orders`'
		);
		return true;
	}
	
	
	/**
	 * 
	 * Remove sofort_product table
	 */
	private function removeProductTable() {
		Shopware()->Db()->exec(
			'DROP TABLE IF EXISTS `sofort_product`'
		);
		return true;
	}
	
	
	/**
	 * 
	 * remove sofort_status table
	 */
	private function removeStatusTable() {
		Shopware()->Db()->exec(
			'DROP TABLE IF EXISTS `sofort_status`'
		);
		return true;
	}
	
	
	/**
	 * Create the orders table
	 * @return true
	 */
	private function createSettingsTable() {
		Shopware()->Db()->exec(
			'CREATE TABLE IF NOT EXISTS `sofort_user_settings` (
			`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
			`userId` int(11) NOT NULL,
			`ls_account_number` VARCHAR(32) NOT NULL,
			`ls_bank_code` VARCHAR(32) NOT NULL,
			`ls_holder` VARCHAR(64) NOT NULL,
			`sv_customer_protection` TINYINT(1) DEFAULT 0,
			`su_customer_protection` TINYINT(1) DEFAULT 0,
			`sv_dhw_accepted`  TINYINT(1) DEFAULT 0,
			`sr_dhw_accepted`  TINYINT(1) DEFAULT 0,
			`sl_dhw_accepted`  TINYINT(1) DEFAULT 0,
			`ideal_chosen_bank`  INT(11) DEFAULT 0
			)'
		);
		return true;
	}
	
	
	/**
	 * Restore original sofortberweisung if necessary
	 * Sofortueberweisung got deleted in some parts in a former version
	 * this is to restore those parts
	 * 
	 * http://forum.shopware.de/installation-einstieg-f9/neue-sofort-multi-schnittstelle-funktioniert-nicht-t5222.html
	 */
	private function setUeberweisungBack() {
		Shopware()->Db()->exec(
			'INSERT INTO `s_core_checklist` (`type`, `checked`, `winurl`, `skeleton`, `attr1`, `area`, `subarea`, `option`, `module`, `paymentmean`) VALUES
			( 1, 0, "../presetting/settingsdetail.php?id=66", "", "", "Grundeinstellungen", "Schnittstellen > Sofortberweisung.de", "berprfung der Einstellungen", "", "sofortueberweisung");'
		);
		Shopware()->Db()->exec(
			'INSERT IGNORE INTO `s_core_config_groups` (`id`, `name`, `position`, `parent`, `file`, `description`, `action`) VALUES
			(66,"Sofortberweisung",0,17,"","","")'
		);
		Shopware()->Db()->exec(
			'INSERT IGNORE INTO `s_core_config` (`id`, `group`, `name`, `value`, `description`, `required`, `warning`, `detailtext`, `multilanguage`, `fieldtype`) VALUES
			(179, 66, "sSOFORTUSERID", "", "User-ID", 0, 0, "", 0, ""),
			(180, 66, "sSOFORTSECRETKEY", "", "Sicherheitsschlssel", 0, 0, "", 0, ""),
			(181, 66, "sSOFORTPROJECTID", "", "Projekt-ID", 0, 0, "", 0, "")'
		);
		return true;
	}
	
	
	/**
	 * 
	 * Remove the settings table when plugin is being uninstalled ...
	 */
	private function removeSettingsTable() {
		Shopware()->Db()->exec(
			'DROP TABLE IF EXISTS `sofort_user_settings`'
		);
		return true;
	}
	
	
	/**
	 * 
	 * Fetch all active payments and return an array containing information about
	 * @return Array payments
	 */
	private function getActiveSofortPaymentMeans() {
		$sql = 'SELECT `id`, `active`, `name`, `description` FROM `s_core_paymentmeans` WHERE active = 1 ORDER BY position ASC';
		$paymentMeans = Shopware()->Db()->fetchAll($sql);
		
		$sofortMeans = array();
		foreach($paymentMeans as $mean) {
			$sofortMeans[$mean['id']] = array(
				'id' => $mean['id'],
				'name' => $mean['name'],
				'description' => $mean['description'],
			);
		}
		return $sofortMeans;
	}
	
	
	/**
	 *
	 * Payment Network Frontend Controller
	 * @param Enlight_Event_EventArgs $args
	 */
	public static function onSofortPaymentCall(Enlight_Event_EventArgs $args) {
		Shopware()->Template()->addTemplateDir(dirname(__FILE__).'/Templates/Frontend/');
		return dirname(__FILE__).'/Controllers/Frontend/Sofort.php';
	}
	
	
	/**
	 * 
	 * Event for handling notifications
	 * @param Enlight_Event_EventArgs $args
	 */
	public static function onNotificationCall(Enlight_Event_EventArgs $args) {
		Shopware()->Template()->addTemplateDir(dirname(__FILE__).'/Templates/Frontend/');
		return dirname(__FILE__).'/Controllers/Frontend/SofortNotification.php';
	}
	
	
	/**
	 *
	 * Payment Network Backend Settings Controller
	 * @param Enlight_Event_EventArgs $args
	 */
	public static function onBackendSofortSettings(Enlight_Event_EventArgs $args) {
		Shopware()->Template()->addTemplateDir(dirname(__FILE__).'/Templates/Backend/');
		return dirname(__FILE__).'/Controllers/Backend/SofortSettings.php';
	}
	
	
	/**
	 *
	 * Payment Network Backend Orders Controller
	 * @param Enlight_Event_EventArgs $args
	 */
	public static function onBackendSofortOrders(Enlight_Event_EventArgs $args) {
		Shopware()->Template()->addTemplateDir(dirname(__FILE__).'/Templates/Backend/');
		return dirname(__FILE__).'/Controllers/Backend/SofortOrders.php';
	}
	
	
	/**
	 * 
	 * Fetch bank account for lastschriftbysofort
	 * @param Enlight_Event_EventArgs $args
	 * @return Array 
	 */
	private function getBankAccount($args) {
		$view = $args->getSubject()->View();
		$userId = $view->sUserData['billingaddress']['id'];
		if(!empty($userId)) {
			$sql = 'SELECT `ls_account_number`, `ls_bank_code`, `ls_holder` FROM `sofort_user_settings` WHERE userID = '.$userId.'';
			$bankAccount = Shopware()->Db()->fetchAll($sql);
			
			if(!empty($bankAccount)) {
				return $bankAccount[0];
			}
		}
		
		return array();
	}
	
	
	/**
	 * 
	 * Fetch user settings
	 * @param Enlight_Event_EventArgs $args
	 * @return Array 
	 */
	private function getUserSettingsByUserId($userId) {
		if(!empty($userId)) {
			$sql = 'SELECT * FROM `sofort_user_settings` WHERE userID = '.$userId.'';
			$settings = Shopware()->Db()->fetchAll($sql);
			
			if(!empty($settings)) {
				return $settings[0];
			}
		}
		return array();
	}
	
	
	/**
	 * 
	 * Fetch user settings
	 * @param Enlight_Event_EventArgs $args
	 * @return Array 
	 */
	private function getChosenPaymentMethod($userId) {
		if(!empty($userId)) {
			$sql = 'SELECT paymentID FROM `s_user` WHERE id = '.$userId.'';
			$chosenPaymentMethod = Shopware()->Db()->fetchOne($sql);
			
			return $chosenPaymentMethod;
		}
		return 0;
	}
	
	
	/**
	 * 
	 * Insert/Update payment settings for the current user
	 * @param Enlight_Event_EventArgs $args
	 */
	static function saveSofortSettings(Enlight_Event_EventArgs $args) {
		$param = $args->getSubject()->Request()->getParams();
		$view = $args->getSubject()->View();
		$userId = $view->sUserData['billingaddress']['id'];
		
		
		if(!empty($param['register'])) {
			
			if(!self::sofortUserSettingsExist($userId) && !empty($userId)) {
				// initiate table
				$sql = 'INSERT INTO `sofort_user_settings` (`userId`, `ls_account_number`, `ls_bank_code`, `ls_holder`) VALUES (?, ?, ?, ?);';
				Shopware()->Db()->query($sql, array($userId, '', '', ''));
			}
			
			$accountNumber = (!empty($param['lastschriftbysofort_account_number'])) ? $param['lastschriftbysofort_account_number'] : '';
			$bankCode = (!empty($param['lastschriftbysofort_bank_code'])) ? $param['lastschriftbysofort_bank_code'] : '';
			$holder = (!empty($param['lastschriftbysofort_holder'])) ? $param['lastschriftbysofort_holder'] : '';
			
			// just in case we haven't created a user yet, return here
			if(empty($userId)) return false;
			$sql = array();
			$sql[] = 'UPDATE `sofort_user_settings` SET `su_customer_protection` = "'.(($param['sofortueberweisung_cp'] == 'on') ? 1 : 0).'" WHERE `userId` = ?;';
			$sql[] = 'UPDATE `sofort_user_settings` SET `sv_customer_protection` = "'.(($param['vorkassebysofort_cp'] == 'on') ? 1 : 0).'" WHERE `userId` = ?;';
			$sql[] = 'UPDATE `sofort_user_settings` SET `sv_dhw_accepted` = "'.(($param['vorkassebysofort_dhw'] == 'on') ? 1 : 0).'" WHERE `userId` = ?;';
			$sql[] = 'UPDATE `sofort_user_settings` SET `sr_dhw_accepted` = "'.(($param['sofortrechnung_dhw'] == 'on') ? 1 : 0).'" WHERE `userId` = ?;';
			$sql[] = 'UPDATE `sofort_user_settings` SET `sl_dhw_accepted` = "'.(($param['lastschriftbysofort_dhw'] == 'on') ? 1 : 0).'" WHERE `userId` = ?;';
			$fields = array(
				$userId,
			);
			// update all settings accordingly
			foreach($sql as $query) {
				Shopware()->Db()->query($query, $fields);
			}
			
			$bankAccount = self::getBankAccount($args);
			
			if(empty($bankAccount) && !empty($accountNumber) && !empty($bankCode) && !empty($holder)) {
				$sql = 'INSERT INTO `sofort_user_settings` (`userId`, `ls_account_number`, `ls_bank_code`, `ls_holder`) VALUES (?, ?, ?, ?);';
				$fields = array(
					$userId,
					$accountNumber,
					$bankCode,
					$holder,
				);
				Shopware()->Db()->query($sql, $fields);
			} elseif(!empty($bankAccount) && !empty($accountNumber) && !empty($bankCode) && !empty($holder)) {
				$view = $args->getSubject()->View();
				$userId = $view->sUserData['billingaddress']['id'];
				$sql = 'UPDATE `sofort_user_settings` SET `ls_account_number` = ?, `ls_bank_code` = ?, `ls_holder` = ? WHERE `userId` = ?;';
				$fields = array(
					$accountNumber,
					$bankCode,
					$holder,
					$userId,
				);
				Shopware()->Db()->query($sql, $fields);
			}
			
			if(isset($param['idealChosenBank'])) {
				$sql = 'UPDATE `sofort_user_settings` SET `ideal_chosen_bank` = ? WHERE `userId` = ?;';
				$fields = array(
					$param['idealChosenBank'],
					$userId,
				);
				Shopware()->Db()->query($sql, $fields);
			}
			
		}
		return false;
	}
	
	
	/**
	 * 
	 * Do user settings exist
	 * @param int $userId
	 */
	static function sofortUserSettingsExist($userId) {
		$sql = 'SELECT COUNT(ID) FROM sofort_user_settings WHERE userid = ?';
		$userSettingsExist = Shopware()->Db()->fetchOne($sql, array($userId));
		return ($userSettingsExist == 1) ? true : false;
	}
	
	
	/**
	 * 
	 * When sites are being loaded (whatever site it is) do something...
	 * This event is being used to display an image on front page (frontend_index_left_menu)
	 * @param Enlight_Event_EventArgs $args
	 */
	static function onLoadFrontpage(Enlight_Event_EventArgs $args) {	
		Shopware()->Template()->addTemplateDir(dirname(__FILE__).'/Templates/Frontend/');
		$view = $args->getSubject()->View();
		$showSofortBanner = Shopware()->Db()->fetchOne(
			'SELECT value FROM `s_core_config` WHERE `name` = "show_sofort_banner"'
		);
		$view->showSofortBanner = $showSofortBanner;
		$view->extendsTemplate(dirname(__FILE__).'/Templates/Frontend/sofort_banner.tpl');
	}
	
	
	/**
	 * 
	 * Fetch the order number
	 * @param uniqueId/secret $secret
	 */
	static function getOrderNumberBySecret($secret) {
		$sql = 'SELECT ordernumber FROM sofort_temp_orders WHERE secret = ?';
		$fields = array($secret);
		$orderNumber = Shopware()->Db()->fetchOne($sql, $fields);
		return $orderNumber;
	}
	
	
	/**
	 * 
	 * Delete a temporary order
	 * @param uniqueId/secret $secret
	 */
	static function deleteTemporaryOrder($secret) {
		$sql = 'DELETE FROM s_order WHERE temporaryID = ?';
		$fields = array($secret);
		return Shopware()->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * Display Payment overview
	 * @param Enlight_Event_EventArgs $args
	 * @param string $template
	 */
	public static function sofortPaymentOverview(Enlight_Event_EventArgs $args, $template) {
		
		Shopware()->Template()->addTemplateDir(dirname(__FILE__).'/Templates/Frontend/');
		Shopware()->Template()->addTemplateDir(dirname(__FILE__).'/Templates/Frontend/payment_methods');
		
		// get the params (e.g. error messages set by invalid fields)
		$param = $args->getSubject()->Request()->getParams();
		
		// fetch the view
		$view = $args->getSubject()->View();
		$view->extendsTemplate(dirname(__FILE__).'/Templates/Frontend/'.$template);
		
		if(!empty($param['sofort_error'])) {
			$errorArray = explode('|', $param['sofort_error']);
			
			$view->errors = $errorArray;
			$view->sErrorMessages = ' ';
		}
		// fetch the active payment methods and pass them to the view
		$payments = self::getActiveSofortPaymentMeans();
		
		$view->sofortPaymentMeans = $payments;
		
		// fetch data for "Datenschutzhinweise" from remote server
		$svDhwUrl = Shopware()->Snippets()->getSnippet('sofort_multipay_checkout')->get('dhw_sv_conditions');
		$sv_dhw = utf8_decode(file_get_contents($svDhwUrl));
		$matches = array();
		preg_match("/<\!-- content -->.*<\!-- \/content -->/s", $sv_dhw, $matches);
		$view->vorkassebysofort_dhw = $matches[0];
		
		$srDhwUrl = Shopware()->Snippets()->getSnippet('sofort_multipay_checkout')->get('dhw_sr_conditions');
		$sr_dhw = utf8_decode(file_get_contents($srDhwUrl));
		$matches = array();
		preg_match("/<\!-- content -->.*<\!-- \/content -->/s", $sr_dhw, $matches);
		$view->sofortrechnung_dhw = $matches[0];
		
		
		$view->dhwNoticeSR = self::makeConditionHint('sr');
		$view->dhwNoticeSV = self::makeConditionHint('sv');
		$view->dhwNoticeLS = self::makeConditionHint('ls');
		
		$lsDhwUrl = Shopware()->Snippets()->getSnippet('sofort_multipay_checkout')->get('dhw_ls_conditions');
		$ls_dhw = utf8_decode(file_get_contents($lsDhwUrl));
		$matches = array();
		preg_match("/<\!-- content -->.*<\!-- \/content -->/s", $ls_dhw, $matches);
		$view->lastschriftbysofort_dhw = $matches[0];
		
		// customer protection in table 's_core_configuration' set?
		$suCustomerProtection = Shopware()->Db()->fetchOne(
			'SELECT value FROM `s_core_config` WHERE `name` = "sofort_ueberweisung_customer_protection"'
		);
		
		$svCustomerProtection = Shopware()->Db()->fetchOne(
			'SELECT value FROM `s_core_config` WHERE `name` = "vorkassebysofort_customer_protection"'
		);
		
		// set the variables for usage in template
		$view->suCustomerProtection = $suCustomerProtection;
		$view->svCustomerProtection = $svCustomerProtection;
		
		$view->bankAccount = self::getBankAccount($args);
		
		$userSettings = self::getUserSettingsByUserId($view->sUserData['additional']['user']['id']);
		
		$chosenPaymentMethod = self::getChosenPaymentMethod($view->sUserData['additional']['user']['id']);
		$view->chosenPaymentMethod = $chosenPaymentMethod;
		
		// set the "Datenschutzhinweis akzeptiert" according to user's settings
		if($userSettings['sr_dhw_accepted'] == 1) {
			$view->sofortrechnung_dhw_checked = 'checked';
		}
		if($userSettings['sv_dhw_accepted'] == 1) {
			$view->vorkassebysofort_dhw_checked = 'checked';
		}
		if($userSettings['sl_dhw_accepted'] == 1) {
			$view->lastschriftbysofort_dhw_checked = 'checked';
		}
		if($userSettings['su_customer_protection'] == 1) {
			$view->su_customer_protection = 'checked';
		}
		if($userSettings['sv_customer_protection'] == 1) {
			$view->sv_customer_protection = 'checked';
		}
		
		// set the logos according to user's settings set in backend
		
		// fetch the settings for logos and text
		$suBannerOrText = Shopware()->Db()->fetchOne(
			'SELECT value FROM `s_core_config` WHERE `name` = "sofort_su_text_or_banner"'
		);
		$view->suBannerOrText = $suBannerOrText;
		$slBannerOrText = Shopware()->Db()->fetchOne(
			'SELECT value FROM `s_core_config` WHERE `name` = "sofort_sl_text_or_banner"'
		);
		$view->slBannerOrText = $slBannerOrText;
		
		// set variables for iDeal
		$sql = 'SELECT value FROM `s_core_config` WHERE `name` = "ideal_api_key" ';
		
		$configKey = Shopware()->Db()->fetchOne($sql);
		$envVariable = getenv('idealApiUrl');
		$apiUrl = (!empty($envVariable)) ? $envVariable : 'https://www.sofort.com/payment/ideal';
		$SofortLib_iDeal_Banks = new SofortLib_iDeal_Banks($configKey, $apiUrl);
		$SofortLib_iDeal_Banks->sendRequest();
		$bankArray = $SofortLib_iDeal_Banks->getBanks();
		$view->availableBanks = $bankArray;
		// selected bank
		$view->selectedBank = $userSettings['ideal_chosen_bank'];
		
		
		self::saveSofortSettings($args);
		return true;
	}
	
	
	/**
	 * 
	 * make a link to display a condition
	 * @param string $value
	 */
	private function makeConditionHint($value) {
		// ich habe die <a onclick=...>Datenschutzhinweise</a> fr diese Zahlart gelesen
		$condition = Shopware()->Snippets()->getSnippet('sofort_multipay_finish')->get('sofort_multipay_condition');
		$conditionHint = Shopware()->Snippets()->getSnippet('sofort_multipay_finish')->get('sofort_multipay_condition_notice');
		
		$conditionLink = '<a href="javascript:void(0)" onclick=showDWH("'.$value.'")><b>'.$condition.'</b></a>';
		return str_replace('{{conditions}}', $conditionLink, $conditionHint);
	}
	
	
	/**
	 * 
	 * Choose your payment, gives an overview of available payment options 
	 * @param Enlight_Event_EventArgs $args
	 */
	public static function onPostDispatchAccount(Enlight_Event_EventArgs $args) {
		// Payment action is to be chosen via HTML form in this case
		$view = $args->getSubject()->View();
		$doc = $args->getSubject();
		$action = $doc->Request()->getActionName();
		if($action == 'payment') {
			// if api key not valid, deactivate payment by sofort so that no one can select one of these payment method
			$apiKeyNotValid = !self::isApiKeyValid();	// negation
			
			if(!empty($view->sPaymentMeans)) {
				$view->sPaymentMeans = self::deactivatePaymentMeans($view->sPaymentMeans, $apiKeyNotValid);
			} elseif(!empty($view->sPayments)) {
				$view->sPayments = self::deactivatePaymentMeans($view->sPayments, $apiKeyNotValid);
			}
			
			self::sofortPaymentOverview($args, 'sofortpayment.tpl');
		}
	}
	
	
	/**
	 * 
	 * if checkout page is called
	 * @param Enlight_Event_EventArgs $args
	 */
	public static function onPostDispatchCheckout(Enlight_Event_EventArgs $args) {
		// Payment action is to be chosen via Javascript in this case
		$view = $args->getSubject()->View();
		$doc = $args->getSubject();
		$action = $doc->Request()->getActionName();
		if($action == 'payment') {
			// if api key not valid, deactivate payment by sofort so that no one can select one of these payment method
			$apiKeyNotValid = !self::isApiKeyValid();	// negation
			
			if(!empty($view->sPaymentMeans)) {
				$view->sPaymentMeans = self::deactivatePaymentMeans($view->sPaymentMeans, $apiKeyNotValid);
			} elseif(!empty($view->sPayments)) {
				$view->sPayments = self::deactivatePaymentMeans($view->sPayments, $apiKeyNotValid);
			}
			
			self::sofortPaymentOverview($args, 'sofortcheckout.tpl');
		}
	}
	
	
	/**
	 * 
	 * if register page is called
	 * @param Enlight_Event_EventArgs $args
	 */
	public static function onPostDispatchRegister(Enlight_Event_EventArgs $args) {
		// Payment action is to be chosen via HTML form in this case
		$view = $args->getSubject()->View();
		$doc = $args->getSubject();
		$action = $doc->Request()->getActionName();
		// if api key not valid, deactivate payment by sofort so that no one can select one of these payment method
		$apiKeyNotValid = !self::isApiKeyValid();	// negation
		
		if(!empty($view->sPaymentMeans)) {
			$view->sPaymentMeans = self::deactivatePaymentMeans($view->sPaymentMeans, $apiKeyNotValid);
		} elseif(!empty($view->sPayments)) {
			$view->sPayments = self::deactivatePaymentMeans($view->sPayments, $apiKeyNotValid);
		}
		
		self::sofortPaymentOverview($args, 'sofortpayment.tpl');
	}
	
	
	/**
	 * 
	 * is api key not valid?
	 * @return boolean
	 */
	public static function isApiKeyValid() {
		$sql = 'SELECT value FROM s_core_config WHERE name = "sofort_api_key_valid"';
		$apiKeyValid = Shopware()->Db()->fetchOne($sql);
		$apiKeyValid = ($apiKeyValid == 1) ? true : false;
		return $apiKeyValid;
	}
	
	
	/**
	 * if $condition is true, unset all sofort payment methods
	 * @param array $paymentMeans
	 * @param boolean $condition
	 * @return array
	 */
	public static function deactivatePaymentMeans($paymentMeans, $condition) {
		if($condition === true) {
			foreach($paymentMeans as $key => $payment) {
				
				if(preg_match('/[a-z]*_multipay/', $payment['name'])) {
					unset($paymentMeans[$key]);
				}
			}
		}
		
		return $paymentMeans;
	}
	
	
	/**
	 * 
	 * Get the info text from document.sofort.com
	 */
	private function getInfoText() {
		$infoText = Shopware()->Snippets()->getSnippet('sofort_multipay_backend')->get('beta_products');
		$infoText = utf8_decode(@file_get_contents($infoText));
		$matches = array();
		preg_match("/<\!-- content -->.*<\!-- \/content -->/s", $infoText, $matches);
		$infoText = $matches[0];
		
		if(!empty($infoText)) {
			return '<div style="color: #ff0000;">'.$infoText.'<br /></div>';
		}
	}
	
	
	/**
	 * 
	 * Get the module's version
	 * (non-PHPdoc)
	 * @see Shopware_Components_Plugin_Bootstrap::getVersion()
	 */
	public function getVersion(){
		return "1.3.0";
	}
	
	
	/**
	 * 
	 * Display some information about this plugin
	 */
	public function getInfo() {
		
		$productLogoString = '';
		
		foreach($this->products as $product) {
			$productLogoString .= $product['logo'];
			$productLogoString.'&nbsp;';
		}
		
		return array(
			'version' => $this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_version'),
			'autor' => 'SOFORT AG',
			'copyright' => 'SOFORT AG, 2012',
			'label' => 'sofort gateway',
			'source' => '',
			'description' => '
				'.$this->getInfoText().'
				<h2>'.$this->snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_payment_modules').'</h2>
				<p>'.$this->snippets->getSnippet('sofort_multipay')->get('sofort_multipay_plugin_description').'</p>
				<div style="display: inline">
					'.$productLogoString.'
				</div>
			',
			'license' => '',
			'support' => 'https://www.payment-network.com/sue_de/integration/list/88',
			'link' => 'http://www.sofort.com',
			'changes' => '',
			'revision' => '$Revision: 2042 $'
		);
	}
}