<?php
require_once(dirname(__FILE__).'/../../library/sofortLib.php');
require_once(dirname(__FILE__).'/../../library/sofortLib_sofortueberweisung_classic.php');

/**
 *
 * Controller for handling user's setting in the backend
 *
 * $Date: 2012-12-04 11:27:29 +0100 (Tue, 04 Dec 2012) $
 * @version sofort 1.0  $Id: SofortSettings.php 5819 2012-12-04 10:27:29Z dehn $
 * @author SOFORT AG http://www.sofort.com (f.dehn@sofort.com)
 * @package Shopware 3.5.x, sofort.com
 *
 */
class Shopware_Controllers_Backend_SofortSettings extends Enlight_Controller_Action {
	
	private $SofortLib_Multipay = null;
	
	private $statuses = array();
	
	
	/**
	 * 
	 * Initiate this controller
	 */
	public function init() {
		$this->View()->addTemplateDir(dirname(__FILE__).'/../../Templates');
		$config = Shopware()->Plugins()->Frontend()->Sofort()->Config();
		// API
		$this->SofortLib_Multipay = new SofortLib_Multipay($config->sofort_configkey);
		$this->statuses = $this->fetchStatuses();
	}
	
	
	/**
	 * 
	 * Fetch the statuses
	 */
	private function fetchStatuses() {
		return parse_ini_file(dirname(__FILE__).'/../../config/config.ini', true);
	}
	
	
	/**
	 * Set the display of banners in fronted
	 */
	private function bannerSettings() {
		$suBannerOrText = Shopware()->Db()->fetchOne(
			'SELECT value FROM `s_core_config` WHERE `name` = "sofort_su_text_or_banner"'
		);
		
		if ($suBannerOrText == 1) {
			$this->View()->bannerSuSelected = 'selected=selected';
		} elseif ($suBannerOrText == 2) {
			$this->View()->textSuSelected = 'selected=selected';
		}
		// print_r($this->View());die;
		$slBannerOrText = Shopware()->Db()->fetchOne(
			'SELECT value FROM `s_core_config` WHERE `name` = "sofort_sl_text_or_banner"'
		);
		
		if ($slBannerOrText == 1) {
			$this->View()->bannerSlSelected = '"selected=selected"';
		} elseif ($slBannerOrText == 2) {
			$this->View()->textSlSelected = '"selected=selected"';
		}
		
		$showSofortBanner = Shopware()->Db()->fetchOne(
			'SELECT value FROM `s_core_config` WHERE `name` = "show_sofort_banner"'
		);
		
		if ($showSofortBanner == 1) {
			$this->View()->showSofortBanner = 'checked';
		} elseif ($showSofortBanner == 2) {
			$this->View()->showSofortBanner = '';
		}
		
	}
	
	
	/**
	 * Core States Settings
	 */
	private function coreStatesSettings() {
		$sql = 'SELECT id, description FROM `s_core_states` WHERE `group` = "payment"';
		$coreStates = Shopware()->Db()->fetchAll($sql);
		
		$this->View()->coreStates = $coreStates;
		
		$sql = 'SELECT value from `s_core_config` WHERE `name` = "sofort_api_key"';
		$apiKey = Shopware()->Db()->fetchOne($sql);
		$this->View()->apiKey = $apiKey;
		
		$sql = 'SELECT value from `s_core_config` WHERE `name` = "ideal_api_key"';
		$idealApiKey = Shopware()->Db()->fetchOne($sql);
		$this->View()->idealApiKey = $idealApiKey;
		
		$sql = 'SELECT value from `s_core_config` WHERE `name` = "ideal_project_password"';
		$idealProjectPassword = Shopware()->Db()->fetchOne($sql);
		$this->View()->idealProjectPassword = $idealProjectPassword;
		
		$sql = 'SELECT value from `s_core_config` WHERE `name` = "ideal_notification_password"';
		$idealNotificationPassword = Shopware()->Db()->fetchOne($sql);
		$this->View()->idealNotificationPassword = $idealNotificationPassword;
		
		// available algorithms
		$supportedAlgorithms = array('sha1', 'sha256', 'sha512');
		$localSupportedAlgorithms = hash_algos();
		$availableEncryptionAlgorithms = array();
		
		foreach($localSupportedAlgorithms as $localAlgo) {
			if(in_array($localAlgo, $supportedAlgorithms)) {
				$availableEncryptionAlgorithms[] = $localAlgo;
			}
		}
		
		$this->View()->availableEncryptionAlgorithms = $availableEncryptionAlgorithms;
		
		$sql = 'SELECT value from `s_core_config` WHERE `name` = "ideal_encryption_algorithm"';
		$selectedEncryptionAlgorithm = Shopware()->Db()->fetchOne($sql);
		$this->View()->selectedEncryptionAlgorithm = $selectedEncryptionAlgorithm;
		
		$sql = 'SELECT value from `s_core_config` WHERE `name` = "sofort_payment_reason"';
		$paymentReason = Shopware()->Db()->fetchOne($sql);
		$this->View()->paymentReason = $paymentReason;
		
		$sql = 'SELECT value from `s_core_config` WHERE `name` = "sofort_payment_reason2"';
		$paymentReason = Shopware()->Db()->fetchOne($sql);
		$this->View()->paymentReason2 = $paymentReason;
		
		$sql = 'SELECT value from `s_core_config` WHERE `name` = "sofort_ueberweisung_customer_protection"';
		$suCustomerProtection = Shopware()->Db()->fetchOne($sql);
		($suCustomerProtection == 'on') ? $this->View()->suCustomerProtection = 'checked' : $this->View()->suCustomerProtection = '';
		
		$sql = 'SELECT value from `s_core_config` WHERE `name` = "sofort_payment_secret"';
		$paymentSecret = Shopware()->Db()->fetchOne($sql);
		($paymentSecret == '') ? $this->View()->paymentSecret = SofortLib_SofortueberweisungClassic::generatePassword() : $this->View()->paymentSecret = $paymentSecret;
		
		$this->initPaymentStates();
		return true;
	}
	
	
	/**
	 * 
	 * Initiate payment states
	 */
	private function initPaymentStates() {
		foreach($this->statuses as $paymentMethod => $status) {
			switch($paymentMethod) {
				case('sr'):
					foreach($this->statuses['sr'] as $paymentState => $paymentStateDescription) {
						$this->setTemplateVariable($paymentStateDescription);
					}
				break;
				case('su'):
					foreach($this->statuses['su'] as $paymentState => $paymentStateDescription) {
						$this->setTemplateVariable($paymentStateDescription);
					}
				break;
				case('sl'):
					foreach($this->statuses['sl'] as $paymentState => $paymentStateDescription) {
						$this->setTemplateVariable($paymentStateDescription);
					}
				break;
				case('ls'):
					foreach($this->statuses['ls'] as $paymentState => $paymentStateDescription) {
						$this->setTemplateVariable($paymentStateDescription);
					}
				break;
				case('sv'):
					foreach($this->statuses['sv'] as $paymentState => $paymentStateDescription) {
						$this->setTemplateVariable($paymentStateDescription);
					}
				break;
				case('iDeal'):
					foreach($this->statuses['iDeal'] as $paymentState => $paymentStateDescription) {
						$this->setTemplateVariable($paymentStateDescription);
					}
				break;
			}
		}
	}
	
	
	/**
	 * 
	 * Setter for template variable
	 * @param string $paymentState
	 */
	private function setTemplateVariable($paymentState) {
		$sql = 'SELECT value from `s_core_config` WHERE `name` = "'.$paymentState.'"';
		$paymentStateValue = Shopware()->Db()->fetchOne($sql);
		if(!empty($paymentStateValue)) {
			$this->View()->$paymentState = $paymentStateValue;
		}
	}
	
	
	/**
	 * Index Action - Display settings in backend
	 */
	public function indexAction() {
		$suChecked = '';
		$svChecked = '';
		
		if ($this->getCustomerProtection('su') == 'on') {
			$suChecked = 'checked';
		}
		
		if($this->getCustomerProtection('sv') == 'on') {
			$svChecked = 'checked';
		}
		
		$this->View()->sofortueberweisung_customer_protection = $suChecked;
		$this->View()->vorkassebysofort_customer_protection = $svChecked;
		$this->bannerSettings();
		$this->coreStatesSettings();
		$this->View()->loadTemplate('settings.tpl');
		$realPath = realpath('.');
		$plusImage = file_get_contents(dirname(__FILE__).'../../../Templates/Backend/Images/plus.gif');
		$this->View()->plusImage = base64_encode($plusImage);
		$minusImage = file_get_contents(dirname(__FILE__).'../../../Templates/Backend/Images/minus.gif');
		$this->View()->minusImage = base64_encode($minusImage);
		$infoImage = file_get_contents(dirname(__FILE__).'../../../Templates/Backend/Images/info.gif');
		$this->View()->infoImage = base64_encode($infoImage);
		$helpOverImage = file_get_contents(dirname(__FILE__).'../../../Templates/Backend/Images/help_over.png');
		$this->View()->helpOverImage = base64_encode($helpOverImage);
		return true;
	}
	
	
	/**
	 * Save settings for module
	 */
	public function saveAction() {
		
		//throw new Exception(print_r($this->Request()->getParams(), 1));
		foreach($this->statuses as $paymentMethod => $status) {
			switch($paymentMethod) {
				case('sr'):
					foreach($this->statuses['sr'] as $paymentState => $paymentStateDescription) {
						$this->savePaymentStatusSetting($paymentStateDescription);
					}
					break;
				case('su'):
					foreach($this->statuses['su'] as $paymentState => $paymentStateDescription) {
						$this->savePaymentStatusSetting($paymentStateDescription);
					}
					break;
				case('sl'):
					foreach($this->statuses['sl'] as $paymentState => $paymentStateDescription) {
						$this->savePaymentStatusSetting($paymentStateDescription);
					}
					break;
				case('ls'):
					foreach($this->statuses['ls'] as $paymentState => $paymentStateDescription) {
						$this->savePaymentStatusSetting($paymentStateDescription);
					}
					break;
				case('sv'):
					foreach($this->statuses['sv'] as $paymentState => $paymentStateDescription) {
						$this->savePaymentStatusSetting($paymentStateDescription);
					}
					break;
				case('ideal'):
					foreach($this->statuses['ideal'] as $paymentState => $paymentStateDescription) {
						$this->savePaymentStatusSetting($paymentStateDescription);
					}
					break;
			}
		}
		
		$gatewayApiKey = $this->Request()->getParam('gatewayApiKey');
		$sql = 'UPDATE `s_core_config` SET value = ? WHERE `name` = "sofort_api_key"';
		$fields = array(
			$gatewayApiKey,
		);
		Shopware()->Db()->query($sql, $fields);
		
		$idealApiKey = $this->Request()->getParam('idealApiKey');
		$sql = 'UPDATE `s_core_config` SET value = ? WHERE `name` = "ideal_api_key"';
		$fields = array(
			$idealApiKey,
		);
		Shopware()->Db()->query($sql, $fields);
		
		// store information about whether the api key is valid or not
		$gatewayApiKey = $this->Request()->getParam('gatewayApiKey');
		$apiKeyValid = $this->testApiAction($gatewayApiKey);
		$sql = 'UPDATE `s_core_config` SET value = ? WHERE `name` = "sofort_api_key_valid"';
		$fields = array(
			$apiKeyValid,
		);
		Shopware()->Db()->query($sql, $fields);
		
		$paymentSecret = $this->Request()->getParam('paymentSecret');
		$sql = 'UPDATE `s_core_config` SET value = ? WHERE `name` = "sofort_payment_secret"';
		$fields = array(
			$paymentSecret,
		);
		Shopware()->Db()->query($sql, $fields);
		
		$projectPassword = $this->Request()->getParam('idealProjectPassword');
		$sql = 'UPDATE `s_core_config` SET value = ? WHERE `name` = "ideal_project_password"';
		$fields = array(
			$projectPassword,
		);
		Shopware()->Db()->query($sql, $fields);
		
		$notificationPassword = $this->Request()->getParam('idealNotificationPassword');
		$sql = 'UPDATE `s_core_config` SET value = ? WHERE `name` = "ideal_notification_password"';
		$fields = array(
			$notificationPassword,
		);
		Shopware()->Db()->query($sql, $fields);
		
		$idealEncryptionAlgorithm = $this->Request()->getParam('idealEncryptionAlgorithm');
		$sql = 'UPDATE `s_core_config` SET value = ? WHERE `name` = "ideal_encryption_algorithm"';
		$fields = array(
			$idealEncryptionAlgorithm,
		);
		Shopware()->Db()->query($sql, $fields);
		
		$paymentReason = $this->Request()->getParam('paymentReason1');
		$sql = 'UPDATE `s_core_config` SET value = ? WHERE `name` = "sofort_payment_reason"';
		$fields = array(
			$paymentReason,
		);
		Shopware()->Db()->query($sql, $fields);
		
		$paymentReason = $this->Request()->getParam('paymentReason2');
		$sql = 'UPDATE `s_core_config` SET value = ? WHERE `name` = "sofort_payment_reason2"';
		$fields = array(
			$paymentReason,
		);
		Shopware()->Db()->query($sql, $fields);
		
		$showSofortBanner = $this->Request()->getParam('showSofortBanner');
		$sql = 'UPDATE `s_core_config` SET value = ? WHERE `name` = "show_sofort_banner"';
		$fields = array(
			($showSofortBanner == 'on') ? 1 : 0,
		);
		Shopware()->Db()->query($sql, $fields);
		
		$sofortSuTextOrBanner = $this->Request()->getParam('sofort_su_text_or_banner');
		if ($sofortSuTextOrBanner == 'banner_su') {
			$sofortSuTextOrBanner = 1;
		} elseif ($sofortSuTextOrBanner == 'text_su') {
			$sofortSuTextOrBanner = 2;
		}
		
		$sql = 'UPDATE `s_core_config` SET value = ? WHERE `name` = "sofort_su_text_or_banner"';
		$fields = array(
			$sofortSuTextOrBanner,
		);
		Shopware()->Db()->query($sql, $fields);
		
		$slDisplayOption = $this->Request()->getParam('radio_sl');
		
		if ($slDisplayOption == 'banner_sl') {
			$slOption = 1;
		} elseif ($slDisplayOption == 'text_sl') {
			$slOption = 2;
		}
		
		$sql = 'UPDATE `s_core_config` SET value = ? WHERE `name` = "sofort_sl_display"';
		$fields = array(
			$slOption,
		);
		Shopware()->Db()->query($sql, $fields);
		
		
		$suCustomerProtection = $this->Request()->getParam('su_customer_protection');
		$sql = 'UPDATE `s_core_config` SET value = ? WHERE `name` = "sofort_ueberweisung_customer_protection"';
		$fields = array(
			$suCustomerProtection
		);
		Shopware()->Db()->query($sql, $fields);
		
		$this->bannerSettings();
		$this->coreStatesSettings();
		$plusImage = file_get_contents(dirname(__FILE__).'../../../Templates/Backend/Images/plus.gif');
		$this->View()->plusImage = base64_encode($plusImage);
		$minusImage = file_get_contents(dirname(__FILE__).'../../../Templates/Backend/Images/minus.gif');
		$this->View()->minusImage = base64_encode($minusImage);
		$infoImage = file_get_contents(dirname(__FILE__).'../../../Templates/Backend/Images/info.gif');
		$this->View()->infoImage = base64_encode($infoImage);
		$this->View()->loadTemplate("settings.tpl");
		$helpOverImage = file_get_contents(dirname(__FILE__).'../../../Templates/Backend/Images/help_over.png');
		$this->View()->helpOverImage = base64_encode($helpOverImage);
		return true;
	}
	
	
	/**
	 * 
	 * Save payment status
	 * @param string $paymentState
	 */
	private function savePaymentStatusSetting($paymentState) {
		$paymentStateValue = $this->Request()->getParam($paymentState);
		
		if(!empty($paymentStateValue)) {
			$sql = 'UPDATE `s_core_config` SET value = ? WHERE `name` = "'.$paymentState.'"';
			$fields = array(
				$paymentStateValue,
			);
			Shopware()->Db()->query($sql, $fields);
			return true;
		}
		
		return false;
	}
	
	
	/**
	 *
	 * Get customer protection setting for a product
	 * @param String $product
	 */
	private function getCustomerProtection($product) {
		if ($product == 'su') {
			return Shopware()->Db()->FetchOne('SELECT value FROM `s_core_config` WHERE `name` = "sofort_ueberweisung_customer_protection"');
		} elseif ($product == 'sv') {
			return Shopware()->Db()->FetchOne('SELECT value FROM `s_core_config` WHERE `name` = "vorkassebysofort_customer_protection"');
		}
		
		return false;
	}
	
	
	/**
	 * Use TransactionData to test connection and API key against Payment Network
	 */
	public function testApiAction($apiKey = '') {
		$apiKey = ($this->Request()->apiKey) ? $this->Request()->apiKey : $apiKey;
		$this->View()->loadTemplate("empty.tpl");
		// API
		$SofortLib_TransactionData = new SofortLib_TransactionData($apiKey);
		$SofortLib_TransactionData->setTransaction('00000')->sendRequest();
		
		if (!$SofortLib_TransactionData->isError()) {
			echo true;
			return true;
		}
		
		echo false;
		return false;
	}
}