<?php
require_once(dirname(__FILE__).'/../../library/sofortLib.php');
require_once(dirname(__FILE__).'/../../library/sofortLib_ideal_classic.php');
require_once(dirname(__FILE__).'/../Helper/Helper.php');
/**
 *
 * Controller for handling payments by sofort multipay
 *
 * $Date: 2012-11-30 11:37:29 +0100 (Fri, 30 Nov 2012) $
 * @version sofort 1.0  $Id: Sofort.php 5808 2012-11-30 10:37:29Z dehn $
 * @author SOFORT AG http://www.sofort.com (f.dehn@sofort.com)
 * @package Shopware 3.5.x, sofort.com
 *
 */
class Shopware_Controllers_Frontend_Sofort extends Shopware_Controllers_Frontend_Payment {

	/**
	 * reference to sofortLib
	 * @var sofortLib
	 */
	private $SofortLib_Multipay = null;
	
	private $Shopware = null;
	
	private $PnagInvoice = null;
	
	/**
	 * Language Snippets
	 * @var Object
	 */
	private $Snippets = null;
	
	private $testMode = false;
	
	/**
	 * Version
	 * @var string
	 */
	private $version = 'sofort_shopware_1.3.0';
	
	/**
	 * a random unique id used as "token" for payments
	 * @var string
	 */
	private $uniqueId = '';
	
	/**
	 * User's configuration key issued by Payment Network
	 * @var string
	 */
	private $configKey = '';
	
	private $idealConfigKey = '';
	
	/**
	 * Project Password for iDeal Payments
	 */
	private $projectPassword = '';
	
	/**
	 * Algorithm for encryption in iDeal Payments
	 * @var unknown_type
	 */
	private $idealEncryptionAlgorithm = 'sha1';
	
	
	/**
	 * 
	 * Global error codes
	 * @var array
	 */
	private $errorCodes = array();
	
	/**
	 *
	 * payment reason the user set in settings
	 * @var string
	 */
	private $paymentReason = '';
	
	private $paymentReason2 = '';
	
	private $paymentMethodString = '';
	
	private $paymentMethod = array();
	
	private $bankAccount = array();
	
	private $dateFormat = 'd.m.Y H:i:s';
	
	private $ShopwareUpdateHelper = null;
	
	private $statuses = array();
	
	
	/**
	 *
	 * Init this script
	 */
	public function init() {
		
		if(!$this->Shopware) {
			$this->setShopware(Shopware());
		}
		
		$this->ShopwareUpdateHelper = new ShopwareUpdateHelper($this->Shopware);
		
		$this->Snippets = $this->Shopware->Snippets();
		$this->View()->addTemplateDir(dirname(__FILE__).'/../../Templates');
		
		$sql = 'SELECT value FROM `s_core_config` WHERE `name` = "sofort_api_key" ';
		
		if(empty($this->configKey)) {
			$this->configKey = $this->Shopware->Db()->fetchOne($sql);
		}
		
		$sql = 'SELECT value FROM `s_core_config` WHERE `name` = "ideal_api_key" ';
		
		if(empty($this->idealConfigKey)) {
			$this->idealConfigKey = $this->Shopware->Db()->fetchOne($sql);
		}
		
		$sql = 'SELECT value FROM `s_core_config` WHERE `name` = "ideal_encryption_algorithm" ';
		
		if(empty($this->idealEncryptionAlgorithm)) {
			$this->idealEncryptionAlgorithm = $this->Shopware->Db()->fetchOne($sql);
		}
		
		$sql = 'SELECT value FROM `s_core_config` WHERE `name` = "ideal_project_password" ';
		
		if(empty($this->projectPassword)) {
			$this->projectPassword = $this->Shopware->Db()->fetchOne($sql);
		}
		
		$this->SofortLib_Multipay = new SofortLib_Multipay($this->configKey);
		$this->SofortLib_Multipay->setVersion($this->version);
		$secret = $this->Shopware->Plugins()->Frontend()->Sofort()->Config()->sofort_paymentsecret;
		// create a random secret that acts as an identifier for orders
		// prevents abuse of incoming notifications
		$this->uniqueId = md5($secret.'|'.mt_rand());
		
		$sql = 'SELECT value from `s_core_config` WHERE `name` = "sofort_payment_reason"';
		$this->paymentReason = $this->Shopware->Db()->fetchOne($sql);
		$sql = 'SELECT value from `s_core_config` WHERE `name` = "sofort_payment_reason2"';
		$this->paymentReason2 = $this->Shopware->Db()->fetchOne($sql);
		
		$this->statuses = $this->fetchStatuses();
	}
	
	
	/**
	 * 
	 * Fetch the statuses
	 */
	private function fetchStatuses() {
		return parse_ini_file(dirname(__FILE__).'/../../config/config.ini', true);
	}
	
	
	/**
	 * 
	 * Getter for invoice object
	 */
	public function getInvoice() {
		return $this->PnagInvoice;
	}
	
	
	
	/**
	 * 
	 * Setter for test mode
	 * @param boolean $flag
	 */
	public function setTestMode($flag) {
		$this->testMode = $flag;
	}
	
	
	/**
	 * 
	 * Is in test mode (testing purposes)
	 */
	public function isInTestMode() {
		return $this->testMode;
	}
	
	
	/**
	 * 
	 * Setter for Shopware object
	 * @param object $instance
	 */
	public function setShopware(Shopware $instance) {
		$this->Shopware = $instance;
	}
	
	
	/**
	 * 
	 * Setter for config key
	 * @param string $string
	 */
	public function setConfigKey($string) {
		$this->configKey = $string;
	}
	
	
	/**
	 * 
	 * Setter for ideal config key
	 * @param unknown_type $string
	 */
	public function setIdealConfigKey($string) {
		$this->idealConfigKey = $string;
	}
	
	
	/**
	 * 
	 * Setter for payment method
	 * @param string $method
	 */
	public function setPaymentMethod($method = array('name' => 'sofortrechnung_multipay')) {
		$this->paymentMethod = $method;
	}
	
	
	/**
	 * 
	 * Setter for bank account
	 * @param string $account
	 */
	public function setBankAccount($account = array()) {
		$this->bankAccount = $account;
	}
	
	
	/**
	 * 
	 * Getter for Sofortlib_Multipay object
	 */
	public function getSofortObject() {
		return $this->SofortLib_Multipay;
	}
	
	
	/**
	 *
	 * Index controller action
	 * @throws Exception
	 */
	public function indexAction() {
		$Object = null;
		
		switch ($this->getPaymentShortName()) {
			case 'sofortrechnung_multipay':
				$Object = $this->setSofortrechnung();
				break;
			case 'vorkassebysofort_multipay':
				$Object = $this->setVorkassebysofort();
				break;
			case 'sofortueberweisung_multipay':
				$Object = $this->setSofortueberweisung();
				break;
			case 'sofortlastschrift_multipay':
				$Object = $this->setSofortlastschrift();
				break;
			case 'lastschriftbysofort_multipay':
				$Object = $this->setLastschriftbysofort();
				break;
			case 'iDeal':
				$Object = $this->setIdeal();
				break;
			default:
				throw new Exception('Error while processing payment. Not logged in?', 404);
				break;
		}
		
		return $Object;
	}
	
	
	/**
	 * 
	 * Initiate Rechnung by SOFORT
	 */
	private function setSofortrechnung() {
		$this->paymentMethodString = 'sr';
		$basket = $this->getBasket();
		$user = $this->getUser();
		$billingAddress = $user['billingaddress'];
		$shippingAddress = $user['shippingaddress'];
		$billingLastname = $billingAddress['lastname'];
		$shippingLastname = $shippingAddress['lastname'];
		
		if($billingAddress['salutation'] === 'mr' ) {
			$billingSalutation = 2;
		} elseif($billingAddress['salutation'] === 'ms' ) {
			$billingSalutation = 3;
		}
		
		if($shippingAddress['salutation'] === 'mr' ) {
			$shippingSalutation = 2;
		} elseif($shippingAddress['salutation'] === 'ms' ) {
			$shippingSalutation = 3;
		}
		
		$countryInvoice = $user['additional']['country']['countryiso'];
		$countryShipping = (isset($user['additional']['countryShipping']['countryiso'])) ? $user['additional']['countryShipping']['countryiso'] : $countryInvoice;
		$this->PnagInvoice = new PnagInvoice($this->configKey, '');
		$this->PnagInvoice->setVersion($this->version);
		$this->PnagInvoice->setCustomerId($billingAddress['id']);
		$company = (isset($billingAddress['company'])) ? $billingAddress['company'] : '';
		$department = (!empty($company)) ? $billingAddress['department'] : '';
		
		if(!empty($company)) {
			$vatId = isset($billingAddress['ustid']) ? $billingAddress['ustid'] : '';
			$this->PnagInvoice->setDebitorVatNumber($vatId);
		}
		
		$this->PnagInvoice->addInvoiceAddress(utf8_encode($billingAddress['firstname']), utf8_encode($billingLastname), utf8_encode($billingAddress['street']), $billingAddress['streetnumber'], $billingAddress['zipcode'], utf8_encode($billingAddress['city']), $billingSalutation, $countryInvoice, $department, '',utf8_encode($company));
		$company = (isset($shippingAddress['company'])) ? $shippingAddress['company'] : '';
		$this->PnagInvoice->addShippingAddress(utf8_encode($shippingAddress['firstname']), utf8_encode($shippingLastname), utf8_encode($shippingAddress['street']), utf8_encode($shippingAddress['streetnumber']), utf8_encode($shippingAddress['zipcode']), utf8_encode($shippingAddress['city']), $shippingSalutation, $countryShipping, utf8_encode($company));
		$this->PnagInvoice->setEmailCustomer($user['additional']['user']['email']);
		
		foreach ($basket['content'] as $article) {
			$surchargeNumber = $this->getSurchargeNumber();
			$discountNumber = $this->getDiscountNumber();
			$paymentDiscountNumber = $this->getPaymentDiscountNumber();
			$paymentSurchargeNumber = $this->getPaymentSurchargeNumber();
			$productType = 0;
			
			if($article['ordernumber'] == $surchargeNumber) {
				$article['articleID'] = 'swsurcharge';
				$productType = 2;
			} elseif($article['ordernumber'] == $discountNumber) {
				$article['articleID'] = 'swdiscount';
				$productType = 2;
			} elseif($article['ordernumber'] == $paymentDiscountNumber) {
				$article['articleID'] = 'swpayment';
				$productType = 2;
			} elseif($article['ordernumber'] == $paymentSurchargeNumber) {
				$article['articleID'] = 'swpaymentabs';
				$productType = 2;
			}
			
			// Get the real name
			$realArticleName = $this->getRealArticleName($article, array('Gutschein'));
			
			// notice the html encoded article names!
			if (strlen($realArticleName) != strlen($article['articlename'])) {
				$realArticleDescription = trim(substr(html_entity_decode($article['articlename']), strlen($realArticleName)));
			} else {
				$realArticleDescription = '';
			}
			
			// add "normal" articles to invoice
			$this->PnagInvoice->addItemToInvoice($article['articleID'].'|'.$article['ordernumber'], utf8_encode($article['ordernumber']), utf8_encode($realArticleName), $article['priceNumeric'], $productType, utf8_encode($realArticleDescription), $article['quantity'], $article['taxPercent']);
		}
		
		if ($basket['sShippingcosts'] > 0) {
			$shippingSnippet = $this->Snippets->getSnippet("sofort_multipay_checkout")->get("shipping_costs");
			$shippingSnippetShort = 'shpmntvk';
			// get dispatcher ID
			$dispatchId = (!empty($this->Shopware->Session()->sDispatch)) ? $this->Shopware->Session()->sDispatch : 0;
			// add shipping costs to invoice
			$this->PnagInvoice->addItemToInvoice($shippingSnippetShort.'|'.$shippingSnippetShort, $shippingSnippetShort, $shippingSnippet, $basket['sShippingcosts'], 1, $this->getDispatcherName($dispatchId), 1, $basket['sShippingcostsTax']);
		}
		
		$this->PnagInvoice->setSuccessUrl($this->makeSuccessUrl());
		$this->PnagInvoice->setTimeoutUrl($this->makeTimeoutUrl());
		$this->PnagInvoice->setAbortUrl($this->makeAbortUrl());
		$this->PnagInvoice->setNotificationUrl($this->makeNotificationUrl());
		$this->placeOrder();
		// TODO: reason2
		$this->PnagInvoice->setReason($this->paymentReason, $this->paymentReason2);
		// $this->PnagInvoice->setOrderId($orderId);
		$this->PnagInvoice->checkout();
		
		if ($this->isInTestMode()) {
			return $this;
		}
		
		if ($this->PnagInvoice->isError()) {
			$errorMsg = $this->PnagInvoice->getError();
			$allErrors = $this->PnagInvoice->getErrors();
			$errorCodes = $this->mergeErrors($allErrors);
			$this->redirect($this->makeAbortUrl($errorMsg, $errorCodes));
		} else {
			// get the transactionId returned from API
			$transactionId = $this->PnagInvoice->getTransactionId();
			// set the transactionId to table 'sofort_orders'
			$this->setOrderTransaction($transactionId, $this->uniqueId);
			// fetch the pending state expression for the actual payment method
			$pendingState = $this->getPendingState($this->paymentMethod['name']);
			$orderId = $this->doSaveOrder($transactionId, $this->paymentMethod['name'], $pendingState);
			$this->redirect($this->PnagInvoice->getPaymentUrl());
			return true;
		}
	}
	
	
	/**
	 * 
	 * Initiate Vorkasse by SOFORT
	 */
	private function setVorkassebysofort() {
		$this->paymentMethodString = 'sv';
		$account = empty($this->bankAccount) ? $this->getBankAccount() : $this->bankAccount;
		// get customer protection
		$svCustomerProtection = $this->Shopware->Db()->fetchOne('SELECT value FROM `s_core_config` WHERE `name` = "vorkassebysofort_customer_protection"');
		
		if ($svCustomerProtection == 'on') {
			$this->SofortLib_Multipay->setSofortvorkasseCustomerprotection();
		} else {
			$this->SofortLib_Multipay->setSofortvorkasse();
		}
		
		$user = $this->getUser();
		$this->SofortLib_Multipay->setAmount($this->getAmount());
		$this->SofortLib_Multipay->setEmailCustomer($user['additional']['user']['email']);
		$this->SofortLib_Multipay->setSuccessUrl($this->makeSuccessUrl());
		$this->SofortLib_Multipay->setTimeoutUrl($this->makeTimeoutUrl());
		$this->SofortLib_Multipay->setAbortUrl($this->makeAbortUrl());
		$this->SofortLib_Multipay->setNotificationUrl($this->makeNotificationUrl());
		$this->placeOrder();
		// $this->SofortLib_Multipay->setReason($this->Snippets->getSnippet("sofort_multipay_bootstrap")->get("ordernumber").' '.$orderId, $this->paymentReason);
		
		// if this is a 'test transaction', add a sender account
		if(getenv('test_sv') == true) {
			$this->SofortLib_Multipay->setSenderAccount('00000', '12345', 'Tester Testaccount');
		}
		
		$this->SofortLib_Multipay->sendRequest();
		
		if($this->isInTestMode()) {
			return $this;
		}
		
		if($this->SofortLib_Multipay->isError()) {
			$errorMsg = $this->SofortLib_Multipay->getError();
			$allErrors = $this->SofortLib_Multipay->getErrors();
			$errorCodes = array();
			$errorCodes = $this->mergeErrors($allErrors);
			
			// in case of error, redirect to error page
			$this->redirect($this->makeAbortUrl($errorMsg, $errorCodes));
		} else {
			$transactionId = $this->SofortLib_Multipay->getTransactionId();
			$this->setOrderTransaction($transactionId, $this->uniqueId);
			$pendingState = $this->getPendingState($this->paymentMethod['name']);
			$orderId = $this->doSaveOrder($transactionId, $this->paymentMethod['name'], $pendingState);
			$this->redirect($this->SofortLib_Multipay->getPaymentUrl());
			return true;
		}
	}
	
	
	/**
	 * 
	 * initiate sofortberweisung
	 */
	private function setSofortueberweisung() {
		$this->paymentMethodString = 'su';
		$basket = $this->getBasket();
		$suCustomerProtection = $this->Shopware->Db()->fetchOne('SELECT value FROM `s_core_config` WHERE `name` = "sofort_ueberweisung_customer_protection"');
		
		if ($suCustomerProtection == 'on') {
			$this->SofortLib_Multipay->setSofortueberweisungCustomerprotection();
		} else {
			$this->SofortLib_Multipay->setSofortueberweisung();
		}
		
		$cartIdString = $this->Snippets->getSnippet("sofort_multipay_checkout")->get("cart_id");
		$this->SofortLib_Multipay->setAmount($this->getAmount());
		$this->SofortLib_Multipay->setSuccessUrl($this->makeSuccessUrl());
		$this->SofortLib_Multipay->setTimeoutUrl($this->makeTimeoutUrl());
		$this->SofortLib_Multipay->setAbortUrl($this->makeAbortUrl());
		$this->SofortLib_Multipay->setNotificationUrl($this->makeNotificationUrl());
		$this->placeOrder();
		// TODO: reason2
		$this->SofortLib_Multipay->setReason($this->paymentReason, $this->paymentReason2);
		// $this->SofortLib_Multipay->setReason($this->Snippets->getSnippet("sofort_multipay_bootstrap")->get("ordernumber").' '.$orderId, $this->paymentReason);
		$this->SofortLib_Multipay->sendRequest();
		
		if($this->isInTestMode()) {
			return $this;
		}
		
		if($this->SofortLib_Multipay->isError()) {
			$errorMsg = $this->SofortLib_Multipay->getError();
			$allErrors = $this->SofortLib_Multipay->getErrors();
			$i = 0;
			$errorCodes = array();
			$errorCodes = $this->mergeErrors($allErrors);
			
			$this->redirect($this->makeAbortUrl($errorMsg, $errorCodes));
		} else {
			$transactionId = $this->SofortLib_Multipay->getTransactionId();
			$this->setOrderTransaction($transactionId, $this->uniqueId);
			$pendingState = $this->getPendingState($this->paymentMethod['name']);
			$orderId = $this->doSaveOrder($transactionId, $this->paymentMethod['name'], $pendingState);
			$this->redirect($this->SofortLib_Multipay->getPaymentUrl());
			return true;
		}
	}
	
	
	/**
	 *
	 * initiate sofortlastschrift
	 */
	private function setSofortlastschrift() {
		$this->paymentMethodString = 'sl';
		$account = empty($this->bankAccount) ? $this->getBankAccount() : $this->bankAccount;
		$basket = $this->getBasket();
		$this->SofortLib_Multipay->setSofortlastschrift();
		$cartIdString = $this->Snippets->getSnippet("sofort_multipay_checkout")->get("cart_id");
		$this->SofortLib_Multipay->setAmount($this->getAmount());
		$user = $this->getUser();
		$user = $user['billingaddress'];
		$this->SofortLib_Multipay->setSuccessUrl($this->makeSuccessUrl());
		$this->SofortLib_Multipay->setTimeoutUrl($this->makeTimeoutUrl());
		$this->SofortLib_Multipay->setAbortUrl($this->makeAbortUrl());
		$this->SofortLib_Multipay->setNotificationUrl($this->makeNotificationUrl());
		$salutation = 2;
		
		if ($user['salutation'] == 'ms') {
			$salutation = 3;
		}
		$countryInvoice = (isset($user['additional']['country']['countryiso'])) ? $user['additional']['country']['countryiso'] : '';
		$this->SofortLib_Multipay->setSofortlastschriftAddress(utf8_encode($user['firstname']), utf8_encode($user['lastname']), utf8_encode($user['street']), utf8_encode($user['streetnumber']), $user['zipcode'], utf8_encode($user['city']), $salutation, $countryInvoice);
		$this->placeOrder();
		//TODO reason2
		$this->SofortLib_Multipay->setReason($this->paymentReason, $this->paymentReason2);
		// $this->SofortLib_Multipay->setReason($this->Snippets->getSnippet("sofort_multipay_bootstrap")->get("ordernumber").' '.$orderId, $this->paymentReason);
		$this->SofortLib_Multipay->sendRequest();
		
		if ($this->isInTestMode()) {
			return $this;
		}
		
		if ($this->SofortLib_Multipay->isError()) {
			$errorMsg = $this->SofortLib_Multipay->getError();
			$allErrors = $this->SofortLib_Multipay->getErrors();
			$errorCodes = array();
			$errorCodes = $this->mergeErrors($allErrors);
			
			$this->redirect($this->makeAbortUrl($errorMsg, $errorCodes));
		} else {
			$transactionId = $this->SofortLib_Multipay->getTransactionId();
			$this->setOrderTransaction($transactionId, $this->uniqueId);
			$pendingState = $this->getPendingState($this->paymentMethod['name']);
			$orderId = $this->doSaveOrder($transactionId, $this->paymentMethod['name'], $pendingState);
			$this->redirect($this->SofortLib_Multipay->getPaymentUrl());
			return true;
		}
	}
	
	
	/**
	 *
	 * initiate Lastschrift by SOFORT
	 */
	private function setLastschriftbysofort() {
		$this->paymentMethodString = 'ls';
		$account = empty($this->bankAccount) ? $this->getBankAccount() : $this->bankAccount;
		$basket = $this->getBasket();
		$cartIdString = $this->Snippets->getSnippet("sofort_multipay_checkout")->get("cart_id");
		
		$this->SofortLib_Multipay->setLastschrift();
		$this->SofortLib_Multipay->setSenderAccount($account['ls_bank_code'], $account['ls_account_number'], utf8_encode($account['ls_holder']));
		$this->SofortLib_Multipay->setAmount($this->getAmount());
		$this->SofortLib_Multipay->setSuccessUrl($this->makeSuccessUrl());
		$this->SofortLib_Multipay->setTimeoutUrl($this->makeTimeoutUrl());
		$this->SofortLib_Multipay->setAbortUrl($this->makeAbortUrl());
		$this->SofortLib_Multipay->setNotificationUrl($this->makeNotificationUrl());

		$user = $this->getUser();
		$user = $user['billingaddress'];
		$salutation = 2;
		
		if($user['salutation'] == 'ms') {
			$salutation = 3;
		}
		
		$countryInvoice = (isset($user['additional']['country']['countryiso'])) ? $user['additional']['country']['countryiso'] : '';
		$this->SofortLib_Multipay->setLastschriftAddress(utf8_encode($user['firstname']), utf8_encode($user['lastname']), utf8_encode($user['street']), utf8_encode($user['streetnumber']), $user['zipcode'], utf8_encode($user['city']), $salutation, $countryInvoice);
		$this->placeOrder();
		// TODO: reason2
		$this->SofortLib_Multipay->setReason($this->paymentReason, $this->paymentReason2);
		// $this->SofortLib_Multipay->setReason($this->Snippets->getSnippet("sofort_multipay_bootstrap")->get("ordernumber").' '.$orderId, $this->paymentReason);
		$this->SofortLib_Multipay->sendRequest();
		
		if ($this->isInTestMode()) {
			return $this;
		}
		
		if ($this->SofortLib_Multipay->isError()) {
			$errorMsg = $this->SofortLib_Multipay->getError();
			$allErrors = $this->SofortLib_Multipay->getErrors();
			$errorCodes = array();
			$i = 0;
			
			// make some error message like errors[]=8054,8033
			foreach ($allErrors as $error) {
				foreach ($error as $key => $value) {
					if ($key == 'code') {
						array_push($errorCodes, $value);
					}
				}
				
				$i++;
			}
			
			$this->redirect($this->makeAbortUrl($errorMsg, $errorCodes));
		} else {
			$transactionId = $this->SofortLib_Multipay->getTransactionId();
			$pendingState = $this->getPendingState($this->paymentMethod['name']);
			$orderId = $this->doSaveOrder($transactionId, $this->paymentMethod['name'], $pendingState);
			$this->setOrderTransaction($transactionId, $this->uniqueId);
			
			$this->redirect($this->SofortLib_Multipay->getPaymentUrl());
			return true;
		}
	}
	
	/**
	 * 
	 * initiate iDeal payment
	 */
	private function setIdeal() {
		$SofortLib_iDealClassic = new SofortLib_iDealClassic($this->idealConfigKey, $this->projectPassword, $this->idealEncryptionAlgorithm);
		$user = $this->getUser();
		$billingAddress = $user['billingaddress'];
		$billingFirstname = $billingAddress['firstname'];
		$billingLastname = $billingAddress['lastname'];
		$SofortLib_iDealClassic->setAmount($this->getAmount(), 'EUR');
		$SofortLib_iDealClassic->setSenderHolder($billingFirstname.' '.$billingLastname);
		$bankAccount = $this->getIdealBankAccount();
		$SofortLib_iDealClassic->setSenderBankCode($bankAccount);
		$SofortLib_iDealClassic->setSenderCountryId('NL');
		$this->placeOrder();
		// TODO: reason2
		$SofortLib_iDealClassic->addUserVariable($this->uniqueId);
		$pendingState = $this->getPendingState($this->paymentMethod['name']);
		$orderId = $this->doSaveOrder($this->uniqueId, $this->paymentMethod['name'], $pendingState);
		$SofortLib_iDealClassic->setReason('iDeal', $orderId);
		$paymentUrl = $SofortLib_iDealClassic->getPaymentUrl();
		$this->redirect($paymentUrl);
		return $SofortLib_iDealClassic;
	}
	
	
	/**
	 * 
	 * initiate 'sofort_orders' table
	 */
	private function placeOrder() {
		
		if(empty($this->paymentMethod)) {
			$this->paymentMethod = $this->ShopwareUpdateHelper->getPaymentDetails($this->getPaymentShortName());
		}
		
		// save in own table
		$this->insertIntoSofortOrders($this->paymentMethod['name'], $this->paymentMethod['description'], $this->uniqueId);
	}
	
	
	/**
	 * 
	 * Merge all occurred errors for easier display
	 * @param array $errors
	 */
	private function mergeErrors($errors) {
		$i = 0;
		$errorCodes = array();
		// make some error message like errors[]=8054,8033
		foreach($errors as $error) {
			foreach($error as $key => $value) {
				if($key == 'code') {
					array_push($errorCodes, $value);
				}
			}
			
			$i++;
		}
		
		return $errorCodes;
	}
	
	
	/**
	 * 
	 * Wrapper for Payment classes method 'saveOrder'
	 * @param string $transactionId
	 * @param string $paymentMethod
	 * @param int $pendingState
	 */
	private function doSaveOrder($transactionId, $paymentMethod, $pendingState, $sendMail = 1) {
		$fields = array(
			$transactionId, 
			$this->uniqueId, 
			$pendingState,
			$this->paymentMethod['name'],
		);
		$orderId = $this->saveOrder($transactionId, $this->uniqueId, $pendingState, $sendMail);
		$this->updateOrderNumber($orderId, $transactionId);
		return $orderId;
	}
	
	
	/**
	 * 
	 * insert into sofort_orders
	 * @param string $paymentMethod
	 * @param string $paymentDescription
	 * @param string $secret
	 */
	private function insertIntoSofortOrders($paymentMethod, $paymentDescription, $secret) {
		$sql = 'INSERT INTO `sofort_orders` (`paymentMethod`, `paymentDescription`, `transactionId`, `secret`, `paymentStatus`) 
				VALUES (?, ?, "empty", ?, "pending");';
		$fields = array(
			$paymentMethod,
			$paymentDescription,
			$secret,
		);
		return $this->Shopware->Db()->query($sql, $fields);
	}	
	
	
	/**
	 * 
	 * Getter for the real article name
	 * @param string $article
	 * @param array $excludes
	 */
	private function getRealArticleName($article, $excludes = array()) {
		if(in_array($article['articlename'] , $excludes)) {
			return $article['articlename'];
		}
		$realArticleName = $this->getArticleName($article['articleID']);
		
		// only empty, if article is shipping costs or sth. like this
		if(empty($realArticleName)) {
			$realArticleName = $article['articlename'];
		}
		
		$realArticleName = html_entity_decode($realArticleName);
		return $realArticleName;
	}
	
	
	/**
	 * 
	 * Getter for voucher description
	 * @param int $voucherId
	 */
	private function getVoucherDescription($voucherId) {
		$sql = 'SELECT description FROM s_emarketing_vouchers WHERE id = ?';
		return $this->Shopware->Db()->fetchOne($sql, array($articleId));
	}
	
	
	/**
	 * 
	 * Getter for article name
	 * @param int $articleId
	 */
	private function getArticleName($articleId) {
		$sql = 'SELECT name FROM s_articles WHERE id = ?';
		return $this->Shopware->Db()->fetchOne($sql, array($articleId));
	}
	
	
	/**
	 * 
	 * Getter for dispatcher name
	 * @param int $dispatchId
	 */
	private function getDispatcherName($dispatchId) {
		if($dispatchId == 0) return '';
		$sql = 'SELECT name FROM s_premium_dispatch WHERE id = ?';
		return $this->Shopware->Db()->fetchOne($sql, array($dispatchId));
	}
	
	
	/**
	 * 
	 * Getter for surcharge number
	 */
	private function getSurchargeNumber() {
		$sql = "SELECT value FROM s_core_config WHERE name='sSURCHARGENUMBER'";
		return $this->Shopware->Db()->fetchOne($sql);
	}
	
	
	/**
	 * 
	 * Getter for discount number
	 */
	private function getDiscountNumber() {
		$sql = "SELECT value FROM s_core_config WHERE name='sDISCOUNTNUMBER'";
		return $this->Shopware->Db()->fetchOne($sql);
	}
	
	
	/**
	 * 
	 * Getter for payment discount number
	 */
	private function getPaymentDiscountNumber() {
		$sql = "SELECT value FROM s_core_config WHERE name='sPAYMENTSURCHARGENUMBER'";
		return $this->Shopware->Db()->fetchOne($sql);
	}
	
	
	/**
	 * 
	 * Getter for payment surcharge number
	 */
	private function getPaymentSurchargeNumber() {
		$sql = "SELECT value FROM s_core_config WHERE name='sPAYMENTSURCHARGEABSOLUTENUMBER'";
		return $this->Shopware->Db()->fetchOne($sql);
	}
	
	
	/**
	 * 
	 * Setter for order transaction, iDeal
	 * @param string $transactionId
	 * @param string $uniqueId
	 */
	private function setOrderTransactionForIdeal($transactionId, $uniqueId) {
		$sql = 'UPDATE `s_order` SET transactionId = ? WHERE temporaryID = ?';
		$fields = array(
			$transactionId,
			$uniqueId,
		);
		return $this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * Setter for order transaction
	 * @param string $transactionId
	 * @param string $uniqueId
	 */
	private function setOrderTransaction($transactionId, $uniqueId) {
		$sql = 'UPDATE `sofort_orders` SET transactionId = ? WHERE secret = ?';
		$fields = array(
			$transactionId,
			$uniqueId,
		);
		return $this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 *
	 * Assembles the success URL
	 * @return URL
	 */
	private function makeSuccessUrl() {
		$config = array(
			'action' => 'end',
			'transactionId' => '-TRANSACTION-',
			'unique' => $this->uniqueId,
			'sofortGateway' => true,
			'forceSecure' => true,
		);
		return $this->Front()->Router()->assemble($config);
	}
	
	
	/**
	 *
	 * Assembles the success URL
	 * @return URL
	 */
	private function makeTimeoutUrl() {
		$config = array(
			'action' => 'timeout',
			'transactionId' => '-TRANSACTION-',
			'unique' => $this->uniqueId,
			'forceSecure' => true,
		);
		return $this->Front()->Router()->assemble($config);
	}
	
	
	/**
	 *
	 * Assembles the abort URL
	 * should direct to controller "checkout"...?
	 * @return URL
	 */
	private function makeAbortUrl($message = '', $errorCodes = array()) {
		$config = array(
			'action' => 'cancel',
			'unique' => $this->uniqueId,
			'transactionId' => '-TRANSACTION-',
			'forceSecure' => true,
		);
		
		if ($message != '') {
			$config['message'] = $message;
		}
		
		if (is_array($errorCodes)) {
			$config['errors'] = $errorCodes;
		}
		
		$abortUrl = $this->Front()->Router()->assemble($config);
		return $abortUrl;
	}
	
	
	/**
	 * Assembles the notification URL for notifications coming from sofort.com
	 * @return URL
	 */
	private function makeNotificationUrl() {
		$config = array(
			'controller' => 'sofort_notification',
			'unique' => $this->uniqueId,
		);
		$notificationUrl = $this->Front()->Router()->assemble($config);
		return $notificationUrl;
	}
	
	
	/**
	 *
	 * Fetches the payment secret from DB
	 * @param unknown_type $transactionId
	 */
	private function getPaymentSecret($transactionId) {
		$sql = 'SELECT secret FROM `sofort_orders` WHERE `transactionId` = ? AND paymentMethod != "iDeal"';
		$fields = array(
			$transactionId,
		);
		return $this->Shopware->Db()->fetchOne($sql, $fields);
	}
	
	
	/**
	 * 
	 * Setter for payment status
	 * @param strign $transactionId
	 * @param string $status
	 */
	private function setPaymentStatus($transactionId, $status) {
		$sql = 'UPDATE `sofort_orders` SET paymentStatus = ? WHERE `transactionId` = ?';
		$fields = array(
			$status,
			$transactionId,
		);
		Shopware()->Db()->query($sql, $fields);
		return true;
	}
	
	
	/**
	 *
	 * Fetch bank account for sofortlastschrift
	 * @param Enlight_Event_EventArgs $args
	 * @return Array
	 */
	private function getBankAccount() {
		$userData = $this->getUser();
		$userId = $userData['billingaddress']['id'];
		$sql = 'SELECT `ls_account_number`, `ls_bank_code`, `ls_holder` FROM `sofort_user_settings` WHERE userID = ?';
		$fields = array(
			$userId,
		);
		$bankAccount = $this->Shopware->Db()->fetchAll($sql, $fields);
		
		if (!empty($bankAccount)) {
			return $bankAccount[0];
		}
		
		return array();
	}
	
	
	/**
	 *
	 * Fetch bank account for sofortlastschrift
	 * @param Enlight_Event_EventArgs $args
	 * @return Array
	 */
	private function getIdealBankAccount() {
		$userData = $this->getUser();
		$userId = $userData['billingaddress']['id'];
		$sql = 'SELECT `ideal_chosen_bank` FROM `sofort_user_settings` WHERE userID = ?';
		$fields = array(
			$userId,
		);
		return $bankAccount = $this->Shopware->Db()->fetchOne($sql, $fields);
	}
	
	
	/**
	 * 
	 * Getter for orderId
	 * @param string $transactionId
	 */
	private function getOrderId($transactionId) {
		$sql = 'SELECT `ordernumber` FROM `s_order` WHERE `transactionID` = ?';
		$fields = array(
			$transactionId,
		);
		$orderId = $this->Shopware->DB()->fetchOne($sql, $fields);
		if(!empty($orderId)) {
			//throw new Exception('Error occured in class '.__CLASS__);
			return $orderId;
		}
		return false;
	}
	
	
	/**
	 * 
	 * Getter for temporary orderId
	 * @param string $transactionId
	 */
	private function getOrderTemporaryId($transactionId) {
		$sql = 'SELECT `temporaryID` FROM `s_order` WHERE `transactionID` = ?';
		$fields = array(
			$transactionId,
		);
		$temporaryId = $this->Shopware->DB()->fetchOne($sql, $fields);
		return $temporaryId;
	}
	
	
	/**
	 * 
	 * Getter for pending state
	 * @param string $paymentMethod
	 * @throws Exception
	 */
	private function getPendingState($paymentMethod) {
		$pendingState = '';
		
		switch ($paymentMethod) {
			case 'sofortrechnung_multipay':
				$pendingState = $this->statuses['sr']['pending'];
				break;
			case 'vorkassebysofort_multipay':
				$pendingState = $this->statuses['sv']['pending'];
				break;
			case 'sofortueberweisung_multipay':
				$pendingState = $this->statuses['su']['pending'];
				break;
			case 'sofortlastschrift_multipay':
				$pendingState = $this->statuses['sl']['pending'];
				break;
			case 'lastschriftbysofort_multipay':
				$pendingState = $this->statuses['ls']['pending'];
				break;
			case 'iDeal':
				$pendingState = $this->statuses['iDeal']['pending'];
				break;
			default:
				throw new Exception('Error while processing payment. Not logged in?', 404);
				break;
		}
		$sql = 'SELECT `value` FROM `s_core_config` WHERE `name` = "'.$pendingState.'"';
		return $this->Shopware->Db()->fetchOne($sql);
	}
	
	
	/**
	 * 
	 * Getter for canceled state 
	 * @param string $paymentMethod
	 * @throws Exception
	 */
	private function getCanceledState($paymentMethod) {
		$canceledState = '';
		
		switch ($paymentMethod) {
			case 'sofortrechnung_multipay':
				$canceledState = $this->statuses['sr']['canceled'];
				break;
			case 'vorkassebysofort_multipay':
				$canceledState = $this->statuses['sv']['canceled'];
				break;
			case 'sofortueberweisung_multipay':
				$canceledState = $this->statuses['su']['canceled'];
				break;
			case 'sofortlastschrift_multipay':
				$canceledState = $this->statuses['sl']['canceled'];
				break;
			case 'lastschriftbysofort_multipay':
				$canceledState = $this->statuses['ls']['canceled'];
				break;
			case 'iDeal':
				$canceledState = $this->statuses['iDeal']['canceled'];
				break;
			default:
				throw new Exception('Error while processing payment. Not logged in?', 404);
				break;
		}
		$sql = 'SELECT `value` FROM `s_core_config` WHERE `name` = "'.$canceledState.'"';
		return $this->Shopware->Db()->fetchOne($sql);
	}
	
	
	/**
	 * 
	 * Update the order number
	 * @param string $orderNumber
	 * @param string $transactionId
	 */
	private function updateOrderNumber($orderNumber, $transactionId) {
		$sql = 'UPDATE s_order SET ordernumber = "'.$orderNumber.'" WHERE transactionID = "'.$transactionId.'"';
		$this->Shopware->Db()->query($sql);
	}
	
	
	/**
	 * 
	 * Initiate the order tables and time line
	 * @param string $transactionId
	 * @throws Exception
	 */
	private function initOrderTablesAndTimeLine($transactionId) {
		$paymentDetails = $this->ShopwareUpdateHelper->getPaymentDetails($this->paymentMethod);
		$orderId = $this->getOrderId($transactionId);
		$basket = $this->getBasket();
		$fields = array(
			$transactionId,
			$orderId,
			$this->paymentMethod,
			$basket
		);
		$this->ShopwareUpdateHelper->initOrderTablesAndTimeLine($transactionId, $orderId, $this->paymentMethod, $basket);
	}
	
	
	/**
	 * 
	 * Set sofort_status to canceled
	 * @param string $transactionId
	 */
	private function setSofortStatusCanceled($transactionId) {
		$sql = 'INSERT INTO sofort_status
			(sofort_product_id, status_id, status, status_reason, invoice_status, invoice_objection)
			VALUES ((SELECT id FROM `sofort_products` WHERE transactionId = ?), "", "canceled", "canceled", "canceled", "")
		';
		$fields = array(
			$transactionId,
		);
		$this->Shopware->Db()->query($sql, $fields);
		$time = date($this->dateFormat, time());
		$cancelString = $this->Snippets->getSnippet('sofort_multipay_cancel')->get('order_cancel');
		$cancelString = $time.' '.$cancelString;
		$sql = 'UPDATE `sofort_products` SET `comment` = ?, `date_modified` = NOW() WHERE `transactionId` = ?';
		$fields = array(
			$cancelString,
			$transactionId,
		);
		$this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * Set the order's comment according to the message sent by notification
	 * @param string $transactionId
	 * @param string $comment
	 * @param boolean $overwrite overwrite the old comments, set the current one as new
	 */
	private function setOrderComment($transactionId, $comment, $overwrite = false) {
		$oldComment = ''; $newComment = '';
		
		if (!$overwrite) { // if false, old comment is not being overwritten
			$sql = 'SELECT `comment` FROM `s_order` WHERE `transactionID` = ?';
			$fields = array(
				$transactionId,
			);
			$oldComment = $this->Shopware->Db()->FetchOne($sql, $fields);
		}
		
		$actDate = date($this->dateFormat, time());
		// add new comment at the end of the old one
		$newComment = (empty($oldComment)) ? $actDate.' - '.$comment: $oldComment.'<br />'.$actDate.' - '.$comment;
		$newComment = str_replace('{{paymentMethodStr}}', $this->paymentMethodString, $newComment);
		$newComment = str_replace('{{tId}}', '', $newComment);
		$newComment = str_replace('{{time}}', '', $newComment);
		
		// update the order comment
		$sql = 'UPDATE `s_order` SET `comment` = ? WHERE `transactionID` = ?';
		$fields = array(
			$newComment,
			$transactionId,
		);
		$this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * Does an order exist
	 * @param string $transactionId
	 */
	private function orderExists($transactionId) {
		$sql = 'SELECT order_id FROM `sofort_products` WHERE transactionId = ? AND order_id != 0';
		$fields = array(
			$transactionId,
		);
		return $this->Shopware->Db()->fetchOne($sql, $fields);
	}
	
	
	/**
	 * If payment has been canceled whether if the user requested to or if some errors occured
	 */
	public function cancelAction() {
		$this->paymentMethod = $this->ShopwareUpdateHelper->getPaymentDetails($this->getPaymentShortName());
		$this->View()->loadTemplate('Frontend/payment_cancel.tpl');
		$message = $this->Request()->getParam('message');
		$transactionId = $this->Request()->getParam('transactionId');
		$this->uniqueId = $this->Request()->getParam('unique');
		$parameters = $this->Request()->getParams();
		$errorString = $this->getErrorMessagesFromUrlParameters($parameters);
		
		// set the canceled order's status to >canceled<
		if (!empty($transactionId) && !empty($this->uniqueId) && empty($this->errorCodes)) {
			$sql = 'SELECT COUNT(id) FROM `sofort_orders` WHERE `transactionId` = ? AND `secret` = ?';
			$fields = array(
				$transactionId,
				$this->uniqueId,
			);
			$authorized = $this->Shopware->Db()->fetchOne($sql, $fields);
			
			// decide, whether this function call is authorized or not...
			if ($authorized) {
				$this->initOrderTablesAndTimeLine($transactionId);
				$this->setOrderCanceled($transactionId);
				$this->setSofortStatusCanceled($transactionId);
				$this->repopulateBasket();
			}
		}
		
		// remove last comma if existing
		$errorString = substr($errorString, 0, -2);
		$this->View()->errorString = $errorString;
		$this->View()->sofortPaymentMethod = $this->ShopwareUpdateHelper->getPaymentDetails($this->paymentMethod);
		$this->View()->message = $message;
		$this->View()->sAmountWithTax = 10;
		$this->View()->errorCodes = $this->errorCodes;
		
		$this->View()->sBasket = $this->getBasket();
		$checkoutUrl = $this->Front()->Router()->assemble(array(
			'controller' => 'checkout',
			'action' => 'cart'
		));
		
		$this->View()->checkoutUrl = $checkoutUrl;
		return;
	}
	
	
	/**
	 * Parse GET parameters and set global error codes accordingly
	 * @param array $parameters
	 * @return string
	 */
	private function getErrorMessagesFromUrlParameters($parameters) {
		$errorString = '';
		
		foreach ($parameters as $key => $value) {
			// fetch all errors from request (errors[1], errors[2], ...)
			if (preg_match('/errors\[\d\]/', $key)) {
				
				$error = '';
				if (strpos($value, '.') !== false) {
					$position = strpos($value, '.');
					$error = substr($value, 0, $position);
					array_push($this->errorCodes, $error);
				} else {
					array_push($this->errorCodes, $value);
				}
				
				$errorString .= $value.', ';
			}
		}
		
		// every error has to be handled, but don't care about duplicates
		$this->errorCodes = array_unique($this->errorCodes);
		return $errorString;
	}
	
	
	/**
	 * 
	 * Finish payment and save order
	 */
	public function endAction() {
		$this->View()->loadTemplate('Frontend/finish.tpl');
		$basket = $this->getBasket();
		$this->View()->sBasket = $this->getBasket();
		$this->View()->sShippingcosts = $basket['sShippingcosts'];
		$this->View()->sAmount = $basket['sAmount'];
		
		// get the payment method
		$this->paymentMethod = $this->ShopwareUpdateHelper->getPaymentDetails($this->getPaymentShortName());
		// assign it to the view
		$this->View()->sofortPaymentDetails = $this->paymentMethod;
		
		$transactionId = '';
		$transactionId = $this->Request()->getParam('transactionId');
		
		// show payment details to the customer for Vorkasse by sofort
		if ($this->paymentMethod['name'] == 'vorkassebysofort_multipay') {
			$vorkasseParams = array(
				'holder' => urldecode($this->Request()->getParam('holder')),
				'accountNumber' => $this->Request()->getParam('account_number'),
				'iban' => $this->Request()->getParam('iban'),
				'bank_code' => $this->Request()->getParam('bank_code'),
				'bic' => $this->Request()->getParam('bic'),
				'amount' => $this->Request()->getParam('amount'),
				'reason_1' => urldecode($this->Request()->getParam('reason_1')),
				'reason_2' => $this->getOrderId($transactionId),
			);
			
			$this->View()->vorkasseParams = $vorkasseParams;
		}
		
		$this->View()->transactionId = $transactionId;
		// get the secret/unique key from request
		$this->uniqueId = $this->Request()->getParam('unique');
		
		if($this->checkIfRequestIsValid($transactionId, $this->paymentMethod['name'])) {
			// if it's a valid iDeal payment
			if ($this->checkIfIdealIsValid($this->uniqueId)) {
				// table sofort_orders
				$this->setTransactionIdForIdeal($transactionId, $this->uniqueId);
				// table s_order
				$this->setOrderTransactionForIdeal($transactionId, $this->uniqueId);
			}
			// if it's a valid gateway request
			// multiple execution must not update on and on...
			if($orderId = $this->orderExists($transactionId)) {
				unset($this->Shopware->Session()->sOrderVariables->sBasket);
				return $orderId;
			}
			$pendingState = $this->getPendingState($this->paymentMethod['name']);
			// delete temporary order by uniqueId/secret
			//$this->ShopwareUpdateHelper->deleteTemporaryOrder($this->uniqueId);
			// update the order's number
			$this->setPaymentStatus($transactionId, $pendingState);	// set the payment status
			$temporaryId = $this->getOrderTemporaryId($transactionId);
			$this->savePaymentStatus($transactionId, $temporaryId, $pendingState);	// save the payment status, set to pending
			$this->initOrderTablesAndTimeLine($transactionId);
			//unset($this->Shopware->Session()->sOrderVariables->sBasket);
			return;
		} else {
			$this->redirect($this->makeAbortUrl($this->Snippets->getSnippet("sofort_multipay_checkout")->get("order_error")));
		}
		
		return $this->forward('error');
	}
	
	
	/**
	 * 
	 * Setter for transactionId, iDeal
	 * @param unknown_type $transactionId
	 * @param unknown_type $secret
	 */
	private function setTransactionIdForIdeal($transactionId, $secret) {
		$sql = 'UPDATE `sofort_orders` SET transactionId = ? WHERE secret = ?';
		$fields = array(
			$transactionId,
			$secret,
		);
		return $this->Shopware->Db()->query($sql, $fields);
	}
	
	/**
	 * compare saved secret with the one fetched from request
	 * @param unknown_type $transactionId
	 */
	private function checkIfRequestIsValid($transactionId, $paymentMethod) {
		
		if($this->paymentMethod['name'] == 'iDeal' && $this->checkIfIdealIsValid($this->uniqueId)) {
			return true;
		} else if($this->uniqueId == $this->getPaymentSecret($transactionId)) {
			return true;
		}
		return false;
	}
	
	/**
	 * compare saved secret with the one fetched from request in case of iDeal
	 */
	private function checkIfIdealIsValid($secret) {
		$sql = 'SELECT paymentMethod FROM sofort_orders WHERE secret = ?';
		$fields = array(
			$secret,
		);
		$result = $this->Shopware->Db()->fetchOne($sql, $fields);
		if($result === 'iDeal') return true;
		return false;
	}
	
	
	/**
	 * 
	 * Timeout action (in case a payment runs into timeout)
	 */
	public function timeoutAction() {
		$this->View()->sBasket = $this->getBasket();
		$this->paymentMethod = $this->ShopwareUpdateHelper->getPaymentDetails($this->getPaymentShortName());
		$this->View()->sofortPaymentMethod = $this->paymentMethod;
		$this->View()->loadTemplate('Frontend/payment_timeout.tpl');
		$transactionId = $this->Request()->getParam('transactionId');
		$this->uniqueId = $this->Request()->getParam('unique');
		$checkoutUrl = $this->Front()->Router()->assemble(array(
			'controller' => 'checkout',
			'action' => 'cart'
		));
		
		$this->View()->checkoutUrl = $checkoutUrl;
		// set the canceled order's status to >canceled<
		if (!empty($transactionId) && !empty($this->uniqueId)) {
			$sql = 'SELECT COUNT(id) FROM `sofort_orders` WHERE `transactionId` = ? AND `secret` = ?';
			$fields = array($transactionId, $this->uniqueId);
			$authorized = $this->Shopware->Db()->fetchOne($sql, $fields);
			// decide, whether this function call is authorized or not...
			if ($authorized) {
				$this->initOrderTablesAndTimeLine($transactionId);
				$this->setOrderCanceled($transactionId);
				$this->setSofortStatusCanceled($transactionId);
				$this->repopulateBasket();
			}
		}
	}
	
	
	/**
	 * 
	 * Set an order to canceled
	 * @param string $transactionId
	 */
	private function setOrderCanceled($transactionId) {
		$sofortCanceledState = $this->getCanceledState($this->paymentMethod['name']);
		
		$sql = 'UPDATE `s_order` SET `status` = ? WHERE `transactionID` = ?';
		$fields = array(
			$sofortCanceledState,
			$transactionId,
		);
		$this->Shopware->Db()->query($sql, $fields);
		
		$this->setOrderCleared($transactionId, $sofortCanceledState);
		
		// set the sofort_orders's status to >canceled<
		$sql = 'UPDATE `sofort_orders` SET `paymentStatus` = "canceled" WHERE `transactionID` = ?';
		$fields = array(
			$transactionId,
		);
		$this->Shopware->Db()->query($sql, $fields);
		// unset the secret/token
		$sql = 'UPDATE `sofort_orders` SET `secret` = "" WHERE `transactionID` = ?';
		$fields = array(
			$transactionId,
		);
		$this->Shopware->Db()->query($sql, $fields);
		$this->setOrderComment($transactionId, $this->Snippets->getSnippet('sofort_multipay_cancel')->get('order_cancel'));
	}
	
	/**
	 * 
	 * Set the order cleared (s_order.cleared)
	 * @param String $transactionId
	 * @param int $status
	 */
	private function setOrderCleared($transactionId, $status) {
		$sql = 'UPDATE `s_order` SET `cleared`=? WHERE `transactionID`=?;';
		$fields = array(
			$status,
			$transactionId,
		);
		return $this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * Put an article back into stock
	 * @param int $articleId
	 * @param int $quantity
	 */
	private function restockArticle($articleId, $quantity) {
		$sql = 'UPDATE s_articles_details SET instock = instock + ? WHERE articleID = ?';
		$fields = array(
			$quantity,
			$articleId,
		);
		return $this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * Reset sales numbers
	 * @param int $articleId
	 * @param int $quantity
	 */
	private function resetSoldArticles($articleId, $quantity) {
		$sql = 'UPDATE s_articles_details SET sales = sales - ? WHERE articleID = ?';
		$fields = array(
			$quantity,
			$articleId,
		);
		return $this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * Getter for an order's articles
	 * @param string $transactionId
	 */
	private function fetchArticlesOfOrder($transactionId) {
		$sql = 'SELECT od.articleID, od.quantity FROM `s_order` o
				JOIN s_order_details od ON o.id = od.orderID
				WHERE o.transactionID = ?';
		$fields = array(
			$transactionId,
		);
		return $this->Shopware->Db()->fetchAll($sql, $fields);
	}
	
	
	/**
	 * 
	 * Error Action (in case error occurs via payment)
	 */
	public function errorAction() {
		$this->View()->sBasket = $this->getBasket();
		$this->View()->sofortPaymentMethod = $this->$this->ShopwareUpdateHelper->getPaymentDetails($this->paymentMethod);
		$this->View()->loadTemplate('Frontend/payment_error.tpl');
	}
	
	
	/**
	 * "Re-fills" the cart using data from session, just in case it was cleared during checkout
	 */
	private function repopulateBasket() {
		$basket = $this->Shopware->Session()->sOrderVariables->sBasket;
		$sessionId = $basket['content'][0]['sessionID'];
		$sql = 'SELECT `sessionID` FROM `s_order_basket` WHERE `sessionID` = "'.$sessionId.'"';
		$sessionExist = $this->Shopware->Db()->fetchOne($sql);
		
		if (empty($sessionExist)) {
			foreach ($basket['content'] as $item) {
				$id = $item['id'];
				$session = $item['sessionID'];
				$user = $item['userID'];
				$articlename = $item['articlename'];
				$articleID = $item['articleID'];
				$ordernumber = $item['ordernumber'];
				$shippingfree = $item['shippingfree'];
				$quantity = $item['quantity'];
				$price = $item['priceNumeric'];
				$netprice = $item['netprice'];
				$datum = $item['datum'];
				$modus = $item['modus'];
				$esdarticle = $item['esdarticle'];
				$partnerID = $item['partnerID'];
				$lastviewport = $item['lastviewport'];
				$useragent = $item['useragent'];
				$config = $item['config'];
				$currencyFactor = $item['currencyFactor'];
				$ob_attr1 = $item['ob_attr1'];
				$ob_attr2 = $item['ob_attr2'];
				$ob_attr3 = $item['ob_attr3'];
				$ob_attr4 = $item['ob_attr4'];
				$ob_attr5 = $item['ob_attr5'];
				$ob_attr6 = $item['ob_attr6'];
				$liveshoppingID = $item['liveshoppingID'];
				$bundleID = $item['bundleID'];
				$bundle_join_ordernumber = $item['bundle_join_ordernumber'];
				$sql = "INSERT INTO `s_order_basket` (`id`, `sessionID`, `userID`, `articlename`, `articleID`, `ordernumber`, `shippingfree`, `quantity`, `price`, `netprice`, `datum`, `modus`, `esdarticle`, `partnerID`, `lastviewport`, `useragent`, `config`, `currencyFactor`, `ob_attr1`, `ob_attr2`, `ob_attr3`, `ob_attr4`, `ob_attr5`, `ob_attr6`, `liveshoppingID`, `bundleID`, `bundle_join_ordernumber`) VALUES
				(".$id.", '".$session."', '".$user."', '".$articlename."', '".$articleID."', '".$ordernumber."', '".$shippingfree."', '".$quantity."', '".$price."', '".$netprice."', '".$datum."', '".$modus."', '".$esdarticle."', '".$partnerID."', '".$lastviewport."', '".$useragent."', '".$config."', '".$currencyFactor."', '".$ob_attr1."', '".$ob_attr2."', '".$ob_attr3."', '".$ob_attr4."', '".$ob_attr5."', '".$ob_attr6."', '".$liveshoppingID."', '".$bundleID."', '".$bundle_join_ordernumber."');";
				$this->Shopware->Db()->exec($sql);
			}
		}
		
		return false;
	}
	
	
	/**
	 * 
	 * Override the toString method
	 * @throws Exception
	 */
	public function __toString() {
		$string = 'payment method: '.$this->paymentMethod."\n";
		$string .= 'payment method string: '.$this->paymentMethodString."\n";
		return $string;
	}
}