<?php
require_once(dirname(__FILE__).'/../../library/sofortLib.php');
require_once(dirname(__FILE__).'/../../library/sofortLib_classic_notification.inc.php');
require_once(dirname(__FILE__).'/..//Helper/Observable.php');
require_once(dirname(__FILE__).'/../Helper/Helper.php');
require_once(dirname(__FILE__).'/../Helper/Exceptions/notification_not_succeeded.php');
/**
 *
 * This controller provides a callback for notifications coming from sofort.com
 * It provides an entry point for notifications via XML
 *
 * $Date: 2012-12-13 09:28:11 +0100 (Thu, 13 Dec 2012) $
 * @version sofort 1.0  $Id: SofortNotification.php 5861 2012-12-13 08:28:11Z dehn $
 * @author SOFORT AG http://www.sofort.com (f.dehn@sofort.com)
 * @package Shopware 3.5.x, sofort.com
 *
 */
class Shopware_Controllers_Frontend_SofortNotification extends Enlight_Controller_Action implements Observable {

	private $configKey = '';
	
	private $idealConfigKey = '';
	
	private $transactionId = '';
	
	private $uniqueId = '';
	
	private $time = '';
	
	private $SofortLib_TransactionData = null;
	
	private $SofortLib_ClassicNotification = null;
	
	private $SofortLib_Notification = null;
	
	private $Shopware = null;
	
	private $PnagInvoice = null;
	
	private $testMode = false;
	
	private $paymentStatus = array();
	
	private $paymentMethod = ''; // sr, sv, su ...
	
	private $paymentMethodString = '';
	
	private $shopItems = array();
	
	private $amountOfInvoice = 0;
	
	private $orderIdentifier = 0;
	
	private $statusOfInvoice = 0;
	
	private $observers = array();
	
	private $ShopwareUpdateHelper = null;
	
	/**
	 * Language Snippets
	 * @var Object
	 */
	private $Snippets = null;
	
	private $statuses = array();
	
	private $statusMapping = array();
	
	/**
	 *
	 * initiate this controller
	 */
	public function init() {
		if (!$this->Shopware) {
			$this->setShopware(Shopware());
		}
		
		$this->ShopwareUpdateHelper = new ShopwareUpdateHelper($this->Shopware);
		
		$this->Snippets = $this->Shopware->Snippets();
		$this->View()->addTemplateDir(dirname(__FILE__).'/../../Templates');
		
		$sql = 'SELECT value FROM `s_core_config` WHERE `name` = "sofort_api_key" ';
		$this->configKey = $this->Shopware->Db()->fetchOne($sql);
		
		$sql = 'SELECT value FROM `s_core_config` WHERE `name` = "ideal_api_key" ';
		$this->idealConfigKey = $this->Shopware->Db()->fetchOne($sql);
		
		if($this->configKey == '') {
			$this->configKey = $this->idealConfigKey;	// TODO: check if useful...
		}
		
		if ($this->SofortLib_Notification === null) {
			$this->SofortLib_Notification = new SofortLib_Notification();
		}
		if ($this->SofortLib_TransactionData === null) {
			$this->SofortLib_TransactionData = new SofortLib_TransactionData($this->configKey);
		}
		
		$this->statuses = $this->fetchStatuses();
		$this->statusMapping = $this->initStatusMapping();
		$this->initStatus();
	}
	
	/**
	 * 
	 * fetch the statuses
	 */
	private function fetchStatuses() {
		return parse_ini_file(dirname(__FILE__).'/../../config/config.ini', true);
	}
	
	
	/**
	 * 
	 * Fetch the status mapping
	 */
	private function fetchStatusMapping() {
		$status = parse_ini_file(dirname(__FILE__).'/../../config/config.ini', true);
		return $status;
	}
	
	
	/**
	 * 
	 * Init the status mapping
	 */
	private function initStatusMapping() {
		$status = $this->fetchStatusMapping();
		return $status['mapping'];
	}
	
	
	/**
	 * 
	 * Init all statuses
	 */
	private function initStatus() {
		foreach($this->statuses as $paymentMethod => $status) {
			switch($paymentMethod) {
				case('sr'):
					foreach($this->statuses['sr'] as $paymentState => $paymentStateDescription) {
						$this->initPaymentState('sr', $paymentStateDescription, $paymentStateValue);
					}
				break;
				case('su'):
					foreach($this->statuses['su'] as $paymentState => $paymentStateDescription) {
						$this->initPaymentState('su', $paymentStateDescription, $paymentStateValue);
					}
				break;
				case('sl'):
					foreach($this->statuses['sl'] as $paymentState => $paymentStateDescription) {
						$this->initPaymentState('sl', $paymentStateDescription, $paymentStateValue);
					}
				break;
				case('ls'):
					foreach($this->statuses['ls'] as $paymentState => $paymentStateDescription) {
						$this->initPaymentState('ls', $paymentStateDescription, $paymentStateValue);
					}
				break;
				case('sv'):
					foreach($this->statuses['sv'] as $paymentState => $paymentStateDescription) {
						$this->initPaymentState('sv', $paymentStateDescription, $paymentStateValue);
					}
				break;
				case('iDeal'):
					foreach($this->statuses['iDeal'] as $paymentState => $paymentStateDescription) {
						$this->initPaymentState('iDeal', $paymentStateDescription);
					}
				break;
			}
		}
	}
	
	
	/**
	 * 
	 * Init the payment state
	 * @param string $paymentMethod
	 * @param string $paymentStateDescription
	 */
	private function initPaymentState($paymentMethod, $paymentStateDescription) {
		$sql = 'SELECT `value` FROM `s_core_config` WHERE `name` = "'.$paymentStateDescription.'"';
		$paymentStateValue = $this->Shopware->Db()->fetchOne($sql);
		$this->paymentStatus[$paymentMethod][$paymentStateDescription] = $paymentStateValue;
	}
	
	
	/**
	 * 
	 * Map all statuses
	 * @param string $status
	 */
	private function doStatusMapping($status) {
		return $this->statusMapping[$status];
	}
	
	
	/**
	 * 
	 * Set the test mode flag
	 * @param boolean $flag
	 */
	public function setTestMode($flag) {
		$this->testMode = $flag;
	}
	
	
	/**
	 * 
	 * set the selected payment method
	 * @param string $paymentMethod
	 */
	public function setSelectedPaymentMethod($paymentMethod) {
		$this->paymentMethod = $paymentMethod;
	}
	
	
	/**
	 * 
	 * Getter for the selected payment method
	 */
	public function getSelectedPaymentMethod() {
		return $this->paymentMethod;
	}
	
	
	/**
	 * attach observer (object)
	 * @see Observable::attach()
	 */
	public function attach(Observer $Observer) {
		array_push($this->observers, $Observer);
	}
	
	
	/**
	 * detach observer (object)
	 * @see Observable::detach()
	 */
	public function detach(Observer $Observer) {
		if (in_array($Observer, $this->observers)) {
			$key = array_search($Observer, $this->observers);
			unset($this->observers[$key]);
		}
	}
	
	
	/**
	 * 
	 * Getter for all attached observers
	 */
	public function getObservers() {
		return $this->observers;
	}
	
	
	/**
	 * notify observers (object)
	 * @see Observable::notify()
	 */
	public function notify($key, $message = '') {
		if (count($this->observers) == 0) {
			return;
		}
		
		foreach ($this->observers as $observer) {
			$observer->update($key, $message, $this);
		}
	}
	
	
	/**
	 * 
	 * Set the Snippet Object
	 * @param object $snippets
	 */
	public function setSnippets($snippets) {
		$this->Snippets = $snippets;
	}
	
	
	/**
	 * 
	 * Set the Shopware object
	 * @param Shopware $instance
	 */
	public function setShopware(Shopware $instance) {
		$this->Shopware = $instance;
	}
	
	
	/**
	 * 
	 * Getter for Shopware object
	 */
	public function getShopware() {
		return $this->Shopware;
	}
	
	
	/**
	 * 
	 * Getter for Database object
	 */
	public function getDb() {
		return $this->Shopware->Db();
	}
	
	
	/**
	 * 
	 * Settter for config keyy
	 * @param string $configKey
	 */
	public function setConfigKey($configKey) {
		$this->configKey = $configKey;
	}
	
	
	/**
	 * 
	 * Setter for TransactionData object
	 * @param object $SofortLib_TransactionData
	 */
	public function setTransactionData($SofortLib_TransactionData) {
		$this->SofortLib_TransactionData = $SofortLib_TransactionData;
	}
	
	
	/**
	 * 
	 * Setter for SofortlibNotification object
	 * @param object $SofortLib_Notification
	 */
	public function setSofortLibNotification($SofortLib_Notification) {
		$this->SofortLib_Notification = $SofortLib_Notification;
	}
	
	
	/**
	 * 
	 * Getter for SofortlibNotification object
	 */
	public function getSofortLibNotification() {
		return $this->SofortLib_Notification;
	}
	
	
	/**
	 * 
	 * Setter for TransactionData object
	 * @param object $transactionId
	 */
	public function setTransactionId($transactionId) {
		$this->transactionId = $transactionId;
	}
	
	
	/**
	 * 
	 * Setter for shop items
	 * @param array $shopItems
	 */
	public function setShopItems($shopItems) {
		$this->shopItems = $shopItems;
	}
	
	
	/**
	 * 
	 * Getter for payment method
	 */
	public function getPaymentMethod() {
		return $this->paymentMethod;
	}
	
	
	/**
	 * 
	 * Getter for PnagInvoice object
	 */
	public function getPnagInvoice() {
		return $this->PnagInvoice;
	}
	
	
	/**
	 * 
	 * Setter for PnagInvoice object
	 * @param object $PnagInvoice
	 */
	public function setPnagInvoice($PnagInvoice) {
		$this->PnagInvoice = $PnagInvoice;
	}
	
	
	/**
	 * 
	 * Setter for invoice's amount
	 * @param float $amount
	 */
	public function setAmountOfInvoice($amount) {
		$this->amountOfInvoice = $amount;
	}
	
	
	/**
	 * 
	 * Setter for Order identifier
	 * @param string $orderIdentifier
	 */
	public function setOrderIdentifier($orderIdentifier) {
		$this->orderIdentifier = $orderIdentifier;
	}
	
	
	/**
	 * 
	 * Setter for status of invoice
	 * @param int $statusOfInvoice
	 */
	public function setStatusOfInvoice($statusOfInvoice) {
		$this->statusOfInvoice = $statusOfInvoice;
	}
	
	
	/**
	 * 
	 * Debug method, writes to file
	 * @param mixde $data
	 */
	private function debug($data) {
		$fp = fopen('/home/dehn/shopware.txt', 'a');
		fwrite($fp, date('d M Y H:i:s', time())."\r\n");
		fwrite($fp, print_r($data, 1));
		fwrite($fp, "\r\n");
		fclose($fp);
	}
	
	
	/**
	 *
	 * This method is called if no additional action is specified in URL
	 */
	public function indexAction() {
		// fetch the unique payment id
		$this->View()->loadTemplate('Frontend/forbidden.tpl');
		$this->uniqueId = $this->Request()->getParam('unique');
		
		// in case there is no valid request, just exit
		if (empty($this->uniqueId) || $this->getPayment($this->uniqueId) != 1) {
			header("HTTP/1.0 404 Not Found");
			return;
		}
		
		// decide whether we've got a gateway product or some classic product
		if($this->Request()->getParam('sofortGateway') === 'false') {
			$sql = 'SELECT value FROM s_core_config WHERE name = "ideal_notification_password"';
			$notificationPassword = $this->Shopware->Db()->fetchOne($sql);
			$this->paymentMethod = 'iDeal';
			list($userid, $projectid) = split(':', $this->idealConfigKey);
			$this->SofortLib_ClassicNotification = new SofortLib_ClassicNotification($userid, $projectid, $notificationPassword);
			$this->SofortLib_ClassicNotification->getNotification($this->Request()->getParams());
			$this->transactionId = $this->SofortLib_ClassicNotification->getTransaction();
			$this->time = $this->SofortLib_ClassicNotification->getTime();
		} else {	// this is gateway
			$this->transactionId = $this->SofortLib_Notification->getNotification();
			$this->time = $this->SofortLib_Notification->getTime();
			// fetch transaction details
			$this->SofortLib_TransactionData->setTransaction($this->transactionId)->sendRequest();
			// fetch payment method (su, sr, ...)
			$this->paymentMethod = $this->SofortLib_TransactionData ->getPaymentMethod();
		}
		
		switch($this->paymentMethod) {
			case 'sr':
				$this->paymentMethodString = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sr_public_title');
				return $this->setRechnungBySofort();
				break;
			case 'su':
				$this->paymentMethodString = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_su_public_title');
				return $this->setSofortUeberweisung();
				break;
			case 'sl':
				$this->paymentMethodString = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sl_public_title');
				return $this->setSofortLastschrift();
				break;
			case 'ls':
				$this->paymentMethodString = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_ls_public_title');
				return $this->setLastschriftBySofort();
				break;
			case 'sv':
				$this->paymentMethodString = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sv_public_title');
				return $this->setVorkasseBySofort();
				break;
			case 'iDeal':
				$this->paymentMethodString = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_ideal_title');
				return $this->setIdeal();
				break;
		}
		return true;
	}
	
	
	/**
	 * Notification for Rechnung by sofort
	 * @throws Exception
	 */
	private function setRechnungBySofort() {
		if(!$this->PnagInvoice) {
			$this->PnagInvoice = new PnagInvoice($this->configKey, $this->transactionId);
		}
		
		$oldStatus = $this->getStatusOfInvoice();
		$newStatus = $this->PnagInvoice->getState();
		$oldAmount = $this->getAmountOfInvoice();
		$newAmount = $this->PnagInvoice->getAmount();
		// see if amount has changed -> indicates an updated cart
		$amountChanged = ($oldAmount != $newAmount) ? true : false;
		$newAmountHigher = ($newAmount > $oldAmount) ? true : false;
		$oldAmountHigher = ($oldAmount > $newAmount) ? true : false;
		$statusChanged = ($oldStatus != $newStatus) ? true : false;
		// update timeline everytime a notification comes in
		$this->ShopwareUpdateHelper->updateTimeline($this->transactionId, $this->PnagInvoice->getState(), $this->PnagInvoice->getStatus(), $this->PnagInvoice->getStatusReason(), $this->PnagInvoice->getStatusOfInvoice(), $this->PnagInvoice->getInvoiceObjection(), serialize($this->PnagInvoice->getItems()));
		// update address items if changed in payment wizard
		$this->updateOrderAddresses();
		// unittesting
		
		$this->notify('oldStatus', $oldStatus);
		$this->notify('newStatus', $newStatus);
		$this->notify('oldAmount', $oldAmount);
		$this->notify('newAmount', $newAmount);
		$this->notify('amountChanged', $amountChanged);
		
		//$this->ShopwareUpdateHelper->__($oldAmount, true, 'OldAmount');
		//$this->ShopwareUpdateHelper->__($newAmount, true, 'NewAmount');
		
		// handle invoice states
		switch ($newStatus) {
			case PnagInvoice::PENDING_CONFIRM_INVOICE:	// Offene Rechnung, noch nicht best�tigt #1
				if ($amountChanged && $oldAmountHigher) {
					$cartItemsEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('edit_invoice.CartItemsEdited');
					$newAmountEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.current_amount');
					$cartUpdatedWithNewAmount = $cartItemsEdited.' ('.$newAmountEdited.' '.$newAmount.' EUR)';
					// Merchant has to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartUpdatedWithNewAmount);
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $cartUpdatedWithNewAmount);
					$this->ShopwareUpdateHelper->updateCart($this->PnagInvoice);
					return true;
				} elseif($newAmountHigher) {
					$cartReset = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.edit_invoice.CartReset');
					$newAmountEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.current_amount');
					$cartUpdatedWithNewAmount = $cartReset.' ('.$newAmountEdited.' '.$newAmount.' EUR)';
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartUpdatedWithNewAmount);
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $cartUpdatedWithNewAmount);
					$this->ShopwareUpdateHelper->updateCart($this->PnagInvoice);
				}
				
				if ($statusChanged) {
					$this->setPaymentStatus($this->PnagInvoice->getStatusReason());
					$srConfirmInvoiceString = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.PendingConfirmInvoice');
					$notificationString = $srConfirmInvoiceString.' '.$this->transactionId;
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $notificationString);
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $notificationString);
				}
				
				break;
			case PnagInvoice::LOSS_CANCELED:	// Vollstorno #2
				$this->setPaymentStatus($this->PnagInvoice->getStatusReason());
				$srCanceledString = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sr_canceled');
				// Merchant and customer have to be informed
				$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $srCanceledString);
				$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $srCanceledString);
				$this->ShopwareUpdateHelper->restockCanceledOrder($this->transactionId);
				$this->setOrderStatusToCanceled();
				break;
			case PnagInvoice::LOSS_CONFIRMATION_PERIOD_EXPIRED:	// Zeitraum von 30 Tagen zur Best�tigung abgelaufen #3
				$srPeriodExpiredString = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.LossConfirmationPeriodExpired');
				// Merchant has to be informed
				$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $srPeriodExpiredString);
				$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $srPeriodExpiredString);
				$this->setPaymentStatus($this->paymentStatus['sr']['canceled']);
				break;
			case PnagInvoice::PENDING_NOT_CREDITED_YET_PENDING:	// Rechnung best�tigt, noch keine Zahlung eingegangen #4
				// if cart has been changed, update cart
				if ($amountChanged && $oldAmountHigher) {
					$cartItemsEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('edit_invoice.CartItemsEdited');
					$newAmountEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.current_amount');
					$cartUpdatedWithNewAmount = $cartItemsEdited.' ('.$newAmountEdited.' '.$newAmount.' EUR)';
					// Merchant has to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartUpdatedWithNewAmount);
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $cartUpdatedWithNewAmount);
					$this->ShopwareUpdateHelper->updateCart($this->PnagInvoice);
					return true;
				} elseif($newAmountHigher) {
					$cartReset = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.edit_invoice.CartReset');
					$newAmountEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.current_amount');
					$cartUpdatedWithNewAmount = $cartReset.' ('.$newAmountEdited.' '.$newAmount.' EUR)';
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartUpdatedWithNewAmount);
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $cartUpdatedWithNewAmount);
					$this->ShopwareUpdateHelper->updateCart($this->PnagInvoice);
				}
				
				if ($statusChanged) {	
					$this->setPaymentStatus($this->PnagInvoice->getStatusReason());
					$transactionStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.ConfirmedWaitingForMoney');
					$readyForShipment = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.ReadyForShipment');
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $transactionStatus);
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $readyForShipment);
				}
				
				break;
			/*
			case PnagInvoice::PENDING_NOT_CREDITED_YET_REMINDER_1:	// Mahnung Stufe 1 #5
				// if cart has been changed, update cart
				if ($amountChanged && $oldAmountHigher) {
					$cartItemsEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('edit_invoice.CartItemsEdited');
					// Merchant has to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartItemsEdited);
					$this->ShopwareUpdateHelper->updateCart($this->PnagInvoice);
					return true;
				} elseif($newAmountHigher) {
					$cartReset = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.edit_invoice.CartReset');
					$newAmountEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.current_amount');
					$cartUpdatedWithNewAmount = $cartReset.' ('.$newAmountEdited.' '.$newAmount.' EUR)';
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartUpdatedWithNewAmount);
				}
				
				if ($statusChanged) {
					$transactionStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.ConfirmedWaitingForMoney');
					$invoiceStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.DunningLevel');
					$invoiceStatus = str_replace('{{d}}', '1', $invoiceStatus);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $transactionStatus, $invoiceStatus);
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $invoiceStatus);
				}
				break;
			*/
			/*
			case PnagInvoice::PENDING_NOT_CREDITED_YET_REMINDER_2:	// Mahnung Stufe 2 #6
				// if cart has been changed, update cart
				if ($amountChanged && $oldAmountHigher) {
					$cartItemsEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('edit_invoice.CartItemsEdited');
					// Merchant has to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartItemsEdited);
					$this->ShopwareUpdateHelper->updateCart($this->PnagInvoice);
					return true;
				} elseif($newAmountHigher) {
					$cartReset = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.edit_invoice.CartReset');
					$newAmountEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.current_amount');
					$cartUpdatedWithNewAmount = $cartReset.' ('.$newAmountEdited.' '.$newAmount.' EUR)';
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartUpdatedWithNewAmount);
				}
				
				if ($statusChanged) {
					$transactionStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.ConfirmedWaitingForMoney');
					$invoiceStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.DunningLevel');
					$invoiceStatus = str_replace('{{d}}', '2', $invoiceStatus);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $transactionStatus, $invoiceStatus);
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $invoiceStatus);
				}
				
				break;
			*/
			/*
			case PnagInvoice::PENDING_NOT_CREDITED_YET_REMINDER_3:	// Mahnung Stufe 3 #7
				// if cart has been changed, update cart
				if ($amountChanged && $oldAmountHigher) {
					$cartItemsEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('edit_invoice.CartItemsEdited');
					// Merchant has to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartItemsEdited);
					$this->ShopwareUpdateHelper->updateCart($this->PnagInvoice);
					return true;
				} elseif($newAmountHigher) {
					$cartReset = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.edit_invoice.CartReset');
					$newAmountEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.current_amount');
					$cartUpdatedWithNewAmount = $cartReset.' ('.$newAmountEdited.' '.$newAmount.' EUR)';
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartUpdatedWithNewAmount);
				}
				
				if ($statusChanged) {	
					$transactionStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.ConfirmedWaitingForMoney');
					$invoiceStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.DunningLevel');
					$invoiceStatus = str_replace('{{d}}', '3', $invoiceStatus);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $transactionStatus, $invoiceStatus);
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $invoiceStatus);
				}
				
				break;
			*/
			/*
			case PnagInvoice::PENDING_NOT_CREDITED_YET_DELCREDERE:	// �bergabe an Inkasso #8
				// if cart has been changed, update cart
				if ($amountChanged && $oldAmountHigher) {
					$cartItemsEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('edit_invoice.CartItemsEdited');
					// Merchant has to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartItemsEdited);
					$this->ShopwareUpdateHelper->updateCart($this->PnagInvoice);
					return true;
				} elseif($newAmountHigher) {
					$cartReset = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.edit_invoice.CartReset');
					$newAmountEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.current_amount');
					$cartUpdatedWithNewAmount = $cartReset.' ('.$newAmountEdited.' '.$newAmount.' EUR)';
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartUpdatedWithNewAmount);
				}
				
				if ($statusChanged) {
					$transactionStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.ConfirmedWaitingForMoney');
					$invoiceStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.Encashment');
					$invoiceStatus = str_replace('{{d}}', '2', $invoiceStatus);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $transactionStatus, $invoiceStatus);
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $invoiceStatus);
				}
				
				break;
			*/
			/*
			case PnagInvoice::RECEIVED_CREDITED_REMINDER_1:	// Mahnung Stufe 1 #9
				// if cart has been changed, update cart
				if ($amountChanged && $oldAmountHigher) {
					$cartItemsEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('edit_invoice.CartItemsEdited');
					// Merchant has to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartItemsEdited);
					$this->ShopwareUpdateHelper->updateCart($this->PnagInvoice);
					return true;
				} elseif($newAmountHigher) {
					$cartReset = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.edit_invoice.CartReset');
					$newAmountEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.current_amount');
					$cartUpdatedWithNewAmount = $cartReset.' ('.$newAmountEdited.' '.$newAmount.' EUR)';
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartUpdatedWithNewAmount);
				}
				
				if ($statusChanged) {
					$transactionStatus = $this->Snippets->getSnippet('sofort_multipay_finish')->get('sofort_multipay_credited_to_seller');
					$invoiceStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.DunningLevel');
					$invoiceStatus = str_replace('{{d}}', '1', $invoiceStatus);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $transactionStatus, $invoiceStatus);
					if ($oldStatus !== PnagInvoice::PENDING_NOT_CREDITED_YET_REMINDER_1) {
						$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $invoiceStatus);
					}
				}
				
				break;
			*/
			/*
			case PnagInvoice::RECEIVED_CREDITED_REMINDER_2:	// Mahnung Stufe 2 #10
				// if cart has been changed, update cart
				if ($amountChanged && $oldAmountHigher) {
					$cartItemsEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('edit_invoice.CartItemsEdited');
					// Merchant has to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartItemsEdited);
					$this->ShopwareUpdateHelper->updateCart($this->PnagInvoice);
					return true;
				} elseif($newAmountHigher) {
					$cartReset = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.edit_invoice.CartReset');
					$newAmountEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.current_amount');
					$cartUpdatedWithNewAmount = $cartReset.' ('.$newAmountEdited.' '.$newAmount.' EUR)';
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartUpdatedWithNewAmount);
				}
				
				if ($statusChanged) {
					$transactionStatus = $this->Snippets->getSnippet('sofort_multipay_finish')->get('sofort_multipay_credited_to_seller');
					$invoiceStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.DunningLevel');
					$invoiceStatus = str_replace('{{d}}', '2', $invoiceStatus);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $transactionStatus, $invoiceStatus);
					if ($oldStatus !== PnagInvoice::PENDING_NOT_CREDITED_YET_REMINDER_2) {
						$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $invoiceStatus);
					}
				}
				
				break;
			*/
			/*
			case PnagInvoice::RECEIVED_CREDITED_REMINDER_3:	// Mahnung Stufe 3 #11
				// if cart has been changed, update cart
				if ($amountChanged && $oldAmountHigher) {
					$cartItemsEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('edit_invoice.CartItemsEdited');
					// Merchant has to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartItemsEdited);
					$this->ShopwareUpdateHelper->updateCart($this->PnagInvoice);
					return true;
				} elseif($newAmountHigher) {
					$cartReset = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.edit_invoice.CartReset');
					$newAmountEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.current_amount');
					$cartUpdatedWithNewAmount = $cartReset.' ('.$newAmountEdited.' '.$newAmount.' EUR)';
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartUpdatedWithNewAmount);
				}
				
				if ($statusChanged) {
					$transactionStatus = $this->Snippets->getSnippet('sofort_multipay_finish')->get('sofort_multipay_credited_to_seller');
					$invoiceStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.DunningLevel');
					$invoiceStatus = str_replace('{{d}}', '3', $invoiceStatus);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $transactionStatus, $invoiceStatus);
					if ($oldStatus !== PnagInvoice::PENDING_NOT_CREDITED_YET_REMINDER_3) {
						$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $invoiceStatus);
					}
				}
				
				break;
			*/
			/*
			case PnagInvoice::RECEIVED_CREDITED_DELCREDERE:	// �bergabe an Inkasso #12
				// if cart has been changed, update cart
				if ($amountChanged && $oldAmountHigher) {
					$cartItemsEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('edit_invoice.CartItemsEdited');
					// Merchant has to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartItemsEdited);
					$this->ShopwareUpdateHelper->updateCart($this->PnagInvoice);
					return true;
				} elseif($newAmountHigher) {
					$cartReset = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.edit_invoice.CartReset');
					$newAmountEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.current_amount');
					$cartUpdatedWithNewAmount = $cartReset.' ('.$newAmountEdited.' '.$newAmount.' EUR)';
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartUpdatedWithNewAmount);
				}
				
				if ($statusChanged) {
					$transactionStatus = $this->Snippets->getSnippet('sofort_multipay_finish')->get('sofort_multipay_credited_to_seller');
					$invoiceStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.Encashment');
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $transactionStatus, $invoiceStatus);
					if ($oldStatus !== PnagInvoice::PENDING_NOT_CREDITED_YET_DELCREDERE) {
						$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $invoiceStatus);
					}
				}
				
				break;
			*/
			/*
			case PnagInvoice::RECEIVED_CREDITED_PENDING: // Zahlungseingang auf H�ndlerkonto #13
				// if cart has been changed, update cart
				if ($amountChanged && $oldAmountHigher) {
					$cartItemsEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('edit_invoice.CartItemsEdited');
					// Merchant has to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartItemsEdited);
					$this->ShopwareUpdateHelper->updateCart($this->PnagInvoice);
					return true;
				} elseif($newAmountHigher) {
					$cartReset = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.edit_invoice.CartReset');
					$newAmountEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.current_amount');
					$cartUpdatedWithNewAmount = $cartReset.' ('.$newAmountEdited.' '.$newAmount.' EUR)';
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartUpdatedWithNewAmount);
				}
				
				if ($statusChanged) {
					$transactionStatus = $this->Snippets->getSnippet('sofort_multipay_finish')->get('sofort_multipay_credited_to_seller');
					$invoiceStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.PendingInvoice');
					// Merchant has to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $transactionStatus, $invoiceStatus);
				}
				
				break;
			*/
			case PnagInvoice::REFUNDED_REFUNDED_REFUNDED:	// Gutschrift erstellt. #14
				$this->setPaymentStatus($this->PnagInvoice->getStatusReason());
				$transactionStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.MoneyRefunded');
				// Merchant and customer have to be informed
				$invoiceStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.InvoiceRefunded');
				$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $transactionStatus, $invoiceStatus);
				$invoiceStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.statusRefunded');
				$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $invoiceStatus);
				$this->ShopwareUpdateHelper->restockCanceledOrder($this->transactionId);
				$this->setOrderStatusToCanceled();
				break;
			/*
			case PnagInvoice::PENDING_NOT_CREDITED_YET_RECEIVED:	// Zahlungseingang #15
				// if cart has been changed, update cart
				if ($amountChanged && $oldAmountHigher) {
					$cartItemsEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('edit_invoice.CartItemsEdited');
					// Merchant has to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartItemsEdited);
					$this->ShopwareUpdateHelper->updateCart($this->PnagInvoice);
					return true;
				} elseif($newAmountHigher) {
					$cartReset = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.edit_invoice.CartReset');
					$newAmountEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.current_amount');
					$cartUpdatedWithNewAmount = $cartReset.' ('.$newAmountEdited.' '.$newAmount.' EUR)';
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartUpdatedWithNewAmount);
				}
				
				if ($statusChanged) {
					$this->setPaymentStatus($this->PnagInvoice->getStatusReason());
					$transactionStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.ConfirmedWaitingForMoney');
					$invoiceStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.ReceiptOfPayment');
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $transactionStatus, $invoiceStatus);
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $invoiceStatus);
				}
				
				break;
			*/
			case PnagInvoice::RECEIVED_CREDITED_RECEIVED:	// Zahlungseingang auf H�ndlerkonto #16
				// if cart has been changed, update cart
				if ($amountChanged && $oldAmountHigher) {
					$cartItemsEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('edit_invoice.CartItemsEdited');
					$newAmountEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.current_amount');
					$cartUpdatedWithNewAmount = $cartItemsEdited.' ('.$newAmountEdited.' '.$newAmount.' EUR)';
					// Merchant has to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartUpdatedWithNewAmount);
					$this->ShopwareUpdateHelper->updateCart($this->PnagInvoice);
					return true;
				} elseif($newAmountHigher) {
					$cartReset = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.edit_invoice.CartReset');
					$newAmountEdited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('admin.sr.current_amount');
					$cartUpdatedWithNewAmount = $cartReset.' ('.$newAmountEdited.' '.$newAmount.' EUR)';
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cartUpdatedWithNewAmount);
					$this->ShopwareUpdateHelper->updateCart($this->PnagInvoice);
				}
				
				if ($statusChanged) {
					$transactionStatus = $this->Snippets->getSnippet('sofort_multipay_finish')->get('sofort_multipay_credited_to_seller');
					$invoiceStatus = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.ReceiptOfPayment');
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $transactionStatus, $invoiceStatus);
					//$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $invoiceStatus);
				}
				
				break;
			default:
				break;
		}
		return true;
	}
	
	
	/**
	 * Notification for sofort�berweisung
	 * Status reason [not_credited_yet|not_credited]
	 * @throws Exception
	 */
	private function setSofortUeberweisung() {
		if ($this->SofortLib_TransactionData->isError()) {
			throw new Exception('Error - '.__FILE__.' - '.__LINE__);
			return false;
		} else {
			$status = $this->SofortLib_TransactionData->getStatus();
			$statusReason = $this->SofortLib_TransactionData->getStatusReason();
			$this->setPaymentStatus($statusReason);
			$this->ShopwareUpdateHelper->updateTimeline($this->transactionId, 0, $status, $statusReason, '', '', '');
			$consumerProtection = $this->SofortLib_TransactionData->getConsumerProtection();
			$this->setCustomerProtection($consumerProtection);
			
			switch($statusReason) {
				case 'not_credited_yet':
					$successfulOrder = '';
					
					if($consumerProtection) {
						$this->paymentMethodString = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_su_public_title_ks');
					}
					
					$successfulOrder = $this->Snippets->getSnippet('sofort_multipay_finish')->get('sofort_multipay_confirm_invoice2_vorkasse');
					$successfulOrder = str_replace('{{paymentMethodStr}}', $this->paymentMethodString, $successfulOrder);
					$successfulOrder = str_replace('{{time}}', '', $successfulOrder);
					$successfulOrder = str_replace('{{tId}}', $this->transactionId, $successfulOrder);
					
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $successfulOrder);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $successfulOrder);
					break;
				case 'not_credited':
					
					if ($status === 'loss') {
						$loss = $this->Snippets->getSnippet('sofort_multipay_finish')->get('sofort_multipay_su_status_loss');
						$loss = str_replace('{{time}}', '', $loss);
						// Merchant and customer have to be informed
						$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $loss);
						$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $loss);
					} else {
						$notCredited = $this->Snippets->getSnippet('sofort_multipay_finish')->get('sofort_multipay_not_credited');
						// Merchant and customer have to be informed
						$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $notCredited);
						$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $notCredited);
					}
					break;
				case 'credited':
					$credited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sv.status_credited');
					$credited = str_replace('{{paymentMethodStr}}', $this->paymentMethodString, $credited);
					$credited = str_replace('{{time}}', '', $credited);
					// Merchant and customer have to be informed
					// $this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $credited);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $credited);
					break;
				case 'refunded':
					$refunded = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_su_refunded');
					$refunded = str_replace('{{time}}', '', $refunded);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $refunded);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $refunded);
					break;
				case 'consumer_protection':
					$cpCredited = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_su_customerprotection_credited');
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $cpCredited);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $cpCredited);
					break;
				case 'complaint':
					break;
				case 'default':
					break;
			}
			
			return true;
		}
	}
	
	
	/**
	 * Notification for sofortvorkasse
	 * Status reason [wait_for_money|not_credited]
	 * @throws Exception
	 */
	private function setVorkasseBySofort() {
		if ($this->SofortLib_TransactionData->isError()) {
			throw new Exception('Error - '.__FILE__.' - '.__LINE__);
			return false;
		} else {
			$status = $this->SofortLib_TransactionData->getStatus();
			$statusReason = $this->SofortLib_TransactionData->getStatusReason();
			$this->setPaymentStatus($statusReason);
			$this->ShopwareUpdateHelper->updateTimeline($this->transactionId, 0, $status, $statusReason, '', '', '');
			$consumerProtection = $this->SofortLib_TransactionData->getConsumerProtection();
			$this->setCustomerProtection($consumerProtection);
			
			switch($statusReason) {
				case 'wait_for_money':
					$successfulOrder = '';
					
					if($consumerProtection) {
						$this->paymentMethodString = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sv_public_title_ks');
					}
					
					$successfulOrder = $this->Snippets->getSnippet('sofort_multipay_finish')->get('sofort_multipay_confirm_invoice2_vorkasse');
					$successfulOrder = str_replace('{{paymentMethodStr}}', $this->paymentMethodString, $successfulOrder);
					$successfulOrder = str_replace('{{time}}', '', $successfulOrder);
					$successfulOrder = str_replace('{{tId}}', $this->transactionId, $successfulOrder);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $successfulOrder.$this->transactionId);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $successfulOrder.$this->transactionId);
					break;
				case 'not_credited':
					
					if ($status === 'loss') {
						$loss = $this->Snippets->getSnippet('sofort_multipay_finish')->get('sofort_multipay_su_status_loss');
						$loss = str_replace('{{time}}', '', $loss);
						// Merchant and customer have to be informed
						$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $loss);
						$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $loss);
					} else {
						$notCredited = $this->Snippets->getSnippet('sofort_multipay_finish')->get('sofort_multipay_not_credited');
						// Merchant and customer have to be informed
						$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $notCredited);
						$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $notCredited);
					}
					break;
				case 'credited':
					$credited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sv.status_credited');
					$credited = str_replace('{{time}}', '', $credited);
					$credited = str_replace('{{paymentMethodStr}}', $this->paymentMethodString, $credited);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $credited);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $credited);
					break;
				case 'refunded':
					$refunded = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_su_refunded');
					$refunded = str_replace('{{time}}', '', $refunded);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $refunded);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $refunded);
					break;
				case 'partially_credited':
					$partially = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sv_partially_credited');
					$partially = str_replace('{{time}}', '', $partially);
					$partially = str_replace('{{paymentMethodStr}}', $this->paymentMethodString, $partially);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $partially);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $partially);
					break;
				case 'overpayment':
					$overpayment = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sv_received_overpayment');
					$overpayment = str_replace('{{time}}', '', $overpayment);
					$overpayment = str_replace('{{paymentMethodStr}}', $this->paymentMethodString, $overpayment);
					$overpayment = str_replace('{{received_amount}}', $this->SofortLib_TransactionData->getAmountReceived().' &euro;', $overpayment);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $overpayment);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $overpayment);
					break;
				case 'compensation':
					$compensation = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sv_refunded_compensation');
					$compensation = str_replace('{{time}}', '', $compensation);
					$compensation = str_replace('{{refunded_amount}}', $this->SofortLib_TransactionData->getAmountRefunded().' &euro;', $compensation);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $compensation);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $compensation);
					break;
				case 'complaint':
					break;
				case 'default':
					break;
			}
			
			return true;
		}
	}
	
	
	/**
	 * Notification for sofortlastschrift
	 * Status reason [not_credited_yet|rejected]
	 * @throws Exception
	 */
	private function setSofortLastschrift() {
		if ($this->SofortLib_TransactionData->isError()) {
			throw new Exception('Error - '.__FILE__.' - '.__LINE__);
		} else {
			$status = $this->SofortLib_TransactionData->getStatus();
			$statusReason = $this->SofortLib_TransactionData->getStatusReason();
			$this->setPaymentStatus($statusReason);
			$this->ShopwareUpdateHelper->updateTimeline($this->transactionId, 0, $status, $statusReason, '', '', '');
			
			switch($statusReason) {
				case 'not_credited_yet':
					$successfulOrder = $this->Snippets->getSnippet('sofort_multipay_finish')->get('sofort_multipay_confirm_invoice2_vorkasse');
					$successfulOrder = str_replace('{{paymentMethodStr}}', $this->paymentMethodString, $successfulOrder);
					$successfulOrder = str_replace('{{time}}', '', $successfulOrder);
					$successfulOrder = str_replace('{{tId}}', '', $successfulOrder);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $successfulOrder.$this->transactionId);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $successfulOrder.$this->transactionId);
					break;
				case 'rejected':
					$snippet = $this->Snippets->getSnippet('sofort_multipay_finish')->get('sofort_multipay_debit_returned2');
					$snippet = str_replace('{{time}}', $this->time, $snippet);
					$returnDebit = $snippet;
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $returnDebit);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $returnDebit);
					break;
				case 'credited':
					$credited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sv.status_credited');
					$credited = str_replace('{{paymentMethodStr}}', $this->paymentMethodString, $credited);
					$credited = str_replace('{{time}}', '', $credited);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $credited);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $credited);
					break;
				case 'refunded':
					$refunded = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_su_refunded');
					$refunded = str_replace('{{time}}', '', $refunded);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $refunded);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $refunded);
					break;
				case 'compensation':
					break;
				case 'default':
					break;
			}
			
			return true;
		}
	}
	
	
	/**
	 * Notification for Lastschrift by sofort
	 * Status reason [not_credited_yet|rejected]
	 * @throws Exception
	 */
	private function setLastschriftBySofort() {
		if ($this->SofortLib_TransactionData->isError()) {
			throw new Exception('Error - '.__FILE__.' - '.__LINE__);
		} else {
			$status = $this->SofortLib_TransactionData->getStatus();
			$statusReason = $this->SofortLib_TransactionData->getStatusReason();
			$this->setPaymentStatus($statusReason);
			$this->ShopwareUpdateHelper->updateTimeline($this->transactionId, 0, $status, $statusReason, '', '', array());
			
			switch($statusReason) {
				case 'not_credited_yet':
					$successfulOrder = $this->Snippets->getSnippet('sofort_multipay_finish')->get('sofort_multipay_confirm_invoice2_vorkasse');
					$successfulOrder = str_replace('{{paymentMethodStr}}', $this->paymentMethodString, $successfulOrder);
					$successfulOrder = str_replace('{{time}}', '', $successfulOrder);
					$successfulOrder = str_replace('{{tId}}', '', $successfulOrder);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $successfulOrder.$this->transactionId);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $successfulOrder.$this->transactionId);
					break;
				case 'rejected':
					$snippet = $this->Snippets->getSnippet('sofort_multipay_finish')->get('sofort_multipay_debit_returned2');
					$snippet = str_replace('{{time}}', $this->time, $snippet);
					$returnDebit = $snippet;
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $returnDebit);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $returnDebit);
					break;
				case 'credited':
					$credited = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sv.status_credited');
					$credited = str_replace('{{paymentMethodStr}}', $this->paymentMethodString, $credited);
					$credited = str_replace('{{time}}', '', $credited);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $credited);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $credited);
					break;
				case 'refunded':
					$refunded = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_su_refunded');
					$refunded = str_replace('{{time}}', '', $refunded);
					// Merchant and customer have to be informed
					$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $refunded);
					$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $refunded);
					break;
				case 'compensation':
					
					break;
				case 'default':
					break;
			}
			return true;
		}
	}
	
	
	/**
	 * 
	 * Notification for iDeal
	 */
	private function setIdeal() {
		$status = $this->SofortLib_ClassicNotification->getStatus();
		$statusReason = $this->SofortLib_ClassicNotification->getStatusReason();
		
		/* race conditions between the user interaction with iDeal (returning to the shop) and notification by sofort.com
		 * might come in asynchroniously, so "first come, first serve"... so user interaction or notification might initiate those tables in database,
		 * whatever comes in first
		 */
		// initiate order tables as notification might be the first occurence where the shop gets a notice of an order
		$this->initiateIDeal();
		
		$this->setPaymentStatus($statusReason);
		$this->ShopwareUpdateHelper->updateTimeline($this->transactionId, 0, $status, $statusReason, '', '', array());
		
		switch($statusReason) {
			case 'not_credited_yet':
				$snippet = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_ideal_status_pending');
				$snippet = str_replace('{{paymentMethodStr}}', $this->paymentMethodString, $snippet);
				$snippet = str_replace('{{time}}', '', $snippet);
				$snippet = str_replace('{{tId}}', $this->transactionId, $snippet);
				$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $snippet);
				$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $snippet);
				break;
			case 'credited':
				$snippet = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sv.status_credited');
				$snippet = str_replace('{{paymentMethodStr}}', $this->paymentMethodString, $snippet);
				$snippet = str_replace('{{time}}', '', $snippet);
				// $this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $snippet);
				$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $snippet);
				break;
			case 'automatic_refund_to_customer':
				$snippet = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_ideal_status_late_succeed');
				// $this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $snippet);
				$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $snippet);
				break;
			case 'not_credited':
				$snippet = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_ideal_status_loss');
				$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $snippet);
				$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $snippet);
				break;
			case 'compensation':
				$snippet = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_ideal_status_compensation');
				$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $snippet);
				$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $snippet);
				break;
			case 'refunded':
				$snippet = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_su_refunded');
				$this->ShopwareUpdateHelper->setCustomerComment($this->transactionId, $snippet);
				$this->ShopwareUpdateHelper->setMerchantComment($this->transactionId, $snippet);
				break;
			default:
				break;
		}
		
	}
	
	
	/**
	 * 
	 * Inititate the payment solution for iDEAL
	 */
	private function initiateIDeal() {
		$sql = 'UPDATE `sofort_orders` SET `transactionId` = ? WHERE `secret` = ?';
		$fields = array(
			$this->transactionId,
			$this->uniqueId,
		);
		$this->Shopware->Db()->query($sql, $fields);
		
		$sql = 'UPDATE `s_order` SET `transactionId` = ? WHERE `transactionId` = ?';
		$fields = array(
			$this->transactionId,
			$this->uniqueId,
		);
		$this->Shopware->Db()->query($sql, $fields);
		
		$sql = 'SELECT ordernumber FROM s_order WHERE transactionId = ?';
		$fields = array(
			$this->transactionId,
		);
		$orderId = $this->Shopware->Db()->fetchOne($sql, $fields);
		
		/*
		$sql = 'UPDATE `sofort_products` SET `order_id` = ? WHERE `transactionId` = ?';
		$fields = array(
			$orderId,
			$this->transactionId,
		);
		return $this->Shopware->Db()->query($sql, $fields);
		*/
		$this->ShopwareUpdateHelper->initOrderTablesAndTimeLine($this->transactionId, $orderId, array('name' => $this->paymentMethod), 0);
	}
	
	
	/**
	 * Update both the tables "sofort_orders" and "s_order" of shopware to set the payment status accordingly
	 * @param unknown_type $status
	 */
	private function setPaymentStatus($status) {
		$sql = 'UPDATE `sofort_orders` SET paymentStatus = ? WHERE `transactionId` = ?';
		$fields = array(
			$status,
			$this->transactionId,
		);
		$this->Shopware->Db()->query($sql, $fields);
		$sql = 'UPDATE `s_order` SET cleared = ? WHERE `transactionId` = ?';
		$paymentCleared = $this->paymentStatus[$this->paymentMethod][$this->statuses[$this->paymentMethod][$this->doStatusMapping($status)]];
		
		$fields = array(
			$paymentCleared,
			$this->transactionId,
		);
		
		$this->Shopware->Db()->query($sql, $fields);
		return true;
	}
	
	
	/**
	 * 
	 * Setter for customer protection
	 * @param boolean $status
	 */
	private function setCustomerProtection($status) {
		$sql = 'UPDATE `sofort_orders` SET customerProtection = ? WHERE `transactionId` = ?';
		$fields = array(
			$status,
			$this->transactionId,
		);
		$this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * Set order status to canceled
	 */
	private function setOrderStatusToCanceled() {
		$sql = 'UPDATE `s_order` SET status = 4 WHERE `transactionId` = ?';
		$fields = array(
			$this->transactionId,
		);
		$this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * Getter for status of invoice
	 */
	private function getStatusOfInvoice() {
		if ($this->testMode) {
			return $this->statusOfInvoice;
		}
		$sql = 'SELECT s.status_id FROM `sofort_status` s
				JOIN sofort_products p on p.id = s.sofort_product_id
				WHERE p.transactionId = ?
				ORDER BY s.date_modified DESC LIMIT 0,1';
		$fields = array(
			$this->transactionId,
		);
		return $this->Shopware->Db()->fetchOne($sql, $fields);
	}
	
	
	/**
	 * 
	 * Getter for timeline of invoice
	 */
	private function getTimelineOfInvoice() {
		$sql = 'SELECT sofort_status.status_id FROM `sofort_status`
				JOIN sofort_products on sofort_products.id = sofort_status.sofort_product_id
				WHERE sofort_products.transactionId = ?
				ORDER BY sofort_status.id DESC';
		$fields = array(
			$this->transactionId,
		);
		$invoiceTimeline = $this->Shopware->Db()->fetchAll($sql, $fields);
		return $invoiceTimeline;
	}
	
	
	/**
	 * 
	 * Getter for amount of invoice
	 */
	private function getAmountOfInvoice() {
		if ($this->testMode) {
			return $this->amountOfInvoice;
		}
		$sql = 'SELECT `invoice_amount` FROM `s_order` WHERE `transactionId` = ?';
		$fields = array(
			$this->transactionId,
		);
		$amount = $this->Shopware->Db()->fetchOne($sql, $fields);
		return number_format($amount, 2);
	}
	
	
	/**
	 * 
	 * Getter for order identifier
	 */
	private function getOrderIdentifier() {
		if ($this->testMode) {
			return $this->orderIdentifier;
		}
		$sql = 'SELECT `id`, `ordernumber` FROM `s_order` WHERE `transactionId` = ?';
		$fields = array(
			$this->transactionId,
		);
		$identifier = $this->Shopware->Db()->fetchAll($sql, $fields);
		return $identifier[0];	// $identifier[0][id] as return value?
	}
	
	
	/**
	 *
	 * Fetches the payment from DB
	 * @param unknown_type $transactionId
	 */
	private function getPayment($secret) {
		if ($this->testMode) return true;
		$sql = 'SELECT COUNT(transactionId) FROM `sofort_orders` WHERE `secret` = ?';
		$fields = array(
			$secret,
		);
		$payment = $this->Shopware->Db()->fetchOne($sql, $fields);
		return $payment;
	}
	
	
	/**
	 * 
	 * Update order addresses
	 */
	private function updateOrderAddresses() {
		$transactionId = $this->SofortLib_TransactionData->getTransaction();
		$invoiceAddress = $this->SofortLib_TransactionData->getInvoiceAddress();
		$shippingAddress = $this->SofortLib_TransactionData->getShippingAddress();
		
		$invoiceSalutation = ($invoiceAddress['salutation'] == 2) ? 'mr' : 'mrs';
		$shippingSalutation = ($invoiceAddress['salutation'] == 2) ? 'mr' : 'mrs';
		
		$sql = 'SELECT userID FROM s_order WHERE transactionID = ?';
		$fields = array(
			$transactionId,
		);
		$userId = $this->Shopware->Db()->fetchOne($sql, $fields);
		
		$invoiceLastname = $invoiceAddress['lastname'];
		$shippingLastname = $shippingAddress['lastname'];
		
		/*
		$sql = 'SELECT company FROM `s_user_billingaddress` WHERE userID = '.$userId;
		$invoiceCompany = $this->Shopware->Db()->fetchOne($sql);
		$sql = 'SELECT company FROM `s_user_shippingaddress` WHERE userID = '.$userId;
		$shippingCompany = $this->Shopware->Db()->fetchOne($sql);
		
		if (!empty($invoiceCompany) && strpos($invoiceLastname, $invoiceCompany) !== false) {
			// remove company name from lastname if present
			$invoiceLastname = substr($invoiceLastname, 0, strpos($invoiceLastname, ' - '.$invoiceCompany));
		}
		if (!empty($shippingCompany) && strpos($shippingLastname, $shippingCompany) !== false) {
			// remove company name from lastname if present
			$shippingLastname = substr($shippingLastname, 0, strpos($shippingLastname, ' - '.$shippingCompany));
		}
		*/
		
		$sql = 'UPDATE s_user_billingaddress 
				SET salutation = ?,
				firstname = ?,
				lastname = ?,
				street = ?,
				streetnumber = ?,
				zipcode = ?,
				city = ?
				WHERE userID = ?
			';
		$fields = array(
			utf8_decode($invoiceSalutation),
			utf8_decode($invoiceAddress['firstname']),
			utf8_decode($invoiceLastname),
			utf8_decode($invoiceAddress['street']),
			utf8_decode($invoiceAddress['street_number']),
			utf8_decode($invoiceAddress['zipcode']),
			utf8_decode($invoiceAddress['city']),
			$userId,
		);
		$this->Shopware->Db()->query($sql, $fields);
		$sql = 'UPDATE s_user_shippingaddress 
				SET salutation = ?,
				firstname = ?,
				lastname = ?,
				street = ?,
				streetnumber = ?,
				zipcode = ?,
				city = ?
				WHERE userID = ?
			';
		$fields = array(
			utf8_decode($shippingSalutation),
			utf8_decode($shippingAddress['firstname']),
			utf8_decode($shippingLastname),
			utf8_decode($shippingAddress['street']),
			utf8_decode($shippingAddress['street_number']),
			utf8_decode($shippingAddress['zipcode']),
			utf8_decode($shippingAddress['city']),
			$userId,
		);
		$this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * Getter for tax id
	 * @param float $value
	 */
	private function getTaxIdByValue($value) {
		$sql = 'SELECT id FROM s_core_tax WHERE tax = ? ';
				$fields = array(
					$value,
				);
		return $this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * Get differences between two article arrays
	 * @param array $firstArray
	 * @param array $secondArray
	 */
	private function getArticleDifferences($firstArray, $secondArray) {
		$firstProductKeys = $this->getProductKeysFromPnagArticle($firstArray);
		$secondProductKeys = $this->getProductKeysFromPnagArticle($secondArray);
		$differences = array_diff($firstProductKeys, $secondProductKeys);
		return $differences;
	}
	
	
	/**
	 * 
	 * Getter for product keys of array
	 * @param array $array
	 */
	function getProductKeysFromPnagArticle($array) {
		$productKeys = array();
		
		foreach ($array as $element) {
			array_push($productKeys, $element->product_number);
		}
		
		return $productKeys;
	}
	
	
	/**
	 * 
	 * fetch shop items
	 * @param string $orderId
	 */
	private function fetchShopItems($orderId) {
		$sql = 'SELECT od.*, tax.tax,
			(SELECT invoice_shipping FROM s_order WHERE s_order.ordernumber = ?) as shipping_costs
			FROM s_order_details od
			JOIN s_order o on o.id = od.orderID
			JOIN s_core_tax tax on tax.id = od.taxID
			WHERE od.ordernumber = ?';
		$fields = array(
			$orderId,
			$orderId,
		);
		return $this->Shopware->Db()->fetchAll($sql, $fields);
	}
	
	
	/**
	 * 
	 * Getter for order items
	 * @param string $orderId
	 */
	private function getOrderItemsFromShopware($orderId) {
		if ($this->testMode === false) {
			// first, get structured data from shopware
			$shopItems = $this->fetchShopItems($orderId);
		} else {
			$shopItems = $this->shopItems;
		}
		
		// make some nice objects to have a little more convenient base to compare
		$pnagItems = array();
		$PnagArticle = null;
		
		foreach ($shopItems as $article) {
			$PnagArticle = new PnagArticle($article['id'], utf8_encode($article['articleordernumber']), 0, utf8_encode($article['name']), '', $article['quantity'], $article['price'], $article['tax']);
			array_push($pnagItems, $PnagArticle);
		}
		
		// some shipping costs as well
		if (!empty($shopItems[0]['shipping_costs'])) {
			$shippingSnippet = $this->Snippets->getSnippet("sofort_multipay_checkout")->get("shipping_costs");
			$shippingSnippetShort = 'shpmnt_vk';
			$PnagArticle = new PnagArticle($shopItems[0]['orderID'], $shippingSnippetShort, 0, $shippingSnippet, '', 1, $shopItems[0]['shipping_costs'], $shopItems[0]['tax']);
			array_push($pnagItems, $PnagArticle);
		}
		
		$this->notify('pnagItems', $pnagItems);
		return $pnagItems;
	}
}
