<?php
/**
 *
 * This controller provides a callback for notifications coming from sofort.com
 * It provides an entry point for notifications via XML
 *
 * $Date: 2012-12-06 14:45:56 +0100 (Thu, 06 Dec 2012) $
 * @version sofort 1.0  $Id: Helper.php 5824 2012-12-06 13:45:56Z dehn $
 * @author SOFORT AG http://www.sofort.com (f.dehn@sofort.com)
 * @package Shopware 3.5.x, sofort.com
 *
 */
require_once(dirname(__FILE__).'/Observable.php');

/**
 *
 * Helper Class
 *
 * $Date: 2012-12-06 14:45:56 +0100 (Thu, 06 Dec 2012) $
 * @version sofort 1.0  $Id: Helper.php 5824 2012-12-06 13:45:56Z dehn $
 * @author SOFORT AG http://www.sofort.com (f.dehn@sofort.com)
 * @package Shopware 3.5.x, sofort.com
 *
 */
class ShopwareUpdateHelper implements Observable{
	
	private $Shopware = null;
	
	private $Snippets = null;
	
	private $dateFormat = 'd.m.Y H:i:s';
	
	private $observers = array();
	
	/**
	 * 
	 * Constructor for Helper
	 * @param Object $Shopware
	 */
	function __construct($Shopware) {
		$this->Shopware = $Shopware;
		$this->Snippets = $this->Shopware->Snippets();
	}
	
	
	/**
	 * 
	 * Sets an entry to the timeline
	 * @param string $transactionId
	 * @param int $invoiceStatusId
	 * @param string $status
	 * @param string $statusReason
	 * @param string $invoiceStatus
	 * @param string $invoiceObjection
	 * @param array $items
	 */
	public function updateTimeline($transactionId, $invoiceStatusId, $status, $statusReason, $invoiceStatus, $invoiceObjection, $items = array()) {
		
		$sql = 'SELECT id FROM sofort_products WHERE transactionId = ?';
		$fields = array(
			$transactionId,
		);
		$orderId = $this->Shopware->Db()->fetchOne($sql, $fields);
		$sql = "INSERT INTO sofort_status
				(sofort_product_id, status_id, status, status_reason, invoice_status, invoice_objection, items)
				VALUES (?, ?, ?, ?, ?, ?, ?)
		";
		
		$fields = array(
			$orderId,
			$invoiceStatusId,
			$status,
			$statusReason,
			$invoiceStatus,
			$invoiceObjection,
			$items,
		);
		
		$this->Shopware->Db()->query($sql, $fields);
		
		
		$sql = 'UPDATE sofort_orders set paymentStatus = ? WHERE transactionId = ?';
		return $this->Shopware->Db()->query($sql, array($statusReason, $transactionId));
	}
	
	
	/**
	 * 
	 * Initializa order tables + timeline
	 * @param string $transactionId
	 * @param string $orderId
	 * @param string $paymentMethod
	 * @param float $amount
	 * @throws Exception
	 */
	public function initOrderTablesAndTimeLine($transactionId, $orderId, $paymentMethod, $amount) {
		// if($transactionId == '' || $orderId == '' || $paymentMethod == '') throw new Exception('Error occured '.__CLASS__.' '.__LINE__);
		$pendingString = '';
		
		switch($paymentMethod['name']) {
			case 'sofortueberweisung_multipay' :
			case 'su':
				$pendingString = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_su_pending');
				break;
			case 'vorkassebysofort_multipay' :
			case 'sv':
				$pendingString = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sv_pending');
				break;
			case 'sofortrechnung_multipay' :
			case 'sr':
				$pendingString = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sr_pending');
				break;
			case 'sofortlastschrift_multipay' :
			case 'sl':
				$pendingString = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_sl_pending');
				break;
			case 'lastschriftbysofort_multipay' :
			case 'ls':
				$pendingString = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_multipay_ls_pending');
				break;
			case 'iDeal':
				$pendingString = $this->Snippets->getSnippet('sofort_multipay_bootstrap')->get('sofort_ideal_status_pending');
				$pendingString = str_replace('{{paymentMethodStr}}', $paymentMethod['name'], $pendingString);
				$pendingString = str_replace('{{time}}', '', $pendingString);
				$pendingString = str_replace('{{tId}}', $transactionId, $pendingString);
				break;
			default:
				throw new Exception('Session Timeout - Order tables + timeline could not be updated correctly: '.__CLASS__.' '.__LINE__);
		}
		$this->initStatusAndProductTable($orderId, $transactionId, $amount, $pendingString);
	}
	
	
	/**
	 * 
	 * Fetch the details from "s_core_paymentmeans" by short name like sr, su, ...
	 */
	public function getPaymentDetails($paymentMethod) {
		$sql = 'SELECT name, description, additionaldescription FROM `s_core_paymentmeans` WHERE `name` = ? ';
		$fields = array(
			$paymentMethod,
		);
		$paymentDetails = $this->Shopware->Db()->fetchAll($sql, $fields);
		return $paymentDetails[0];
	}
	
	/**
	 * 
	 * Given a transaction by transactionId, secret and payment method one can easily check if an unsaved order has been saved 
	 * and never been written to the database
	 * @param string $secret
	 * @param string $transactionId
	 * @param string $paymentMethod
	 * @deprecated
	 */
	public function checkIfUnsavedOrderExistAndSave($secret, $transactionId, $paymentMethod) {
		$sql = 'SELECT * FROM `sofort_temp_orders` WHERE `secret` = "'.$secret.'"';
		$order = $this->Shopware->Db()->fetchAll($sql);
		
		if(!empty($order)) {
			$orderNumber = $order[0]['ordernumber'];
			$cartContents = unserialize($order[0]['cart_content']);
			$userData = unserialize($order[0]['user_data']);
			$customerComment = $order[0]['customer_content'];
			$comment = $order[0]['comment'];
			$orderTime = $order[0]['order_time'];
			$amount = number_format(str_replace(',', '.', $cartContents['Amount']), 2);
			
			$this->initOrderTablesAndTimeLine($transactionId, $orderNumber, $paymentMethod, $amount);
			$this->saveOrder($transactionId, $orderNumber, $orderTime, $cartContents, $userData, $customerComment, $comment);
			$this->deleteTemporaryOrder($secret);
			$userId = $userData['billingaddress']['userID'];
			return $this->clearCart($userId);
		}
		return false;
	}
	
	
	/**
	 * 
	 * Clear the cart s_order_basket
	 * @param int $userId
	 */
	private function clearCart($userId) {
		$sql = 'DELETE FROM s_order_basket WHERE userID = ?';
		$fields = array(
			(int)$userId,
		);
		return $this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * save the serialized cart
	 * @param int $orderNumber
	 * @param string $unique
	 * @param array $cartContents
	 * @param array $userData
	 * @param string $customerComment
	 * @param string $comment
	 * @deprecated
	 */
	public function saveAndSerializeCartContents($orderNumber, $unique, $cartContents, $userData, $customerComment, $comment) {
		$sql = 'INSERT INTO sofort_temp_orders (ordernumber, order_time, secret, cart_content, user_data, customer_comment, comment) VALUES (?, ?, ?, ?, ?, ?, ?)';
		$fields = array(
			$orderNumber,
			date('Y-m-d H:i:s', time()),
			$unique,
			serialize($cartContents),
			serialize($userData),
			$customerComment,
			$comment,
		);
		$this->Shopware->Db()->query($sql, $fields);
		return true;
	}
	
	
	/**
	 * 
	 * Delete a temporary order given by secret
	 * @param string $secret
	 * @deprecated
	 */
	public function deleteTemporaryOrder($secret) {
		$sql = 'DELETE FROM sofort_temp_orders WHERE secret = ?';
		$fields = array(
			$secret,
		);
		$this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * Init two tables: sofort_products and sofort_status
	 * @param int $orderId
	 * @param string $transactionId
	 * @param float $amount
	 * @param string $comment
	 */
	private function initStatusAndProductTable($orderId, $transactionId, $amount, $comment) {
		
		if($this->checkIfOrderTablesAlreadyInitiated($orderId)) {
			return true;
		}
		
		$sql = "INSERT INTO `sofort_products`
				(`order_id`, `transactionId`, `amount`, `comment`)
					VALUES (?, ?, ?, ?);
		";
		$time = date($this->dateFormat, time());
		$fields = array(
			$orderId,
			$transactionId,
			$amount,
			$time.' - '.$comment,
		);
		$this->Shopware->Db()->query($sql, $fields);
		$productId = $this->Shopware->Db()->lastInsertId();
		$sql = 'INSERT INTO sofort_status
				(sofort_product_id, status_id, status, status_reason, invoice_status, invoice_objection, items, comment)
				VALUES (?, ?, "empty", "empty", "empty", "empty", ?, ?)
		';
		// use cart content and invoice status only for Rechnung by SOFORT
		$invoiceStatus = ($this->paymentMethod['name'] == 'sofortrechnung_multipay') ? $this->PnagInvoice->getInvoiceStatus() : '';
		$invoiceItems = ($this->paymentMethod['name'] == 'sofortrechnung_multipay') ? serialize($this->PnagInvoice->getItems()) : '';
		$fields = array(
			$productId,
			$invoiceStatus,
			$invoiceItems,
			$comment,
		);
		$this->Shopware->Db()->query($sql, $fields);
		return true;
	}
	
	/**
	 * 
	 * Checks if order tables are already initiated
	 * @param string $orderId
	 */
	private function checkIfOrderTablesAlreadyInitiated($orderId) {
		// TODO:check if no entrys exist
		$sql = 'SELECT COUNT(*) as count FROM sofort_products WHERE order_id = ?';
		$fields = array(
			$orderId,
		);
		$count = $this->Shopware->Db()->fetchOne($sql, $fields);
		return ($count['count'] > 0) ? 1 : 0;
	}
	
	/**
	 * 
	 * Updates the cart existing in the shop. Takes an invoice object as a parameter and returns true|false
	 * @param PnagInvoice $PnagInvoice
	 */
	public function updateCart(PnagInvoice $PnagInvoice) {
		$transactionId = $PnagInvoice->getTransactionId();
		$orderIdentifier = $this->getOrderIdentifier($transactionId);
		$orderId = $orderIdentifier['ordernumber'];
		
		// normalize articleId itemId|articleNumber
		$PnagInvoice = $this->normalizeArticles($PnagInvoice);
		$boughtArticles = $this->fetchArticlesOfOrder($transactionId);
		
		// walk through every bought article and see what quantity has changed since the last update
		// set all stocks and seller statistics back accordingly
		foreach ($boughtArticles as $article) {
			foreach($PnagInvoice->getItems() as $invoiceItem) {
				if($invoiceItem->itemId == $article['articleID']) {
					if($invoiceItem->quantity > $article['quantity']) {
						$this->doRestockAndResetSoldArticle($article['articleID'], $invoiceItem->quantity);
					} elseif($invoiceItem->quantity < $article['quantity']) {
						$difference = $article['quantity'] - $invoiceItem->quantity;
						$this->doRestockAndResetSoldArticle($article['articleID'], $difference);
					} else {
						continue;
					}
				}
			}
		}
		
		// update and delete articles
		$this->updateSofortProductItems($orderId, $PnagInvoice, $transactionId);
		$this->updateOrder($orderId, $PnagInvoice, $transactionId);
		$this->updateShipping($orderId, $PnagInvoice, $transactionId);
		
		return $this->updateOrderDetails($orderId, $PnagInvoice);
	}
	
	
	/**
	 * 
	 * A wrapper for setting stock and statistics
	 * @param int $articleId
	 * @param number $quantity 
	 */
	private function doRestockAndResetSoldArticle($articleId, $quantity) {
		$this->restockArticle($articleId, $quantity);
		$this->resetSoldArticle($articleId, $quantity);
	}
	
	
	/**
	 * 
	 * Reset the sales number
	 * @param int $articleId
	 * @param int $quantity
	 */
	private function resetSoldArticle($articleId, $quantity) {
		$sql = 'UPDATE s_articles_details SET sales = sales - ? WHERE articleID = ?';
		$fields = array(
			$quantity,
			$articleId,
		);
		return $this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * Puts articles back into stock
	 * @param string $articleId
	 * @param float $quantity
	 */
	private function restockArticle($articleId, $quantity) {
		$sql = 'UPDATE s_articles_details SET instock = instock + ? WHERE articleID = ?';
		$fields = array(
			$quantity,
			$articleId,
		);
		return $this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * Puts articles back into stock
	 * @param string $transactionId
	 */
	public function restockCanceledOrder($transactionId) {
		$articlesToRestock = $this->fetchArticlesOfOrder($transactionId);
		
		if (is_array($articlesToRestock) && !empty($articlesToRestock)) {
			foreach($articlesToRestock as $article) {
				$this->doRestockAndResetSoldArticle($article['articleID'], $article['quantity']);
			}
		} else {
			return false;
		}
		
		return true;
	}
	
	
	/**
	 * 
	 * Get an order's articles by transactionId
	 * @param string $transactionId
	 */
	private function fetchArticlesOfOrder($transactionId) {
		$sql = 'SELECT od.articleID, od.quantity FROM `s_order` o
				JOIN s_order_details od ON o.id = od.orderID
				WHERE o.transactionID = ?';
		$fields = array(
			$transactionId,
		);
		return $this->Shopware->Db()->fetchAll($sql, $fields);
	}
	
	
	/**
	 * 
	 * Given constructed articleIds lile "articleId|example", normalize it to "articleId" and return the given invoice object
	 * @param object $PnagInvoice
	 */
	private function normalizeArticles($PnagInvoice) {
		$newItems = array();
		foreach($PnagInvoice->getItems() as $item) {
			$newItems[] = $item;
			$itemId = explode('|', $item->itemId);
			$item->itemId = $itemId[0];
		}
		$PnagInvoice->setItems($newItems);
		return $PnagInvoice;
	}
	
	
	/**
	 * get the order number as well as the order id
	 * @param string $transactionId
	 */
	private function getOrderIdentifier($transactionId) {
		$sql = 'SELECT `id`, `ordernumber` FROM `s_order` WHERE `transactionId` = ?';
		$fields = array(
			$transactionId,
		);
		$identifier = $this->Shopware->Db()->fetchAll($sql, $fields);
		return $identifier[0];
	}
	
	
	/**
	 * 
	 * update sofort_products, set new amount here
	 * @param unknown_type $orderId
	 * @param PnagInvoice $PnagInvoice
	 * @param unknown_type $transactionId
	 */
	private function updateSofortProductItems($orderId, PnagInvoice $PnagInvoice, $transactionId) {
		$sql = "UPDATE `sofort_products` SET `amount` = ? WHERE transactionId = ?";
		$fields = array(
			$PnagInvoice->getAmount(),
			$transactionId,
		);
		$this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * Update order, set quantity of items and them
	 * @param int $orderId
	 * @param PnagInvoice $PnagInvoice
	 */
	private function updateOrder($orderId, PnagInvoice $PnagInvoice, $transactionId) {
		$invoiceArticles = $PnagInvoice->getItems();
		$amountNet = 0;
		
		foreach ($invoiceArticles as $item) {
			$amountNet += $item->quantity * $item->unitPrice * 100 / ((100 + $item->tax));
		}
		
		$sql = 'UPDATE `s_order` SET `invoice_amount` = ?, `invoice_amount_net` =  ? WHERE `transactionID` = ?';
		$fields = array(
			$PnagInvoice->getAmount(),
			round($amountNet, 2),
			$transactionId,
		);
		$this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * Update shipping costs
	 * @param string $orderId
	 * @param object $PnagInvoice
	 * @param string $transactionId
	 */
	private function updateShipping($orderId, $PnagInvoice, $transactionId) {
		$invoiceArticles = $PnagInvoice->getItems();
		$shippingNet = 0;
		$shipping = 0;
		
		foreach ($invoiceArticles as $item) {
			
			if($item->itemId == 'shpmntvk') {
				$shippingNet = $this->makeNetFromGross($item->unitPrice, $item->tax);
				$shipping = $item->unitPrice;
			}
			
		}
		
		$sql = 'UPDATE `s_order` SET `invoice_shipping` = ?, `invoice_shipping_net` = ? WHERE `transactionID` = ?';
		$fields = array(
			$shipping,
			$shippingNet,
			$transactionId
		);
		$this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * Calculate the net amount with a given gross amount
	 * @param unknown_type $value
	 * @param unknown_type $tax
	 */
	public function makeNetFromGross($value, $tax, $numberFormat = 2) {
		$value = $value / (100 + $tax) * 100;
		return number_format($value, $numberFormat);
	}
	
	
	/**
	 * Get the latest invoice state
	 * @param string $transactionId
	 */
	private function getLatestInvoiceState($transactionId) {
		$sql = "SELECT st.* FROM sofort_status st
			JOIN sofort_products pr on pr.id = st.sofort_product_id
			WHERE pr.transactionId = ?'
			order by st.date_modified DESC limit 0,1
		";
		$fields = array(
			$transactionId,
		);
		$actualState = $this->Shopware->Db()->fetchAll($sql, $fields);
		return $actualState[0];
	}
	
	
	/**
	 * 
	 * Update core table to new incoming values in case a cart/basket has been edited
	 * Handle shipping as well as any additional values
	 * @param int $orderId
	 * @param PnagInvoice $PnagInvoice
	 */
	private function updateOrderDetails($orderId, PnagInvoice $PnagInvoice) {
		$invoiceArticles = array();
		$newItems = array();
		$shopArticles = $this->getOrderItemsFromShopware($orderId);
		$invoiceArticles = $PnagInvoice->getItems();
		$this->synchInvoiceItems($orderId, $shopArticles, $invoiceArticles);
		$i = 0;
		
		foreach ($invoiceArticles as $item) {
			$itemId = $item->getItemId();
			$newItems[0]['id'] = (!empty($itemId)) ? $itemId : '';
			$newItems[0]['quantity'] = $item->getQuantity();
			$newItems[0]['price'] = $item->getUnitPrice();
			$newItems[0]['productNumber'] = $item->getProductNumber();
			
			if (!empty($itemId)) {
				$sql = 'UPDATE `s_order_details` SET `quantity` = ?, price = ? WHERE articleID = ? AND ordernumber = ? AND articleordernumber = ?';
				$fields = array(
					$newItems[0]['quantity'],
					$newItems[0]['price'],
					$itemId,
					$orderId,
					$newItems[0]['productNumber'],
				);
				$this->Shopware->Db()->query($sql, $fields);
			}
			
			$i++;
		}
		
		return true;
	}
	
	
	/**
	 * 
	 * For testing purposes
	 * @param array $shopItems
	 * @param array $invoiceItems
	 */
	public function testSynchInvoiceItems($shopItems, $invoiceItems) {
		return $this->synchInvoiceItems('08154711', $shopItems, $invoiceItems);
	}
	
	
	/**
	 * 
	 * Add or remove articles by keeping the request's invoice in synch with shop's invoice
	 * @param int $orderId
	 * @param array $shopItems
	 * @param array $invoiceItems
	 * @return boolean
	 */
	private function synchInvoiceItems($orderId, $shopItems, $invoiceItems) {
		$missingItems = $this->getItemDifferences($shopItems, $invoiceItems);
		$this->notify('missingItems', $missingItems);
		$newItems = $this->getItemDifferences($invoiceItems, $shopItems);
		$shippingSnippet = 'shpmntvk';	// had to choose sth similar to shipment/versandkosten, there's no article representing shipment in database (s_order_details)
		
		// some items have been deleted
		if (!empty($missingItems)) {
			foreach ($missingItems as $item) {
				$sql = 'DELETE FROM s_order_details WHERE ordernumber = ? AND articleordernumber = ? ';
				$fields = array(
					$orderId,
					$item,
				);
				$this->Shopware->Db()->query($sql, $fields);
				
				if($item === $shippingSnippet) {
					$sql = 'UPDATE s_order SET invoice_shipping = 0, invoice_shipping_net = 0 WHERE ordernumber = ? ';
					$fields = array(
						$orderId,
					);
					$this->Shopware->Db()->query($sql, $fields);
				}
			}
			// some items were set back from external
		} elseif(!empty($newItems)) {
			foreach($invoiceItems as $item) {
				if($item->getProductNumber() === $shippingSnippet) {
					$sql = 'UPDATE s_order SET invoice_shipping = ?, invoice_shipping_net = ? WHERE ordernumber = ? ';
					$fields = array(
						$item->getUnitPrice(),
						number_format(($item->getUnitPrice() / (1+ $item->getTax() / 100)), 2),
						$orderId,
					);
					$this->Shopware->Db()->query($sql, $fields);
				} else {
					$modus = 0;
					
					if($item->getUnitPrice() == 0.00 && $item->getTax() == 0.00) {
						$modus = 1;
					}
					// if there ain't an article yet, add it to core table
					if(!$this->articleExists($orderId, $item->getItemId(), $item->getProductNumber())) {	
						$sql = 'INSERT INTO s_order_details 
								(orderID, ordernumber, articleID, articleordernumber, price, quantity, name, modus, taxID)
								VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)
						';
						$fields = array(
							$this->getOrderIdByOrderNumber($orderId),
							$orderId,
							$item->getItemId(),
							utf8_decode($item->getProductNumber()),
							$item->getUnitPrice(),
							$item->getQuantity(),
							utf8_decode($item->getTitle()),
							$modus,
							$this->getTaxIdByValue((int)$item->getTax()),
						);
						$this->Shopware->Db()->query($sql, $fields);
					}
				}
				
			}
			
		}
		
		return true;
	}
	
	
	/**
	 * 
	 * Does an article exist
	 * @param string $orderNumber
	 * @param string $articleId
	 * @param string $articleOrdernumber
	 */
	private function articleExists($orderNumber, $articleId, $articleOrdernumber) {
		$sql = 'SELECT COUNT(*) FROM s_order_details WHERE ordernumber = ? AND articleID = ? AND articleordernumber = ?';
		$fields = array(
			$orderNumber,
			$articleId,
			$articleOrdernumber,
		);
		return $this->Shopware->Db()->fetchOne($sql, $fields);
	}
	
	
	/**
	 * 
	 * Get the orderId by given order number
	 * @param string $orderNumber
	 */
	private function getOrderIdByOrderNumber($orderNumber) {
		$sql = 'SELECT id FROM s_order WHERE ordernumber = ?';
		$fields = array(
			$orderNumber,
		);
		return $this->Shopware->Db()->fetchOne($sql, $fields);
	}
	
	
	/**
	 * 
	 * Get taxId by value
	 * @param float $value
	 */
	private function getTaxIdByValue($value) {
		$sql = 'SELECT id FROM s_core_tax WHERE tax = ? ';
		$fields = array(
			$value,
		);
		return $this->Shopware->Db()->fetchOne($sql, $fields);
	}
	
	
	/**
	 * 
	 * Determine the differences between two arrays carrying product keys
	 * @param array $firstArray
	 * @param array $secondArray
	 */
	private function getItemDifferences($firstArray, $secondArray) {
		$firstProductKeys = $this->getProductKeysFromItem($firstArray);
		$secondProductKeys = $this->getProductKeysFromItem($secondArray);
		$differences = array_diff($firstProductKeys, $secondProductKeys);
		return $differences;
	}
	
	
	/**
	 * 
	 * Get all the product keys from an item
	 * @param unknown_type $array
	 * @return array
	 */
	function getProductKeysFromItem($array) {
		$productKeys = array();
		
		foreach ($array as $element) {
			array_push($productKeys, $element->productNumber);
		}
		
		return $productKeys;
	}
	
	
	/**
	 * 
	 * Get all the ordered items by given orderId
	 * @param string $orderId
	 */
	private function getOrderItemsFromShopware($orderId) {
		$shopItems = $this->fetchShopItems($orderId);
		
		// make some nice objects to have a little more convenient base to compare
		$pnagItems = array();
		$PnagArticle = null;
		
		foreach ($shopItems as $article) {
			$PnagArticle = new PnagArticle($article['articleID'], utf8_encode($article['articleordernumber']), 0, utf8_encode($article['name']), '', $article['quantity'], number_format($article['price'], 2), number_format($article['tax'], 2));
			array_push($pnagItems, $PnagArticle);
		}
		
		// some shipping costs as well
		if (!empty($shopItems[0]['shipping_costs'])) {
			$shippingSnippet = $this->Snippets->getSnippet("sofort_multipay_checkout")->get("shipping_costs");
			$shippingSnippetShort = 'shpmntvk';
			$PnagArticle = new PnagArticle($shopItems[0]['orderID'], $shippingSnippetShort, 0, $shippingSnippet, '', 1, $shopItems[0]['shipping_costs'], $shopItems[0]['tax']);
			array_push($pnagItems, $PnagArticle);
		}
		
		return $pnagItems;
	}
	
	
	/**
	 * 
	 * Fetch all shop items by given orderId
	 * @param string $orderId
	 */
	private function fetchShopItems($orderId) {
		$sql = 'SELECT od.*, tax.tax,
			(SELECT invoice_shipping FROM s_order WHERE s_order.ordernumber = ?) as shipping_costs
			FROM s_order_details od
			JOIN s_order o on o.id = od.orderID
			JOIN s_core_tax tax on tax.id = od.taxID
			WHERE od.ordernumber = ?';
		$fields = array(
			$orderId,
			$orderId,
		);
		$elements = $this->Shopware->Db()->fetchAll($sql, $fields);
		return $elements;
	}
	
	
	/**
	 * sets the comment for the merchant, visible for backend users only, overwrite if $overwrite is true
	 * @param string $transactionId
	 * @param string $transactionStatus
	 * @param string $invoiceStatus
	 * @param boolean $overwrite
	 */
	public function setMerchantComment($transactionId, $transactionStatus, $invoiceStatus = '', $overwrite = false) {
		
		if(empty($invoiceStatus)) {
			$comment = $transactionStatus;
		} else {
			$transactionStateLabel = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.InvoiceStateLabel');
			$invoiceStateLabel = $this->Snippets->getSnippet('sofort_multipay_backend')->get('sr.transactionStateLabel');
			$comment = $transactionStateLabel.' '.$transactionStatus.' '.$invoiceStateLabel.' '.$invoiceStatus;
		}
		
		$this->setInternalComment($transactionId, $comment, $overwrite);
		$this->updateTimelineComment($transactionId, $comment);
	}
	
	
	/**
	 * 
	 * Set a comment to last updated status
	 * @param transactionId $transactionId
	 * @param string $comment
	 */
	private function updateTimelineComment($transactionId, $comment) {
		$sql = 'SELECT MAX(sofort_status.id) FROM sofort_status, sofort_products WHERE sofort_status.sofort_product_id = sofort_products.id AND sofort_products.transactionId = ?';
		$fields = array($transactionId);
		$lastStatusId = $this->Shopware->Db()->FetchOne($sql, $fields);
		$sql = 'UPDATE sofort_status  
				JOIN sofort_products on sofort_products.id = sofort_status.sofort_product_id
				SET sofort_status.comment = ?
				WHERE sofort_status.id = '.$lastStatusId.'';
		$fields = array(
			$comment,
		);
		return $this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * Set a comment visible to the customer only, overwrite if $overwrite is true
	 * @param string $transactionId
	 * @param string $comment
	 * @param boolean $overwrite
	 */
	public function setCustomerComment($transactionId, $comment, $overwrite = false) {
		$this->setExternalComment($transactionId, $comment, $overwrite);
	}
	
	
	/**
	 * 
	 * Set a comment visible to backend users only
	 * @param string $transactionId
	 * @param string $comment
	 * @param boolean $overwrite
	 */
	private function setInternalComment($transactionId, $comment, $overwrite = false) {
		$sql = 'SELECT `comment` FROM `sofort_products` WHERE `transactionId` = ?';
		$fields = array(
			$transactionId,
		);
		$oldComment = $this->Shopware->Db()->fetchOne($sql, $fields);
		$time = date($this->dateFormat, time());
		$comment = $time.' - '.$comment;
		$newComment = (!$overwrite) ? $oldComment."\n".$comment : $comment;
		$sql = 'UPDATE `sofort_products` SET `comment` = "'.$newComment.'", `date_modified` = NOW() WHERE `transactionId` = ?';
		$fields = array(
			$transactionId,
		);
		return $this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * Set the order's comment according to the message sent by notification
	 * @param string $transactionId
	 * @param string $comment
	 * @param boolean $overwrite overwrite the old comments, set the current one as new
	 */
	private function setExternalComment($transactionId, $comment, $overwrite = false) {
		$oldComment = $newComment = '';
		
		if(!$overwrite) { // if false, old comment is not being overwritten
			$sql = 'SELECT `comment` FROM `s_order` WHERE `transactionID` = ?';
			$fields = array(
				$transactionId,
			);
			$oldComment = $this->Shopware->Db()->FetchOne($sql, $fields);
		}
		
		$actDate = date($this->dateFormat, time());
		// add new comment at the end of the old one
		$newComment = (empty($oldComment)) ? $actDate.' - '.$comment: $oldComment.'<br />'.$actDate.' - '.$comment;
		$newComment = str_replace('{{paymentMethodStr}}', $this->paymentMethodString, $newComment);
		$newComment = str_replace('{{tId}}', '', $newComment);
		$newComment = str_replace('{{time}}', '', $newComment);
		// update the order comment
		$sql = 'UPDATE `s_order` SET `comment` = ? WHERE `transactionID` = ?';
		$fields = array(
			$newComment,
			$transactionId,
		);
		$this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * 
	 * Save the order s_order
	 * @param string $transactionId
	 * @param string $orderNumber
	 * @param string $orderTime
	 * @param array $cartContents
	 * @param array $userData
	 * @param string $customerComment
	 * @param string $comment
	 */
	public function saveOrder($transactionId, $orderNumber, $orderTime, $cartContents, $userData, $customerComment, $comment) {
		
		if(empty($customerComment)) {
			$customerComment = '';
		}
		
		$sql = 'INSERT INTO s_order 
				(ordernumber, userID, invoice_amount, invoice_amount_net, invoice_shipping, 
				invoice_shipping_net, ordertime, status, cleared, paymentID, transactionID, 
				comment, customercomment, internalcomment, net, taxfree, partnerID, temporaryID, 
				referer, cleareddate, trackingcode, language, dispatchID, currency, 
				currencyFactor, subshopID, o_attr1, o_attr2, o_attr3, o_attr4, o_attr5, o_attr6, remote_addr)
				VALUES
				(?, ?, ?, ?, ?, ?, ? ,? ,? ,? ,?, ? ,? ,? ,? ,? ,? ,? ,? ,? ,? ,? ,? ,? ,? ,? ,? ,? ,? ,? ,? ,? ,?);
		';
		$internalComment = $net = $taxFree = $partnerId = $temporaryId = $referer = $clearedDate = $trackingCode = $language = $dispatchId = $currency = $currencyFactor = $subshopId = $oAttr1 = $oAttr2 = $oAttr3 = $oAttr4 = $oAttr5 = $oAttr6 = $remoteAddr = '';
		$amount = str_replace(',', '.', $cartContents['Amount']);
		$amountNet = str_replace(',', '.', $cartContents['AmountNet']);
		$shipping = str_replace(',', '.', $cartContents['sShippingcostsWithTax']);
		$shippingNet = str_replace(',', '.', $cartContents['sShippingcostsNet']);
		// add shipping if available
		$amount = $amount + $shipping;
		$amountNet = $amountNet + $shippingNet;
		$fields = array(
			$orderNumber,
			$userData['billingaddress']['id'],
			number_format($amount, 2),
			number_format($amountNet, 2),
			number_format($shipping, 2),
			number_format($shippingNet, 2),
			$orderTime,
			0,	// status
			17,	// cleared
			$userData['additional']['payment'][id],
			$transactionId,
			$comment,
			$customerComment,
			$internalComment,
			$net,
			$taxFree,
			$partnerId,
			$temporaryId,
			$referer,
			$clearedDate,
			$trackingCode,
			$language,
			$dispatchId,
			$currency,
			1, //$currencyFactor,
			$subshopId,
			$oAttr1,
			$oAttr2,
			$oAttr3,
			$oAttr4,
			$oAttr5,
			$oAttr6,
			$remoteAddr,
		);
		$this->Shopware->Db()->query($sql, $fields);
		
		$orderId = $this->Shopware->Db()->lastInsertId();
		
		$sql = 'INSERT INTO `s_order_details` 
				(`orderID`, `ordernumber`, `articleID`, `articleordernumber`, `price`, `quantity`, 
				`name`, `status`, `shipped`, `shippedgroup`, `releasedate`, `modus`, `esdarticle`, 
				`taxID`, `config`, `od_attr1`, `od_attr2`, `od_attr3`, `od_attr4`, `od_attr5`, `od_attr6`) 
				VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?);';
		foreach($cartContents['content'] as $item) {
			$fields = array(
				$orderId,
				$orderNumber,
				$item['articleID'],
				$item['ordernumber'],
				number_format(str_replace(',', '.', $item['price']), 2),
				$item['quantity'],
				$item['articlename'],
				0,
				0,
				0,
				'',
				$item['modus'],
				0,
				$item['taxID'],
				$item['config'],
				$item['ob_attr1'],
				$item['ob_attr2'],
				$item['ob_attr3'],
				$item['ob_attr4'],
				$item['ob_attr5'],
				$item['ob_attr6'],
			);
			$this->Shopware->Db()->query($sql, $fields);
		}
		
		$sql = 'DELETE FROM s_order WHERE paymentID = ? AND ordernumber = 0';
		$fields = array(
			$userData['additional']['payment'][id],
		);
		$this->Shopware->Db()->query($sql, $fields);
	}
	
	
	/**
	 * Attach observer (object)
	 * @see Observable::attach()
	 */
	public function attach(Observer $Observer) {
		array_push($this->observers, $Observer);
	}
	
	
	/**
	 * detach observer (object)
	 * @see Observable::detach()
	 */
	public function detach(Observer $Observer) {
		if(in_array($Observer, $this->observers)) {
			$key = array_search($Observer, $this->observers);
			unset($this->observers[$key]);
		}
	}
	
	
	/**
	 * 
	 * notify observer (object)
	 * @see Observable::notify()
	 */
	public function notify($key, $message = '') {
		if(count($this->observers) == 0) {
			return;
		}
		
		foreach ($this->observers as $observer) {
			$observer->update($key, $message, $this);
		}
	}
	
	
	/**
	 * 
	 * For debugging purposes, writes to a file if desired
	 * @param whatever $var
	 * @param boolean $writeToFile
	 * @param string $hint
	 * @param boolean $backtrace
	 */
	public function __($var, $writeToFile = false, $hint = '***', $backtrace = false) {
		if(getenv('sofortDebug') == true) {
			$var = print_r($var, 1);
			if(!$writeToFile) {
				echo '<pre>';
				print_r($var);
				echo '</pre>';
			} else {
				$fp = fopen('/home/dehn/shopware.txt', 'a');
				fwrite($fp, '---------------------'.$hint.'-------------------------'."\n");
				fwrite($fp,  '['.date('Y-m-d H:i:s').'] ' . $var . "\n");
				fclose($fp);
			}
			if($backtrace) {
				echo '<pre>Backtrace<br />';
				print_r(debug_print_backtrace());
				echo '</pre>';
			}
		}
	}
}